/**
 * Copyright (c) 2004, 2008 INRIA.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 *     INRIA - initial API and implementation
 *
 * $Id: ASMModelAdapter.java,v 1.1 2009/04/21 14:11:03 wpiers Exp $
 */
package org.eclipse.gmt.tcs.injector;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;
import java.util.Set;

import org.eclipse.emf.ecore.EEnumLiteral;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.EStructuralFeature;
import org.eclipse.emf.ecore.impl.EcoreFactoryImpl;
import org.eclipse.m2m.atl.core.IModel;
import org.eclipse.m2m.atl.engine.vm.nativelib.AMN;
import org.eclipse.m2m.atl.engine.vm.nativelib.ASMBoolean;
import org.eclipse.m2m.atl.engine.vm.nativelib.ASMCollection;
import org.eclipse.m2m.atl.engine.vm.nativelib.ASMEnumLiteral;
import org.eclipse.m2m.atl.engine.vm.nativelib.ASMInteger;
import org.eclipse.m2m.atl.engine.vm.nativelib.ASMModelElement;
import org.eclipse.m2m.atl.engine.vm.nativelib.ASMOclAny;
import org.eclipse.m2m.atl.engine.vm.nativelib.ASMReal;
import org.eclipse.m2m.atl.engine.vm.nativelib.ASMSequence;
import org.eclipse.m2m.atl.engine.vm.nativelib.ASMString;

/**
 * @author <a href="mailto:frederic.jouault@univ-nantes.fr">Frederic Jouault</a>
 * @author <a href="mailto:mikael.barbero@obeo.fr">Mikael Barbero</a>
 */
public class ASMModelAdapter implements ModelAdapter {
	 
	private IModel model;
	 
	public ASMModelAdapter(Object model) {
		this.model = (IModel)model;		
	}

	public Object getModel() {
		return this.model;
	}
	
	private void actualSet(Object ame_, String propertyName, Object value) {
		if (ame_ instanceof ASMModelElement) {
			ASMModelElement ame = (ASMModelElement)ame_;
			ame.set(null, propertyName, (ASMOclAny)value);

		} else if (ame_ instanceof EObject){ 
			EObject ame = (EObject)ame_;	
			EStructuralFeature sf = ame.eClass().getEStructuralFeature(propertyName);
			if (sf.getUpperBound() != 1){
				if (value instanceof ArrayList) { //handle comments
					String comments = "";
					for (Object element : (ArrayList)value) {
						if (element != null && element instanceof String)
							comments = comments + element;
					}					
					if (comments != null)
						((List)ame.eGet(sf)).add(comments);
				}					
				else
					((List)ame.eGet(sf)).add(value);
			}
			else
				ame.eSet(sf, value);			
		}
	}

	public Object get(Object ame_, String propertyName) {
		Object ret = null;
		
		if (ame_ instanceof ASMModelElement) {
			ASMModelElement ame = (ASMModelElement)ame_;
			ret = ame.get(null, propertyName);		// TODO: adapt get
			if(ret instanceof ASMInteger) {
				ret = new Integer(((ASMInteger)ret).getSymbol());
			} else if(ret instanceof ASMReal) {
				ret = new Double(((ASMReal)ret).getSymbol());
			} else if(ret instanceof ASMBoolean) {
				ret = (((ASMBoolean)ret).getSymbol() ? Boolean.TRUE : Boolean.FALSE);
			} else if(ret instanceof ASMString) {
				ret = ((ASMString)ret).getSymbol();
			} 
		} else if (ame_ instanceof EObject){
			EObject ame = (EObject)ame_;	
			EStructuralFeature sf = ame.eClass().getEStructuralFeature(propertyName);
			ret = ame.eGet(sf);			
		}		
		return ret;
	}

	public Object createElement(String typeName) {
		Object ret = null;
		ret = model.newElement(model.getReferenceModel().getMetaElementByName(typeName));
		return ret;
	}

	public Set getElementsByType(String typeName) {	// TODO: adapt getElementsByType
		Object valueType = model.getReferenceModel().getMetaElementByName(typeName);
		return model.getElementsByType(valueType);
	} 

	public void set(Object ame, String prop, Object value) {
		//ASMOclAny v = java2asm(value);
		//if(v != null)
		actualSet(ame, prop, value);
	}
	
	private ASMOclAny java2asm(Object value) {
		ASMOclAny ret = null;
		
		if(value instanceof ASMModelElement) {
			ret = (ASMModelElement)value;
		} else if(value instanceof ASMEnumLiteral) {
			ret = (ASMEnumLiteral)value;
		} else if(value instanceof ASMCollection) {
			ret = (ASMCollection)value;
		} else if(value instanceof Collection) {
			ASMSequence seq = new ASMSequence();
			for(Iterator i = ((Collection)value).iterator() ; i.hasNext() ; ) {
				seq.add(java2asm(i.next()));
			}
			ret = seq;
		} else if(value instanceof Integer) {
			ret = new ASMInteger(((Integer)value).intValue());
		} else if(value instanceof Double) {
			ret = new ASMReal(((Double)value).doubleValue());
		} else if(value instanceof Boolean) {
			ret = new ASMBoolean(((Boolean)value).booleanValue());
		} else if(value instanceof String) {
			ret = new ASMString((String)value);
		} else if(value == null) {
		} else {
			//model.reportProblem("Error", "unsupported type " + value.getClass(), "?");
		}
		
		return ret;
	}
	
	public boolean isCandidate(Object ame, String typeName) {
		return true;
//		if (ame instanceof EObject) {
//			return ((EObject)ame).eClass().getName().equals(typeName);
//		}
//		return true;
	}

	public Object getType(Object me_) {
		if (me_ instanceof EObject ) {
			return ((EObject)me_).eClass();
		} else
			return ((ASMModelElement)me_).getType();
	}
	
	public boolean isAModelElement(Object me) {
		return (me instanceof ASMModelElement) || (me instanceof EObject);
	}
	
	public String getString(Object ame, String propName) {
		if (ame instanceof ASMModelElement)
			return AMN.getString((ASMModelElement)ame, propName);
		else
			if (ame instanceof EObject) {
				EObject eobj = (EObject)ame;	
				EStructuralFeature sf = eobj.eClass().getEStructuralFeature(propName);
				return (String)eobj.eGet(sf);			
			}
		return null;
	}	
	
	public Object createEnumLiteral(String name) {
//		EMF
//		new EEnumLiteral(name);
		//((EEnumLiteral)me).getName();
		EEnumLiteral enumLiteral = EcoreFactoryImpl.eINSTANCE.createEEnumLiteral();
		enumLiteral.setLiteral(name);
		return enumLiteral;
	}
}