/**
 * Copyright (c) 2004, 2008 INRIA.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 *     INRIA - initial API and implementation
 *
 * $Id: TCSInjector.java,v 1.7 2008/06/25 12:27:04 fjouault Exp $
 */
package org.eclipse.gmt.tcs.injector;

import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.HashMap;
import java.util.Map;

import org.eclipse.m2m.atl.core.ATLCoreException;
import org.eclipse.m2m.atl.core.IInjector;
import org.eclipse.m2m.atl.core.IModel;

/**
 * @author Frdric Jouault
 * @author Mikal Barbero
 */
public class TCSInjector implements IInjector {
	
	private static Map parameterTypes = new HashMap();
	
	static {
		parameterTypes.put("name", "String");			// required
		parameterTypes.put("keepNL", "String");			// optional, default = false		
		parameterTypes.put("keepLocation", "String");	// optional, default = true		
		parameterTypes.put("keepComments", "String");	// optional, default = true		
		parameterTypes.put("tabSize", "4");		// optional, default = 8
		//parameterTypes.put("parserGenerator", "antlr3");// optional, default = "antlr3" 
		parameterTypes.put("hyperlinks", "Map");		// optional, default = null 
		parameterTypes.put("trace", "Map");				// optional, default = null 
		parameterTypes.put("locationByElement", "Map");	// optional, default = null 
		parameterTypes.put("problems", "Model:Problem");// optional, default = null

		// Useful when the lexer and/or parser cannot be resolved from here
		parameterTypes.put("lexerClass", "Class");		// optional, default = null 
		parameterTypes.put("parserClass", "Class");		// optional, default = null 
	}

	public Map getParameterTypes() {
		return parameterTypes;
	}
	
	public String getPrefix() {
		return "ebnf2";
	}
	
	private ModelAdapter problemsModelAdapter;
	private ModelAdapter targetModelAdapter;


	public void performImportation(IModel format, IModel extent, InputStream in, String other) throws IOException {
		throw new UnsupportedOperationException("Was deprecated a long time ago. It is now unsupported");		
	}

	@Override
	public void inject(IModel targetModel, String source)
			throws ATLCoreException {
		inject(targetModel, source, null);
		
	}

	private static String formatFileName(String fileName) {
		if (fileName.startsWith("file:")) { //$NON-NLS-1$
			return fileName.substring(5);
		}
		return fileName;
	}	
	
	@Override
	public void inject(IModel targetModel, String source,
			Map<String, Object> options) throws ATLCoreException {
		try {
			FileInputStream fis = new FileInputStream(formatFileName(source));
			inject(targetModel, fis, options);
			fis.close();
		} catch (IOException e) {
			throw new ATLCoreException(e.getMessage(), e);
		}
		
	}

	@Override
	public void inject(IModel targetModel, InputStream source,
			Map<String, Object> options) throws ATLCoreException {
		this.targetModelAdapter = new ASMModelAdapter(targetModel);
		new ParserLauncher().parse(targetModelAdapter, source, options);
	}
}
