/*******************************************************************************
 * Copyright (c) 2005 INRIA.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Marcos Didonet Del Fabro - initial API and implementation
 *    Guillaume Gueltas
 *******************************************************************************/


package org.eclipse.gmt.weaver.extension.init;

import java.util.Iterator;
import java.util.List;

import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.xmi.XMIResource;
import org.eclipse.emf.edit.command.AbstractOverrideableCommand;
import org.eclipse.emf.edit.command.AddCommand;
import org.eclipse.emf.edit.command.SetCommand;
import org.eclipse.emf.edit.domain.EditingDomain;
import org.eclipse.gmt.weaver.core.WeaverModelManager;
import org.eclipse.gmt.weaver.editors.interfaces.IWeaverInitializer;
import org.eclipse.gmt.weaver.plugin.util.WeaverInterfaceUtil;
import org.eclipse.gmt.weaver.plugin.util.WeaverUtil;
import org.eclipse.gmt.weaver.providers.IIdentifierAdapter;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.SelectionChangedEvent;


/**
 * Initialization of the weaver
 * @since 1.0
 */
public class WeaverInitialize implements IWeaverInitializer, ISelectionChangedListener {
        
    private WeaverModelManager modelManager; 
    private String wModelName;
    

    /* (non-Javadoc)
     * @see org.eclipse.gmt.weaver.editors.IWeaverInitializer#initWeaverMetamodel(org.eclipse.gmt.weaver.core.WeaverModelManager, org.eclipse.emf.edit.domain.EditingDomain, java.lang.String, java.util.List, boolean)
     */
    public void initWeaverMetamodel(WeaverModelManager manager, EditingDomain editingDomain, 
    		String wModelName, List wModelRefClassNames, List wReferenceNames, 
    		List wModelRef_name, List wModelRef_ref, boolean isNew){
        this.modelManager = manager;
        this.wModelName = wModelName;
        // Create instance of WModel if new
        if (isNew)
           manager.helper.createWeavingElementInRes(wModelName,editingDomain);           
        // Create instance(s) of WModelRef if not exist                  
        setWovenModelResources(wModelRefClassNames,wReferenceNames, wModelRef_name, wModelRef_ref, 
        		editingDomain, isNew);        	
   }
    
    /* (non-Javadoc)
     * @see org.eclipse.jface.viewers.ISelectionChangedListener#selectionChanged(org.eclipse.jface.viewers.SelectionChangedEvent)
     */
    public void selectionChanged(SelectionChangedEvent event){   
    }  
    
    /**
     * Creates a new WModelRef in case there is no element poiting to the resource passed as parameter
     * @param uri
     * @param name the name of the WModelRef subtype
     * @param editingDomain the editing domain
     */    
    public void setModelRef(String className, String refName, String uri, String wName, EditingDomain editingDomain){
        if (this.getModelRef(className,"ref",uri, refName) == null){
        	try{
	            EObject eModelRef = modelManager.helper.createWeavingElement_32(className,editingDomain);
	            EObject wModel = this.getModel();
	            AbstractOverrideableCommand addModelRef = null;
	            if (wModel.eClass().getEStructuralFeature(refName).isMany()){
	            	 addModelRef = (AddCommand)AddCommand.create(editingDomain,wModel,
	    	            	wModel.eClass().getEStructuralFeature(refName),eModelRef);
	            }
	            else{
		             addModelRef = new SetCommand(editingDomain,wModel,
    	            		wModel.eClass().getEStructuralFeature(refName),eModelRef);	            	
	            }	            	            
	            
	            addModelRef.execute();
	            //adds an adapter for id creation into the weaving elements
	            IIdentifierAdapter adapter = WeaverInterfaceUtil.createGetIdentifierAdapter(eModelRef);	   
	            //set ref property
	            SetCommand setRef = new SetCommand(editingDomain,eModelRef,eModelRef.eClass().getEStructuralFeature("ref"),uri);
	            setRef.execute();
	            //set name property
	            setRef = new SetCommand(editingDomain,eModelRef,eModelRef.eClass().getEStructuralFeature("name"),wName);
	            setRef.execute();	  
	            
	            setModelRefId(adapter, eModelRef);
        	}catch(Exception e){
        		// Nothing is do because this exception (in getWeavingMetamodel) is already threw before 
        		// we arrived to this method
        	}
        }               
    }    
    /**
     * set a new value for the reference properties for all existing model refs
     * @param uri
     * @param className the name of the WModelRef subtype
     * @param editingDomain the editing domain
     */    
    public void setExistingModelRef(String className, String wRef_name, String refName, String uri, EditingDomain editingDomain){    	
    	EObject eModelRef = getModelRef(className,"name",wRef_name, refName);     								    
        if (eModelRef != null){
        	try{	            
	            //adds an adapter for id creation into the weaving elements
	            IIdentifierAdapter adapter = WeaverInterfaceUtil.createGetIdentifierAdapter(eModelRef);	            
	            
	            //sets the ref property
	            SetCommand setRef = new SetCommand(editingDomain,eModelRef,eModelRef.eClass().getEStructuralFeature("ref"),uri);
	            setRef.execute();
	            //sets the name property
	            setRef = new SetCommand(editingDomain,eModelRef,eModelRef.eClass().getEStructuralFeature("name"),wRef_name);
	            setRef.execute();
	            
	            setModelRefId(adapter, eModelRef);	            
	            
        	}catch(Exception e){
        		// Nothing is do because this exception (in getWeavingMetamodel) is already threw before 
        		// we arrived to this method
        	}
        }               
    }
    
    /**
     * generates wmodelref ids (method replicated in welementitemprovider)
     * @param adapter
     * @param eModelRef
     */
    private void setModelRefId(IIdentifierAdapter adapter, EObject eModelRef){
		if (adapter.getID() == null){//sets the id as the class name + a date in milliseconds + number of objects
			int a = 0;    				    	
			XMIResource res = (XMIResource) eModelRef.eResource();
			String refClassName = eModelRef.eClass().getName();
			do {
				a ++;
			} while (res.getEObject(refClassName + a)!= null);    				    				
			adapter.setID(refClassName + a);
		}
    }
    /**
     * Returns the unique WModel element in the weaving model
     * @return
     */
    public EObject getModel(){
		for (Iterator i= this.modelManager.getWeavingModelRS().getAllContents() ;i.hasNext();){
		    Object obj = i.next();	    		    
		    if (obj instanceof EObject && ((EObject)obj).eClass().getName().equals(wModelName) )
		        return (EObject)obj;
		}
		return null;
    }
        
    /**
     * Creates a new (if not exists) WModelRef element for each resource
     * @param wModelRefClassNames the list of subtype of WModelRef in order 
     * @param modelResources
     */
    public void setWovenModelResources(List wModelRefClassNames, List wReferenceNames, 
    		List wModelRef_name, List wModelRef_ref, EditingDomain editingDomain, boolean isNew){        
        for(int i = 0; i<wModelRefClassNames.size();i++){
        	
        	String wModelRefClassName = (String) wModelRefClassNames.get(i);
        	String wReferenceName = (String) wReferenceNames.get(i);
        	String wNameProperty = (String) wModelRef_name.get(i);
        	String wRef = (String) wModelRef_ref.get(i);
        	if (isNew)
        		this.setModelRef(wModelRefClassName,wReferenceName,wRef, wNameProperty, editingDomain);
        	else
        		setExistingModelRef(wModelRefClassName,wNameProperty,wReferenceName,wRef,editingDomain);
        }
    }    
        
    
    /**
     * Returns the WModelRef that points to the resource passed in the property name
     * @param className the name of WModelRef subtype
     * @param propertyName
     * @param propertyValue
     * @return
     */
    public EObject getModelRef(String className,String propertyName, String propertyValue, String refName){
        Resource res = this.modelManager.getWeavingModel();
        //loops over all elements of the resource
        for (Iterator i = res.getAllContents(); i.hasNext(); ){
            Object obj = i.next();
            if (obj instanceof EObject){
                EObject eObj = (EObject)obj;
                Resource mm=null;
                try{
                	mm = this.modelManager.getWeavingMetamodel();
                } catch(Exception e) {
                	
                }
                //checks if the object is an instance of the given class name
                if (WeaverUtil.isInstanceOf(eObj, className,mm)) {
                	//gets the value of the given property 
                    String value =  (String)eObj.eGet( eObj.eClass(). getEStructuralFeature(propertyName));                    
                    if (value != null)
                        if (value.equals(propertyValue)){
                        	if (eObj.eContainingFeature().getName().equals(refName))                        	
                        		return eObj;                        	
                        }
                                                            
                }                
            }
        }
        return null;        
    }  
    
	/**
	 * @return Returns the modelManager.
	 */
	public WeaverModelManager getModelManager() {
		return modelManager;
	}
	/**
	 * @param modelManager The modelManager to set.
	 */
	public void setModelManager(WeaverModelManager modelManager) {
		this.modelManager = modelManager;
	}
	/**
	 * @return Returns the wModelName.
	 */
	public String getWModelName() {
		return wModelName;
	}
	/**
	 * @param modelName The wModelName to set.
	 */
	public void setWModelName(String modelName) {
		wModelName = modelName;
	}
    
}
