/*******************************************************************************
 * Copyright (c) 2005 INRIA.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Marcos Didonet Del Fabro - initial API and implementation
 *    Guillaume Gueltas
 *******************************************************************************/


package org.eclipse.gmt.weaver.extension.panel;

import java.io.File;
import java.io.IOException;
import java.net.URL;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;

import org.eclipse.core.runtime.IPath;
import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.EPackage;
import org.eclipse.emf.ecore.EStructuralFeature;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.resource.ResourceSet;
import org.eclipse.emf.ecore.util.EcoreUtil;
import org.eclipse.emf.ecore.xmi.XMIResource;
import org.eclipse.emf.edit.domain.EditingDomain;
import org.eclipse.emf.edit.provider.ComposedAdapterFactory;
import org.eclipse.emf.edit.provider.resource.ResourceItemProviderAdapterFactory;
import org.eclipse.emf.edit.ui.dnd.LocalTransfer;
import org.eclipse.emf.edit.ui.dnd.ViewerDragAdapter;
import org.eclipse.emf.edit.ui.provider.AdapterFactoryContentProvider;
import org.eclipse.emf.edit.ui.provider.AdapterFactoryLabelProvider;
import org.eclipse.gmt.weaver.core.WeaverModelManager;
import org.eclipse.gmt.weaver.core.WeaverXMLMetadata;
import org.eclipse.gmt.weaver.core.data.WovenModelDescriptor;
import org.eclipse.gmt.weaver.editors.interfaces.IWovenModelPanel;
import org.eclipse.gmt.weaver.editors.weaving.SelectionTreeViewer;
import org.eclipse.gmt.weaver.extension.panel.dnd.WovenPanelDndManager;
import org.eclipse.gmt.weaver.extension.plugin.WeaverBaseExtensionPlugin;
import org.eclipse.gmt.weaver.extension.providers.MergeIdentifierAdapter;
import org.eclipse.gmt.weaver.extension.providers.WovenModelsAdapterFactory;
import org.eclipse.gmt.weaver.plugin.util.WeaverUtil;
import org.eclipse.gmt.weaver.providers.IIdentifierAdapter;
import org.eclipse.gmt.weaver.providers.ResourceIdentifierAdapter;
import org.eclipse.jface.dialogs.InputDialog;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.jface.viewers.ContentViewer;
import org.eclipse.jface.viewers.IBaseLabelProvider;
import org.eclipse.jface.viewers.ILabelProvider;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.CLabel;
import org.eclipse.swt.custom.SashForm;
import org.eclipse.swt.custom.ViewForm;
import org.eclipse.swt.dnd.DND;
import org.eclipse.swt.dnd.Transfer;
import org.eclipse.swt.events.DisposeEvent;
import org.eclipse.swt.events.DisposeListener;
import org.eclipse.swt.events.MouseAdapter;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.events.MouseListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.ToolBar;
import org.eclipse.swt.widgets.ToolItem;
import org.eclipse.ui.dialogs.SaveAsDialog;

/**
 * Default woven panel extension
 * @since 1.0
 */
public class MergeWovenModelPanel implements IWovenModelPanel {
	
	private List selectionListeners = new ArrayList();
    protected Viewer viewer;
    protected Composite container;
    protected CLabel titleLabel;
    protected ViewForm viewForm;

    protected ComposedAdapterFactory adapterFactory;
    private WovenPanelDndManager dndManager;

    protected MouseListener mouseListener = new MouseAdapter() {
        public void mouseDown(MouseEvent e) {
        }

        public void mouseDoubleClick(MouseEvent e) {
            if (e.getSource() == titleLabel) {
                doMaximize();
            }
        }
    };

	private StructuredSelection selection;
	private ToolBar actionBar;
	private Image expandAllImage;
	
	private boolean canModify = true;

    /**
     * Constructs a view panel for a view part.
     */
    public MergeWovenModelPanel() {
    }

    /* (non-Javadoc)
     * @see org.eclipse.gmt.weaver.editors.IWovenModelPanel#createViewer(org.eclipse.swt.widgets.Composite)
     */
    public Viewer createViewer(Composite parent) {
        // viewer is a TreeViewer    	
        TreeViewer modelViewer = new SelectionTreeViewer(parent, SWT.MULTI){
        	public void addSelectionChangedListener (ISelectionChangedListener listener){
        		super.addSelectionChangedListener(listener);
        		if (canModify)
        		  selectionListeners.add(listener);
        		
        	}
        	public void removeSelectionChangedListener(ISelectionChangedListener listener){
        		super.removeSelectionChangedListener(listener);
        		//selectionListeners.remove(listener);
        	}
        };
        
        // create adapterfactory
        List factories = new ArrayList();
        factories.add(new ResourceItemProviderAdapterFactory());
        factories.add( new WovenModelsAdapterFactory()); 
        adapterFactory = new ComposedAdapterFactory(factories);

        // adds provider
        modelViewer.setContentProvider(new AdapterFactoryContentProvider(
                adapterFactory));
        modelViewer.setLabelProvider(new AdapterFactoryLabelProvider(
                adapterFactory));

        // adds drag and drop
        
        int dndOperations = DND.DROP_LINK;
        Transfer[] transfers = new Transfer[] { LocalTransfer.getInstance() };
        modelViewer.addDragSupport(dndOperations, transfers,
                new ViewerDragAdapter(modelViewer));
        //we need to pass the viewer because it was not yet associated with the panel
        dndManager = new WovenPanelDndManager(this,modelViewer); 
		modelViewer.addDropSupport(DND.DROP_LINK, transfers,dndManager);
				
        return modelViewer;
    }
   

    /* (non-Javadoc)
     * @see org.eclipse.gmt.weaver.editors.IWovenModelPanel#createControl(org.eclipse.swt.widgets.Composite)
     */
    public void createControl(Composite parent) {
        if (getControl() == null) {
            container = parent;

            // Create view form.
            viewForm = new ViewForm(parent, SWT.NONE);
            viewForm.addDisposeListener(new DisposeListener() {
                public void widgetDisposed(DisposeEvent event) {
                    dispose();
                }
            });
            viewForm.marginWidth = 0;
            viewForm.marginHeight = 0;

            viewer = createViewer(viewForm);
            viewForm.setContent(viewer.getControl());

            // Create a title bar.
            createTitleBar();
            
            viewForm.setTabList(new Control[] { viewer.getControl() });
            //getCreatePropertySheetPage();
        }
    }
    private void setExpandAllButton (){
		ToolItem ti = new ToolItem(actionBar, SWT.PUSH, 0);
        expandAllImage = ImageDescriptor.createFromURL((URL)WeaverBaseExtensionPlugin.getPlugin().getImage("icons/expandAll.gif")).createImage();
		ti.setImage(expandAllImage);
        ti.setSelection(false);
        ti.setToolTipText(WeaverBaseExtensionPlugin.getPlugin().getString("_UI_panel_action_expand"));
        ti.addSelectionListener(new SelectionAdapter(){
             public void widgetSelected(SelectionEvent e){
             	if(viewer != null){
             		((TreeViewer)viewer).expandAll();             		
             	}
             }
        });
    	
    }
    private void setCountElementButton(){
        //TODO add internatioanization strings
    	ToolItem ti = new ToolItem(actionBar, SWT.PUSH, 0);
        expandAllImage = ImageDescriptor.createFromURL((URL)WeaverBaseExtensionPlugin.getPlugin().getImage("icons/count.gif")).createImage();
		ti.setImage(expandAllImage);
        ti.setSelection(false);
        ti.setToolTipText(WeaverBaseExtensionPlugin.getPlugin().getString("_UI_panel_action_count"));        
        ti.addSelectionListener(new SelectionAdapter(){
             public void widgetSelected(SelectionEvent e){
             	if(viewer != null){            		
             		MessageDialog msgDialog = new MessageDialog(viewer.getControl().getShell(),
             				WeaverBaseExtensionPlugin.getPlugin().getString("_UI_element_count_title"),null,
             				WeaverBaseExtensionPlugin.getPlugin().getString("_UI_element_count_text") + 
             				WeaverUtil.countElements((Resource)getViewer().getInput()),
             				0,new String[]{"OK"},0);
             		msgDialog.open();
             	}
             }
        });          	
    }
    private void setSearchElementButton (){
        //TODO add internatioanization strings
    	ToolItem ti = new ToolItem(actionBar, SWT.PUSH, 0);
        expandAllImage = ImageDescriptor.createFromURL((URL)WeaverBaseExtensionPlugin.getPlugin().getImage("icons/search.gif")).createImage();
		ti.setImage(expandAllImage);
        ti.setSelection(false);
        ti.setToolTipText(WeaverBaseExtensionPlugin.getPlugin().getString("_UI_panel_action_search_element"));
        ti.addSelectionListener(new SelectionAdapter(){
        	public void widgetSelected(SelectionEvent e){
        		if(viewer != null){            	             		
        			InputDialog msgDialog = new InputDialog(viewer.getControl().getShell(),
        					WeaverBaseExtensionPlugin.getPlugin().getString("_UI_search_element_title"),
        					WeaverBaseExtensionPlugin.getPlugin().getString("_UI_search_element_caption"),"",null);             		
        			if (msgDialog.open() == org.eclipse.jface.window.Window.OK){
        				boolean found = false;
        				for (Iterator i = WeaverUtil.getAllPackages((Resource)getViewer().getInput()).iterator(); i.hasNext();){             	    		
        					EPackage pkg = (EPackage) i.next();
        					for (Iterator pkgs = pkg.eAllContents(); pkgs.hasNext();){
        						Object obj = pkgs.next();
        						if (obj instanceof EObject){             	    				
        							EStructuralFeature feature = ((EObject)obj).eClass().getEStructuralFeature("name");
        							if (feature != null){//search the Name prop             	    					
        								String name =(String)((EObject)obj).eGet(feature);		             	    				
        								found = name.toLowerCase().indexOf(msgDialog.getValue().toLowerCase()) >= 0;
        								if (found){
        									getViewer().setSelection(new StructuredSelection(obj));
        									break; 
        								}
        							}
        						}
        					}
        				}
        				if (!found) {
		             		MessageDialog aDialog = new MessageDialog(viewer.getControl().getShell(),
		             				WeaverBaseExtensionPlugin.getPlugin().getString("_UI_element_not_found"),null,
		             				WeaverBaseExtensionPlugin.getPlugin().getString("_UI_element_not_found"),
		             				0,new String[]{"OK"},0);
		             		aDialog.open();						        									
						}
        			}
        		}
        	}
        });               
        
    }
    /**
     * Creates a title bar for the pane which includes the view icon and title to
     * the far left
     */
    protected void createTitleBar() {
        // Only do this once.
        if (titleLabel == null) {
            // Title.
            titleLabel = new CLabel(viewForm, SWT.SHADOW_NONE);
            titleLabel.setAlignment(SWT.LEFT);
            titleLabel.setBackground(null, null);
            updateTitles();
            viewForm.setTopLeft(titleLabel);
            titleLabel.addMouseListener(mouseListener);
        }
        actionBar = new ToolBar(viewForm, SWT.FLAT | SWT.WRAP);
        /*expand all*/
        setExpandAllButton();
        /* count the number of elements*/
        setCountElementButton();
        /* selects an element*/
        setSearchElementButton();
        viewForm.setTopCenter(actionBar);
    }

    /**
     * Maximizes the panel
     */
    protected void doMaximize() {
        Control child = viewForm;
        for (Control parent = viewForm.getParent(); parent instanceof SashForm; parent = parent
                .getParent()) {
            SashForm sashForm = (SashForm) parent;
            if (sashForm.getMaximizedControl() == null) {
                sashForm.setMaximizedControl(child);
            } else {
                sashForm.setMaximizedControl(null);
            }
            child = parent;
        }
    }

    /* (non-Javadoc)
     * @see org.eclipse.gmt.weaver.editors.IWovenModelPanel#dispose()
     */
    public void dispose() {
        if ((viewForm != null) && (!viewForm.isDisposed())) {
            viewForm = null;
        }
        if (adapterFactory != null) {
            adapterFactory.dispose();
        }
        if(expandAllImage!= null){
        	expandAllImage.dispose();
        }
        if(actionBar != null){
        	actionBar.dispose();
        }
        	
    }

    /* (non-Javadoc)
     * @see java.lang.Object#toString()
     */
    public String toString() {    	
        String label = "disposed";
        if ((titleLabel != null) && (!titleLabel.isDisposed()))
            label = titleLabel.getText();

        return getClass().getName() + "@" + Integer.toHexString(hashCode())
                + "(" + label + ")";
    }

    /**
     * Updates the title attributes.
     */
    public void updateTitles() {
        titleLabel.update();
    }

    /* (non-Javadoc)
     * @see org.eclipse.gmt.weaver.editors.IWovenModelPanel#setTitle(java.lang.Object)
     */
    public void setTitle(Object object) {
        if (viewer != null) {
            if (viewer instanceof ContentViewer) {
                IBaseLabelProvider labelProvider = ((ContentViewer) viewer)
                        .getLabelProvider();
                if (labelProvider instanceof ILabelProvider) {
                    if (object == null) {
                        titleLabel.setImage(null);
                        titleLabel.setText("");
                    } else {
                        titleLabel.setImage(((ILabelProvider) labelProvider)
                                .getImage(object));
                        titleLabel.setText(((ILabelProvider) labelProvider)
                                .getText(object));
                    }
                }
            }
        }
    }


    /* (non-Javadoc)
     * @see org.eclipse.gmt.weaver.editors.IWovenModelPanel#setTitle(java.lang.String, org.eclipse.swt.graphics.Image)
     */
    public void setTitle(String title, Image image) {
        if (titleLabel != null) {
            titleLabel.setImage(image);
            titleLabel.setText(title);
        }
    }

    /* (non-Javadoc)
     * @see org.eclipse.gmt.weaver.editors.IWovenModelPanel#setInput(org.eclipse.emf.ecore.resource.ResourceSet)
     */
    public void setInput(Object input) {    	
    	viewer.setInput(input);
        viewer.setInput(input);
        ((TreeViewer) viewer).expandToLevel(3);      
    }


    /* (non-Javadoc)
     * @see org.eclipse.gmt.weaver.editors.IWovenModelPanel#getViewer()
     */
    public Viewer getViewer() {
        return viewer;
    }

    /**
     * Gets the ViewForm
     * @return
     */
    public Control getControl() {
        return viewForm;
    }

    /**
     * Gets the view form 
     * @return
     */
    protected ViewForm getViewForm() {
        return viewForm;
    }

    // For populate combo box in properties
    /**
     * Returns a collection with all the EMF objects instantiated in this viewer
     */
    private Collection getEObjects(Resource res) {
    	//ResourceSet resourceSet = (ResourceSet) (ResourceSet) getViewer().getInput();
    	Collection eObjects = new ArrayList();
    	for (Iterator elements = res.getAllContents(); elements.hasNext();) {
    		Object obj = elements.next();
    		eObjects.add(obj);
    	}
    	return eObjects;
    }

    /* (non-Javadoc)
     * @see org.eclipse.gmt.weaver.editors.IWSelectionChangedListener#selectEObject(java.util.List)
     */
    public void selectEObject(List objectIDs, List selectedModelsURI, EObject currentWeaverSelection){
    	getViewer().setSelection(null); //set current selection to null
    	if(dndManager != null)     		
    		dndManager.setWeaverSelection(currentWeaverSelection);
        List listOfSelectedObjects = new ArrayList();
        Iterator models = selectedModelsURI.iterator();
        for (Iterator IDs = objectIDs.iterator(); IDs.hasNext();){
        	String aID = (String) IDs.next();
        	String aURI = (String) models.next();
        	//Resource res = (Resource)((ResourceSet)viewer.getInput()).getResources().get(0);
        	Resource res = null;
        	Object input = viewer.getInput();
        	if (input instanceof ResourceSet)
        		res = (Resource)((ResourceSet)viewer.getInput()).getResources().get(0);
        	else
        		if (input instanceof Resource)
        			res = (Resource)viewer.getInput();        	
        	IIdentifierAdapter resAdapter = WeaverUtil.getIdentifierAdapter(res);
        	if (resAdapter.getID().toString().equals(aURI))
        		for (Iterator i = getEObjects(res).iterator(); i.hasNext();) {
        			Object obj = i.next();        		
        			if (obj instanceof EObject){
        				EObject eObj = (EObject) obj;
        				//checks if the current object ID matchs
        				IIdentifierAdapter adapter = WeaverUtil.getIdentifierAdapter(eObj);
        				//Resource res = (Resource)((ResourceSet)viewer.getInput()).getResources().get(0);
        				//if (eObj.eResource().getURI().equals(res.getURI()) )        				
        				if (adapter == null){ // if there is no adapter, sets as its URI
        					if (EcoreUtil.getURI(eObj).fragment().equals(aID)) {
        						listOfSelectedObjects.add(eObj);
        					}                	
        				}
        				else{//checks the id
        					if (adapter.getID().equals(aID))
        						listOfSelectedObjects.add(eObj);                		                		
        				}        				        				
        			}        		
        		}            
        }
        if (listOfSelectedObjects.size() > 0) {
        	selection = new StructuredSelection(listOfSelectedObjects);
        	
        	removeSelectionListeners();        	
			getViewer().setSelection(selection, true);
			addSelectionListeners();
        }        
    }
    private void removeSelectionListeners(){
    	for (Iterator i = selectionListeners.iterator(); i.hasNext();){
    		Object obj = i.next();
    		if (obj instanceof ISelectionChangedListener)
    		  ((TreeViewer)getViewer()).removeSelectionChangedListener((ISelectionChangedListener) obj);
    	}    	
    }
    private void addSelectionListeners(){
    	canModify = false;
    	for (Iterator i = selectionListeners.iterator(); i.hasNext();){
    		((TreeViewer)getViewer()).addSelectionChangedListener((ISelectionChangedListener) i.next());
    	}    	
    	canModify = true;
    	
    }

	/* (non-Javadoc)
	 * @see org.eclipse.gmt.weaver.editors.IWovenModelPanel#getAdapterFactory()
	 */
	public ComposedAdapterFactory getAdapterFactory() {
		return adapterFactory;
	}

	/**
	 * adds an identifier adapter for every object in a resource,
	 * generates and ID for it,
	 * copy new resource in a another location if user want
	 * @param resource
	 * @param xmlFile
	 */
	protected Resource addIdentifierAdapters(Resource resource, WeaverXMLMetadata xmlFile, String modelID){
		if (resource instanceof XMIResource){
			XMIResource xmiRes = (XMIResource)resource;
			boolean flag = false;
			int a=0;
			//adds an identifier adapter for the resource as well
			
			xmiRes.eAdapters().add(new ResourceIdentifierAdapter(xmiRes));
			for (Iterator elements = resource.getAllContents();elements.hasNext(); ){
				Object obj = elements.next();
				if (obj instanceof EObject){
					//the adapter of the woven objects can be changed here
					//IIdentifierAdapter adapter = new MergeIdentifierAdapter((EObject)obj);
					IIdentifierAdapter adapter = new MergeIdentifierAdapter((EObject)obj);
					((EObject)obj).eAdapters().add(adapter);
					if (adapter.getID() == null){
						String className = (String)((EObject)obj).eClass().getName(); 
						do {//prevents from having the same ID 
	    					a ++;
	    				} while (xmiRes.getEObject(className + a)!= null);    				    				
	    				adapter.setID(className + a);		
	    				flag = true;
					}
				}
			}		
			try {
				if(flag){
					String filename = xmiRes.getURI().path();
					boolean response = MessageDialog.openQuestion(
							this.getControl().getShell(),
							WeaverBaseExtensionPlugin.getPlugin().getString("_UI_woven_panel_question_title"),
							WeaverBaseExtensionPlugin.getPlugin().getString("_UI_woven_panel_question_message",titleLabel.getText(),filename));
					if(!response){
						SaveAsDialog saveAsDialog= new SaveAsDialog(this.getControl().getShell());		
						saveAsDialog.open();
						IPath path= saveAsDialog.getResult();
						if (path != null) {
							URI uri = URI.createPlatformResourceURI(path.toString());
							xmiRes.setURI(uri);
							List modelFile = xmlFile.getWovenModels();
							List tempList = new ArrayList();
							for(int i=0;i<modelFile.size();i++){
								WovenModelDescriptor desc = (WovenModelDescriptor)modelFile.get(i);
								if(desc.getReferenceName().equals(this.getTitleLabel().getText())){
									if(desc.getModelFile().equals("null"))
										desc.setMetamodelFile(path.toString());
									else
										desc.setModelFile(path.toString());
								}
								tempList.add(desc);
							}
							try {
								// Config file
								File propertiesFile = xmlFile.getFile();
								WeaverXMLMetadata xmlFileTemp = new WeaverXMLMetadata(propertiesFile);
								xmlFileTemp.createXMLConfigFile(tempList,
										xmlFile.getWeavingModelPR(),
										xmlFile.getWeavingModelPanel(),
										xmlFile.getWModelName(),
										xmlFile.getMetamodelExtension(),
										xmlFile.getLocalMetamodelExtension(),
										xmlFile.getWeavingEcore());
							} catch (Exception e1) {
								e1.printStackTrace();
								MessageDialog.openWarning(
										this.getControl().getShell(),
										WeaverBaseExtensionPlugin.getPlugin().getString("_UI_woven_panel_warning_title"),
										WeaverBaseExtensionPlugin.getPlugin().getString("_UI_woven_panel_warning_message",e1.getMessage()));
							}
						}	
					}
					xmiRes.save(null);
					return xmiRes;
				}
					
			} catch (IOException e) {
				e.printStackTrace();
				MessageDialog.openWarning(
						this.getControl().getShell(),
						WeaverBaseExtensionPlugin.getPlugin().getString("_UI_woven_panel_warning_title"),
						WeaverBaseExtensionPlugin.getPlugin().getString("_UI_woven_panel_warning_message",e.getMessage()));
			}			
		}
		return resource;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.gmt.weaver.editors.IWovenModelPanel#initializeModelManager(org.eclipse.gmt.weaver.core.WeaverModelManager)
	 */
	public void initializeModelManager(WeaverModelManager manager) {
		dndManager.setWManager(manager);	
		
	}

	/* (non-Javadoc)
	 * @see org.eclipse.gmt.weaver.editors.IWovenModelPanel#initializeEditingDomain(org.eclipse.emf.edit.domain.EditingDomain)
	 */
	public void initializeEditingDomain(EditingDomain editingDomain) {
		dndManager.setWEditingDomain(editingDomain);
		
	}

	/* (non-Javadoc)
	 * @see org.eclipse.gmt.weaver.editors.IWovenModelPanel#initializeModelResource(org.eclipse.emf.ecore.resource.ResourceSet, org.eclipse.gmt.weaver.core.WeaverXMLMetadata)
	 */
	public Resource initializeModelResource(Resource input, WeaverXMLMetadata xmlFile, String modelID) {
		return addIdentifierAdapters(input, xmlFile, modelID);  
	}	
		
	/**
	 * @return Returns the dndManager.
	 */
	public WovenPanelDndManager getDndManager() {
		return dndManager;
	}
	/**
	 * @param dndManager The dndManager to set.
	 */
	public void setDndManager(WovenPanelDndManager dndManager) {
		this.dndManager = dndManager;
	}
	/**
	 * @return Returns the selection.
	 */
	public StructuredSelection getSelection() {
		return selection;
	}
	/**
	 * @param selection The selection to set.
	 */
	public void setSelection(StructuredSelection selection) {
		this.selection = selection;
	}
	/**
	 * @param adapterFactory The adapterFactory to set.
	 */
	public void setAdapterFactory(ComposedAdapterFactory adapterFactory) {
		this.adapterFactory = adapterFactory;
	}
	
	/**
	 * @return Returns the titleLabel.
	 */
	public CLabel getTitleLabel() {
		return titleLabel;
	}
	/**
	 * @param titleLabel The titleLabel to set.
	 */
	public void setTitleLabel(CLabel titleLabel) {
		this.titleLabel = titleLabel;
	}
	

}	
	
	


