/*******************************************************************************
 * Copyright (c) 2005 INRIA.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Marcos Didonet Del Fabro - initial API and implementation
 *******************************************************************************/


package org.eclipse.gmt.weaver.extension.panel;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.eclipse.core.runtime.IPath;
import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.xmi.XMIResource;
import org.eclipse.gmt.weaver.core.WeaverXMLMetadata;
import org.eclipse.gmt.weaver.core.data.WovenModelDescriptor;
import org.eclipse.gmt.weaver.extension.plugin.WeaverBaseExtensionPlugin;
import org.eclipse.gmt.weaver.providers.IIdentifierAdapter;
import org.eclipse.gmt.weaver.providers.ResourceIdentifierAdapter;
import org.eclipse.gmt.weaver.providers.XMIIdentifierAdapter;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.ui.dialogs.SaveAsDialog;

/**
 * Woven panel extension to support XMI identifiers
 * 
 * @since 2.0
 */
public class WovenPanelXMI extends DefaultWovenModelPanel {
	
	
    /**
     * Constructs a view panel for a view part.
     */
    public WovenPanelXMI() {
    }


   	/**
	 * adds an identifier adapter for every object in a resource,
	 * generates and ID for it,
	 * copy new resource in a another location if user want
	 * @param resource
	 * @param xmlFile
	 */
	protected Resource addIdentifierAdapters(Resource resource, WeaverXMLMetadata xmlFile, String modelID){
		if (resource instanceof XMIResource){
			XMIResource xmiRes = (XMIResource)resource;
			boolean flag = false;
			int a=0;
			//adds an identifier adapter for the resource as well
			
			xmiRes.eAdapters().add(new ResourceIdentifierAdapter(xmiRes));
			for (Iterator elements = resource.getAllContents();elements.hasNext(); ){
				Object obj = elements.next();
				if (obj instanceof EObject){
					IIdentifierAdapter adapter = new XMIIdentifierAdapter((EObject)obj);
					((EObject)obj).eAdapters().add(adapter);
					if (adapter.getID() == null){
						String className = (String)((EObject)obj).eClass().getName(); 
						do {//prevents from having the same ID 
	    					a ++;
	    				} while (xmiRes.getEObject(className + a)!= null);    				    				
	    				adapter.setID(className + a);		
	    				flag = true;
					}
				}
			}		
			try {
				if(flag){
					String filename = xmiRes.getURI().path();
					boolean response = MessageDialog.openQuestion(
							this.getControl().getShell(),
							WeaverBaseExtensionPlugin.getPlugin().getString("_UI_woven_panel_question_title"),
							WeaverBaseExtensionPlugin.getPlugin().getString("_UI_woven_panel_question_message",titleLabel.getText(),filename));
					if(!response){
						SaveAsDialog saveAsDialog= new SaveAsDialog(this.getControl().getShell());		
						saveAsDialog.open();
						IPath path= saveAsDialog.getResult();
						if (path != null) {
							URI uri = URI.createPlatformResourceURI(path.toString());
							xmiRes.setURI(uri);
							List modelFile = xmlFile.getWovenModels();
							List tempList = new ArrayList();
							for(int i=0;i<modelFile.size();i++){
								WovenModelDescriptor desc = (WovenModelDescriptor)modelFile.get(i);
								if(desc.getReferenceName().equals(this.getTitleLabel().getText())){
									if(desc.getModelFile().equals("null"))
										desc.setMetamodelFile(path.toString());
									else
										desc.setModelFile(path.toString());
								}
								tempList.add(desc);
							}
							try {
								// Config file
								File propertiesFile = xmlFile.getFile();
								WeaverXMLMetadata xmlFileTemp = new WeaverXMLMetadata(propertiesFile);
								xmlFileTemp.createXMLConfigFile(tempList,
										xmlFile.getWeavingModelPR(),
										xmlFile.getWeavingModelPanel(),
										xmlFile.getWModelName(),
										xmlFile.getMetamodelExtension(),
										xmlFile.getLocalMetamodelExtension(),
										xmlFile.getWeavingEcore());
							} catch (Exception e1) {
								e1.printStackTrace();
								MessageDialog.openWarning(
										this.getControl().getShell(),
										WeaverBaseExtensionPlugin.getPlugin().getString("_UI_woven_panel_warning_title"),
										WeaverBaseExtensionPlugin.getPlugin().getString("_UI_woven_panel_warning_message",e1.getMessage()));
							}
						}	
					}
					xmiRes.save(null);
					return xmiRes;
				}
					
			} catch (IOException e) {
				e.printStackTrace();
				MessageDialog.openWarning(
						this.getControl().getShell(),
						WeaverBaseExtensionPlugin.getPlugin().getString("_UI_woven_panel_warning_title"),
						WeaverBaseExtensionPlugin.getPlugin().getString("_UI_woven_panel_warning_message",e.getMessage()));
			}			
		}
		return resource;
	}

}	
	
	


