/*******************************************************************************
 * Copyright (c) 2005 INRIA.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Marcos Didonet Del Fabro - initial API and implementation
 *    Guillaume Gueltas
 *******************************************************************************/


package org.eclipse.gmt.weaver.extension.panel.dnd;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;

import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.EReference;
import org.eclipse.emf.edit.command.CommandParameter;
import org.eclipse.emf.edit.domain.AdapterFactoryEditingDomain;
import org.eclipse.emf.edit.domain.EditingDomain;
import org.eclipse.emf.edit.provider.IEditingDomainItemProvider;
import org.eclipse.emf.edit.ui.dnd.EditingDomainViewerDropAdapter;
import org.eclipse.gmt.weaver.core.WeaverModelManager;
import org.eclipse.gmt.weaver.editors.interfaces.IWovenModelPanel;
import org.eclipse.gmt.weaver.extension.plugin.WeaverBaseExtensionPlugin;
import org.eclipse.gmt.weaver.plugin.util.WeaverInterfaceUtil;
import org.eclipse.gmt.weaver.plugin.util.WeaverUtil;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.dnd.DND;
import org.eclipse.swt.dnd.DropTargetEvent;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.MenuItem;

/**
 * This class manage the drag and drop for woven panel
 * @since 1.0
 */
public class WovenPanelDndManager extends EditingDomainViewerDropAdapter {
	private final IWovenModelPanel wovenPanel;
	final static int NULL_SELECTION = -1;
	final static int DEFAULT_SELECTION = 0;		
	final static int LINK = 1;
	
	private WeaverModelManager modelManager = null;
	private EObject aTargetObj = null;
	private EditingDomain editingDomain = null;
	private EObject aSource = null;
	private int selectionType;
	private EObject weaverSelection;
	
	/**
	 * This creates a manager of drag and drop for woven panel
	 * @param panel
	 * @param domain
	 * @param viewer
	 * @param mng the model manager
	 */
	public WovenPanelDndManager(IWovenModelPanel panel, Viewer viewer){
		super(null,viewer);
		this.wovenPanel = panel;
		this.selectionType = NULL_SELECTION;
		this.viewer = viewer;			
	}
	/**
	 * Enables disables drag and drop over specific elements (WLinkEnd and WLink)
	 * @param event
	 */
	public void helper(DropTargetEvent event){
		selectionType = DEFAULT_SELECTION;
		event.detail=DND.DROP_NONE;
		//gets the source
		source = getDragSource(event);
		aSource = (EObject)source.iterator().next();
		//gets the target
		Object aObj = extractDropTarget( event.item );
		if (weaverSelection != null)
			if (aObj instanceof EObject){        					
				aTargetObj = (EObject)aObj;
				//doesn't allow drag and drop of weaving elements 
				if (weaverSelection.eResource() != aSource.eResource()){					
					if (  WeaverUtil.isInstanceOf(weaverSelection,"WLink",
							WeaverUtil.getMetamodelPackages(weaverSelection))){
						//event.detail=DND.DROP_NONE;
						event.detail=DND.DROP_LINK;
						selectionType = LINK;							
					}
				}
			}			
	}        		      
	/**
	 * action to drop
	 * @param event
	 */
	public void drop(DropTargetEvent event){
		//super.drop(event);
		if (selectionType == DEFAULT_SELECTION )//just enter into the selection code if an object is selected
			super.drop(event);
		else{
			switch (selectionType) {
			case LINK:				
				 createContextDnDMenu(weaverSelection,aSource,aTargetObj,"WLinkEnd");
				 break;
			}
		}
	}	 	    	    
	/**
	 * Creates a dynamic pop up menu after dragging an element over it if it has more than 1 wlinkend or association end child
	 * @param weavingTarget
	 * @param aSource1
	 * @param aSource2
	 * @param eObjTypeDescr
	 */
	public void createContextDnDMenu(EObject weavingTarget, EObject aSource1, EObject aSource2,  String eObjTypeDescr){
		
		
		AdapterFactoryEditingDomain factory = (AdapterFactoryEditingDomain) editingDomain; 
		IEditingDomainItemProvider adapter = (IEditingDomainItemProvider) factory.getAdapterFactory().adapt(weavingTarget,IEditingDomainItemProvider.class);

		Collection aCollection =null;
		if (adapter != null)
			aCollection = adapter.getNewChildDescriptors(weavingTarget,editingDomain,null);		
		List aList = new ArrayList();
		for (Iterator i = aCollection.iterator(); i.hasNext(); )
			aList.add(i.next());

		//if it is a model element must first create a wlink
		if (aList.size() == 1){				
			CommandParameter cmd = (CommandParameter) aList.get(0);
			WeaverInterfaceUtil.setNewWLinkEnd(aSource1, weavingTarget,cmd.getEReference().getEType().getName(),
					cmd.getEReference().getName(),modelManager,editingDomain);
			WeaverInterfaceUtil.setNewWLinkEnd(aSource2, weavingTarget,cmd.getEReference().getEType().getName(),
					cmd.getEReference().getName(),modelManager,editingDomain);								
		}
		else
			if (aList.size() > 1) {//there is more than 1 element, creates 2 pop up menus				
				createPopUpMenu(aList,weavingTarget,aSource1, WeaverBaseExtensionPlugin.getPlugin().getString("_UI_drag_element"));
				createPopUpMenu(aList,weavingTarget,aSource2, WeaverBaseExtensionPlugin.getPlugin().getString("_UI_drop_element"));																					
			}			
	}
	/**
	 * 
	 */
	private void createPopUpMenu(List aList, EObject weavingTarget, EObject aSource, String title) {
		Menu wElementDndMenu = new Menu(viewer.getControl());
		MenuItem aTitle  = new MenuItem(wElementDndMenu, SWT.PUSH);		
		aTitle.setText(title);
		aTitle.setEnabled(false);
		
		for (Iterator i = aList.iterator(); i.hasNext();){
			CommandParameter cmd = (CommandParameter) i.next();
			if ( modelManager.getHelper().isInstanceOfMetaModel(cmd.getEValue(),"WLinkEnd")){
				MenuItem item  = new MenuItem(wElementDndMenu, SWT.PUSH);
				String wElementTypeDescr = cmd.getEValue().eClass().getName();
				EReference ref = cmd.getEReference();
				if (ref.getEAnnotations().size() >= 0 ) //is subsets
					wElementTypeDescr = "<"+ref.getName()+">" + wElementTypeDescr ;
				item.setText(wElementTypeDescr);										
				if (! cmd.getEReference().isMany() )
					item.setEnabled(weavingTarget.eGet(ref)==null);
				//puts all the needed data in the menu item
				//the source, the target class name
				item.setData(new Object[]{aSource, cmd.getEValue().eClass().getName(), ref.getName()});
				DndSelectionAdapter dndAdapter = new DndSelectionAdapter(viewer, weavingTarget);
				item.addSelectionListener(dndAdapter);
			}
		}
		wElementDndMenu.setVisible(true);		
	}
	/**
	 * This class adapt the drag and drop
	 */
	class DndSelectionAdapter extends SelectionAdapter {
		Viewer viewer = null;
		EObject weavingSource = null;
		public DndSelectionAdapter(Viewer viewer, EObject weavingSource){
			this.viewer = viewer;
			this.weavingSource = weavingSource;
		}
		public void widgetSelected(SelectionEvent e){
				//executes the method for creating a new element or reference
//				the source, the target class name
				WeaverInterfaceUtil.setNewWLinkEnd(
						(EObject)((Object[]) e.widget.getData())[0],
						weavingSource,
						(String)((Object[]) e.widget.getData())[1], 
						(String)((Object[]) e.widget.getData())[2], modelManager, editingDomain );
		}		
	}
	public void setWeaverSelection(EObject currentWeaverSelection) {
		weaverSelection = currentWeaverSelection;		
	}	
	
	/**
	 * Sets the weaving editing domain
	 * @param domain The wDomain to set.
	 */
	public void setWEditingDomain(EditingDomain domain) {
		editingDomain = domain;
	}
	/**
	 * Sets  the weaving model manager
	 * @param manager The wManager to set.
	 */
	public void setWManager(WeaverModelManager manager) {
		modelManager = manager;
	}	
	
}