/*******************************************************************************
 * Copyright (c) 2005 INRIA.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Marcos Didonet Del Fabro - initial API and implementation
 *******************************************************************************/


package org.eclipse.gmt.weaver.extension.providers;

import org.eclipse.emf.common.notify.Notification;
import org.eclipse.emf.common.notify.Notifier;
import org.eclipse.emf.ecore.EModelElement;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.EStructuralFeature;
import org.eclipse.gmt.weaver.providers.IIdentifierAdapter;

/**
 * Handles ID generation for an XMI resource.
 * 
 * @since 1.1
 */

public class MergeIdentifierAdapter implements IIdentifierAdapter {
	
	private EObject target;	
	private Object ID = null;
	
	public MergeIdentifierAdapter(EObject target){
		this.target = target;		
	}		
	/* (non-Javadoc)
	 * @see org.eclipse.gmt.weaver.providers.IIdentifierAdapter#setID(java.lang.Object)
	 */
	public void setID(Object obj) {		
		
	}

	/* (non-Javadoc)
	 * @see org.eclipse.gmt.weaver.providers.IIdentifierAdapter#getID()
	 */
	public Object getID() {
		if (ID == null){
			if (target instanceof EStructuralFeature){
				EObject container = (EObject)target.eGet(target.eClass().getEStructuralFeature("eContainingClass"));
				ID = (String)container.eGet(container.eClass().getEStructuralFeature("name")) +
						'/' + (String)target.eGet(target.eClass().getEStructuralFeature("name"));				
			}
			else  if (target instanceof EModelElement){
				ID = (String)target.eGet(target.eClass().getEStructuralFeature("name"));			
			}
		}
		return ID;
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.emf.common.notify.Adapter#notifyChanged(org.eclipse.emf.common.notify.Notification)
	 */
	public void notifyChanged(Notification notification) {
		//does nothing. not used for this adapter
	}
	/* (non-Javadoc)
	 * @see org.eclipse.emf.common.notify.Adapter#getTarget()
	 */
	public Notifier getTarget() {		
		return target;
	}
	/* (non-Javadoc)
	 * @see org.eclipse.emf.common.notify.Adapter#setTarget(org.eclipse.emf.common.notify.Notifier)
	 */
	public void setTarget(Notifier newTarget) {
		target = (EObject) newTarget;		
	}
	/* (non-Javadoc)
	 * @see org.eclipse.emf.common.notify.Adapter#isAdapterForType(java.lang.Object)
	 */
	public boolean isAdapterForType(Object type) {
		//does nothing. not used for this adapter
		return false;
	}
	
}