/*******************************************************************************
 * Copyright (c) 2006 INRIA.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Marcos Didonet Del Fabro - initial API and implementation
 *
 *******************************************************************************/

package org.eclipse.gmt.weaver.qatl;

import java.net.MalformedURLException;
import java.net.URL;
import java.util.Collection;

import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.EReference;
import org.eclipse.emf.edit.EMFEditPlugin;
import org.eclipse.emf.edit.command.CommandParameter;
import org.eclipse.gmt.weaver.extension.providers.WElementItemProvider;

/**
 * @since AMW2006
 * 
 */

public class TransformationItemProvider extends WElementItemProvider {

	public TransformationItemProvider() {
		super();
	}
	
	/**
     * sets constraints on the returned children objects and adds then into a deleteList
     * @param parent the current parent obj
     * @param child the child
     * @param afectedObj an associated object (usually is the child itself) 
     * @param deleteList the constrained object that will be deleted
     */
    protected void setChildrenConstraints(EObject parent, EObject child, Object afectedObj, Collection deleteList){
        //models cant have links and association ends as child
    	super.setChildrenConstraints(parent, child, afectedObj, deleteList);    	
        if (isInstanceOf(parent,"Binding") && isInstanceOf(child,"WLink")) 
        	if (afectedObj instanceof CommandParameter) {
        		if (((CommandParameter)afectedObj).getEReference() != null) {
        			EReference ref = ((CommandParameter)afectedObj).getEReference();
        			if (ref.getName().equals("child"))
        				deleteList.add(afectedObj);
        		}
        		
        	}
             
    }
    
    private URI getImageURL(String icon, EClass eClass){
    	URL url =null;
    	try {
			url = new URL(Activator.getDefault().getBundle().getEntry("/"),icon);
		} catch (MalformedURLException e) {
			System.out.println("Failed to load image for '" + icon + "'");
			e.printStackTrace();
		}
		return URI.createURI(url.toString() + "#" + eClass.getName());   
    }
    
    /* (non-Javadoc)
     * @see org.eclipse.emf.edit.provider.ItemProviderAdapter#getImage(java.lang.Object)
     */
    public Object getImage(Object object){
    	EObject eObject = (EObject)object;
    	EClass eClass = eObject.eClass();
    	String name = eClass.getName();
    	String icon = getIconPath(name);
    	if (icon != null ) {
    		return getImageURL(icon, eClass);
    	}
    	else
    		return URI.createURI(getResourceLocator().getImage("full/obj16/Item").toString() + "#" + eClass.getName());
    }
    
    /* (non-Javadoc)
     * @see org.eclipse.emf.edit.command.CreateChildCommand$Helper#getCreateChildImage(java.lang.Object, java.lang.Object, java.lang.Object, java.util.Collection)
     */
    public Object getCreateChildImage(Object owner, Object feature, Object child, Collection selection){ 
    	EObject eObject = (EObject)child;
    	EClass eClass = eObject.eClass();
    	String name = eClass.getName();
    	String icon = getIconPath(name);
    	if (icon != null ) {
    		return getImageURL(icon, eClass);
    	}
    	else
    		return EMFEditPlugin.INSTANCE.getImage("full/ctool16/CreateChild");
    }
    
    

}
