/*******************************************************************************
 * Copyright (c) 2006 INRIA.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 *******************************************************************************/

package org.eclipse.gmt.weaver.transformation;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.eclipse.core.runtime.IConfigurationElement;
import org.eclipse.core.runtime.IExtension;
import org.eclipse.gmt.weaver.core.WeaverExtensionManager;
import org.eclipse.gmt.weaver.core.WeaverModelManager;
import org.eclipse.gmt.weaver.core.data.WeaverExtensionPointDescriptor;
import org.eclipse.gmt.weaver.exception.WeaverException;
import org.eclipse.gmt.weaver.transformation.config.TBinding;
import org.eclipse.gmt.weaver.transformation.config.TConfigSet;
import org.eclipse.gmt.weaver.transformation.config.TransformationConfig;
import org.eclipse.gmt.weaver.transformation.util.TransformationConstant;

/**
 * This class is a manager of extensions for the weaver
 * @author - Marcos Didonet Del Fabro (LINA Labs, INRIA team)<br>
 * @since 1.0
 */
public class TransformationExtensionManager extends WeaverExtensionManager {
   
	// list of "extension points descriptors" in the application
    
    // Map of icons
   
    
    private TransformationExtensionManager () {
    	super(null);
    }
    /**
     * Create a new WeaverExtensions
     */
    public TransformationExtensionManager (WeaverModelManager manager) {
    	super(manager);    	
        extensionPoints.add(new WeaverExtensionPointDescriptor(TransformationConstant.TRANSFORMATION_EXT, TransformationConstant.TRANSFORMATION_ID));        
    }
  
    /**
     * returns a list with the matching transformation available
     * @return
     */
    public List getMatchingExtensions() {
    	return getTransformationExtensions(TransformationConstant.MATCHING_TRANSFORMATION);
    }
    
    /**
     * returns a list with the matching transformation available
     * @return
     */  
    public List getHotExtensions() {
    	return getTransformationExtensions(TransformationConstant.HOT_TRANSFORMATION);
    }
    
    
    public List getTransformationConfig() {
      	List extensions = new ArrayList();
    	addAllExtensionsInDescriptor((WeaverExtensionPointDescriptor)extensionPoints.get(TransformationConstant.TRANSFORMATION_EXT));    	
    	WeaverExtensionPointDescriptor extDescriptor = (WeaverExtensionPointDescriptor)extensionPoints.get(TransformationConstant.TRANSFORMATION_EXT);
    	for (Iterator i = extDescriptor.getSelectedExtensions().iterator(); i.hasNext(); ){    		
    		IExtension ext = (IExtension)i.next();    		
    		String extensionName = ext.getNamespaceIdentifier();
    		IConfigurationElement[] elements = ext.getConfigurationElements();
    		for (int j=0; j < elements.length; j++){
    			if (elements[j].getName().equals(TransformationConstant.TRANSFORMATION_CONFIG)) {
    				TConfigSet tcs = new TConfigSet() {};
    				tcs.category = elements[j].getAttribute("category");
    				tcs.description = elements[j].getAttribute("description");
    				tcs.configurationModel = extensionName + "/" +elements[j].getAttribute("configuration_model");
    				extensions.add(tcs);
    			}
    		}
    	}
    	return extensions;
    }
    /**
     * return a list of extensions to "org.eclipse.gmt.weaver.transformationID"
     * this extension point define transformations that can be executed by the prototype
     * @return
     * @throws WeaverException
     */
    private List getTransformationExtensions(String type) {
    	
    	List extensions = new ArrayList();
    	addAllExtensionsInDescriptor((WeaverExtensionPointDescriptor)extensionPoints.get(TransformationConstant.TRANSFORMATION_EXT));    	
    	WeaverExtensionPointDescriptor extDescriptor = (WeaverExtensionPointDescriptor)extensionPoints.get(TransformationConstant.TRANSFORMATION_EXT);
    	    	
    	for (Iterator i = extDescriptor.getSelectedExtensions().iterator(); i.hasNext(); ){    		
    		IExtension ext = (IExtension)i.next();    		
    		String extensionName = ext.getNamespace();
    		IConfigurationElement[] elements = ext.getConfigurationElements();
    		for (int j=0; j < elements.length; j++){
    			if (elements[j].getName().equals(type)) { 
	    			TransformationConfig tc = new TransformationConfig () {}; // this is an anonymous class   	
	    			tc.type = type;
	    			tc.category = elements[j].getAttribute("category");
	    			tc.plugin = ext.getNamespaceIdentifier();
	    			tc.transformation = elements[j].getAttribute("transformation");
	    			tc.suffix = elements[j].getAttribute("file_suffix");
	    			tc.description = elements[j].getAttribute("description");
	    			tc.bindings = new ArrayList();
	    			tc.hasConfigurationModel = false;
	    			
	    			IConfigurationElement[] bindings = elements[j].getChildren();	    			
	    			for (int w=0; w < bindings.length; w++){    				
	    				TBinding b = new TBinding () {};    				
	    				b.isExtraBinding =  bindings[w].getName().equals(TransformationConstant.EXTRA_BINDING);	    				
	    				b.weavingReference = bindings[w].getAttribute("weavingReference");
	    				b.header = bindings[w].getAttribute("header");
	    				b.description = bindings[w].getAttribute("description");
	    				b.metamodelHeader = bindings[w].getAttribute("metamodelHeader");
	    				b.path = extensionName +"/"+bindings[w].getAttribute("path");
	    				b.metamodelPath = bindings[w].getAttribute("metamodelPath");
	    				tc.bindings.add(b);
	    			}
	    			extensions.add(tc);
	    			//also adds a new transformation that contains a parameter model
	    			TransformationConfig tcConfig = (TransformationConfig) tc. clone();
	    			tcConfig.transformation = tcConfig.transformation.replaceAll(".asm", TransformationConstant.PARAM_TRANSFO_SUFFIX+".asm"); //adds a suffix before the extension	    			
	    			tcConfig.hasConfigurationModel = true;
	    			extensions.add(tcConfig);
    			}
    		}
    	}
    	return extensions;
    }	        

}
