/*******************************************************************************
 * Copyright (c) 2006 INRIA.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 *******************************************************************************/

package org.eclipse.gmt.weaver.transformation.matching;

import java.io.IOException;
import java.net.URL;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Collections;
import java.util.Date;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Platform;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.EStructuralFeature;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.gmt.weaver.editors.GlobalWeaverEditor;
import org.eclipse.gmt.weaver.exception.WeaverException;
import org.eclipse.gmt.weaver.plugin.WeaverConstant;
import org.eclipse.gmt.weaver.plugin.WeaverPlugin;
import org.eclipse.gmt.weaver.plugin.util.WeaverResourceUtil;
import org.eclipse.gmt.weaver.plugin.util.WeaverUtil;
import org.eclipse.gmt.weaver.transformation.TransformationActivator;
import org.eclipse.gmt.weaver.transformation.TransformationExtensionManager;
import org.eclipse.gmt.weaver.transformation.config.TransformationConfig;
import org.eclipse.gmt.weaver.transformation.util.TransformationConstant;
import org.eclipse.gmt.weaver.transformation.util.TransformationUtil;
import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.m2m.atl.core.ATLCoreException;
import org.eclipse.m2m.atl.core.IInjector;
import org.eclipse.m2m.atl.core.IModel;
import org.eclipse.m2m.atl.core.IReferenceModel;
import org.eclipse.m2m.atl.core.ModelFactory;
import org.eclipse.m2m.atl.core.emf.EMFModel;
import org.eclipse.m2m.atl.core.launch.ILauncher;
import org.eclipse.m2m.atl.core.service.CoreService;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;

/**
* @author - Marcos Didonet Del Fabro (LINA Labs, INRIA Group), UFPR - Universidade Federal do Parana'<br>
* @since 1.0
*/

public class MatchConfigurationDialog extends Dialog {

	
	private Map map = new HashMap();
	
	private GlobalWeaverEditor editor = null;
	
	private IModel configModel = null;
	private IReferenceModel configMetaModel = null;
	private String configPath = "";
	
	private static String match_p = "match_parameter.ecore";
	private static String match = "match_configuration";
	
	private Button saveFileBtn = null;
	
	private TransformationExtensionManager extensionManager;
	
	/**
	 * Creates a new wizard dialog for add model/metamodel
	 * @param parentShell
	 * @param parent
	 * @param extensionsPanel the array of extensions woven panel
	 * @param wModelRefArray the array of wmodelref we find in metamodel extensions
	 */
	public MatchConfigurationDialog(Shell parentShell, GlobalWeaverEditor editor, TransformationExtensionManager extensionManager, String configPath) {
		super(parentShell);
		this.editor = editor;
		this.extensionManager = extensionManager;
		this.configPath = configPath;
		
	}
	
	
	/* (non-Javadoc)
	 * @see org.eclipse.jface.window.Window#configureShell(org.eclipse.swt.widgets.Shell)
	 */
	public void configureShell(Shell shell){
		super.configureShell(shell);
		shell.setText(TransformationActivator.getDefault().getString("_UI_match_config"));//title
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.jface.dialogs.Dialog#createDialogArea(org.eclipse.swt.widgets.Composite)
	 */
	public Control createDialogArea(Composite parent){
		Composite container = new Composite(parent, SWT.NULL);
		{
			GridLayout layout = new GridLayout();
			layout.numColumns = 1;
			container.setLayout(layout);
		}	
		// Create controls
		createFields(container);
		return parent;
	}
	
	//	iterates over the graphical elements and saves the new values into the eobjects
	private void setModelValues () {
		for (Iterator it = map.keySet().iterator(); it.hasNext();) {
			Object[] element = (Object[]) it.next();
			Object obj  =  map.get(element);
			if (obj instanceof Text) {
				EObject eObj = (EObject)element[0]; 
				EStructuralFeature eFeat = (EStructuralFeature)element[1];
				String str = ((Text)obj).getText();
				if (!str.equals("")) {
					if (eFeat.getEType().getName().equals("Double")) {
						eObj.eSet(eFeat, new Double(str));
					} else
					if (eFeat.getEType().getName().equals("String")) {
						eObj.eSet(eFeat, str);
					} else
					if (eFeat.getEType().getName().equals("Integer")) {
						eObj.eSet(eFeat, new Integer(str));
					}
				}
			}
			else
			if (obj instanceof Button) {
				EObject eObj = (EObject)element[0]; 
				EStructuralFeature eFeat = (EStructuralFeature)element[1];
				eObj.eSet(eFeat, new Boolean(((Button)obj).getSelection()));
			}		
		}		
	}
	/* (non-Javadoc)
	 * @see org.eclipse.jface.dialogs.Dialog#okPressed()
	 */
	public void okPressed(){
		// Verify if the data is valid
		if (! this.verifyData()){
			MessageDialog.openError(getShell(), 
					WeaverPlugin.getPlugin().getString("_UI_warning_label"),
							WeaverPlugin.getPlugin().getString("_UI_wizard_dialog_warning"));
			return;
		}
		setModelValues();//iterates over the graphical elements and saves the new values into the eobjects
		
		try {
			int stepCount = 1;
			
			Resource res = ((EMFModel)configModel).getResource();
			//res.save(Collections. EMPTY_MAP);  model saved, now execute the transformations
			
			SimpleDateFormat dateF = new SimpleDateFormat("mm:ss SSS");
			
			long start = Calendar.getInstance().getTimeInMillis();
			debug ("Matching started at: " + dateF.format(new Date(start)));
			
			IModel current = null;
			TransformationConfig tc = null; //contains the configuration loaded by the extension manager
			
			for (Iterator objs = res.getAllContents(); objs.hasNext();) {
				Object element = objs.next();//searches for the model elements
				if (element instanceof EObject) {					
					if (WeaverUtil.isInstanceOf((EObject)element, TransformationConstant.BASE_TRANSFORMATION_TYPE, 
							((EMFModel)configMetaModel).getResource())){
						EObject eObj = (EObject) element;
						Object value = eObj.eGet(eObj.eClass().getEStructuralFeature(TransformationConstant.EXECUTE_TRANSFO_FEATURE));
						if (value != null) {//if the property value is true, find and execute the transformation
							if (((Boolean)value).booleanValue()){
								Object transformation = eObj.eGet(eObj.eClass().getEStructuralFeature(TransformationConstant.TRANSFO_NAME_FEATURE));
								for (Iterator tcs = extensionManager.getMatchingExtensions().iterator(); tcs.hasNext();) {
									tc = (TransformationConfig) tcs.next();// compares the transformation names									
									if (tc.hasConfigurationModel){
										if (tc.transformation.substring(tc.transformation.lastIndexOf("/")+1).equals(transformation+".asm")) {
											ILauncher launcher = TransformationUtil.setModelsMap(current, editor.getModelManager(), true, true, configModel, configMetaModel);
											current = null;
											current = TransformationUtil.executeTransfo(launcher,editor.getModelManager(),editor.getXmlFile(),tc);
											debug(transformation + " executed at: "+dateF.format( new Date(Calendar.getInstance().getTimeInMillis())));
											if (saveFileBtn.getSelection()) {//save a different file for all models
												String suffix = "_step"+stepCount;
												TransformationUtil.saveNewModel(tc, current, editor.getModelManager(), editor.getXmlFile(), suffix);	
												stepCount++; // used to set up the file suffix
											}											
											break;
										}
									}									
								}								
							}
						}
					}
				}			
			}
			long end = Calendar.getInstance().getTimeInMillis(); 
			debug ("----------------------------------------------------");	
			debug ("Matching finished at: " + dateF.format(new Date(end)));			
			debug ("Total time: " + dateF.format(new Date(end - start)));
			
			//show confirmation messages
			if (! saveFileBtn.getSelection()) {
				IModel savedM = TransformationUtil.saveNewModel(tc, current, editor.getModelManager(), editor.getXmlFile(),"");
				if (savedM != null) {
					MessageDialog.openInformation(editor.getEditorSite().getShell(), tc.description, 
							TransformationActivator.getDefault().getString("_UI_model_created")+ ((EMFModel)savedM).getResource().getURI().toString());					
				}
			} else
				MessageDialog.openInformation(editor.getEditorSite().getShell(), tc.description, 
						TransformationActivator.getDefault().getString("_UI_general_match"));					
		
		} catch (Exception e) {
			e.printStackTrace();
		}
		
		WeaverResourceUtil.refreshResourceFolder(editor.getModelManager().getWeavingModel());
		this.close();
	}
	/*
	 * used to better identify the debug messages
	 */
	private void debug(String message){
		System.out.println(message);
	}

	/**
	 * //TODO Tests the controls - does nothing for the moment. Validate the weights and thresholds, etc.
	 * @return
	 */
	private boolean verifyData() {
		//threshold,
		//normalize
		//weight
		return true;
	}
	
	/**
	 * creates the controls for this model/metamdodel
	 * @param container
	 */
	private void createFields(Composite container) {
		// First line : name
		
		ModelFactory factory;
		IInjector injector;
		try {
			factory = CoreService.getModelFactory(WeaverConstant.EMF);
			injector = CoreService.getInjector(WeaverConstant.EMF);
		} catch (ATLCoreException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			return;
		}
		//AtlModelHandler modelHandler = AtlModelHandler.getDefault(TransformationConstant.AMWModelHandler);
		
		//models
		URL mmurl = getClass().getResource(match_p);
		
		try {//loads configuration metamodel 
			//ResourceSet rs = ASMEMFModel.getResourceSet();
			configMetaModel = factory.newReferenceModel();
			injector.inject(configMetaModel, mmurl.openStream(), Collections.EMPTY_MAP);
		
//			Resource res = rs.createResource(URI.createURI("match_parameter"));
//			URL f = FileLocator.toFileURL(mmurl); 
//			InputStream file = UtilFile.getStreamForFile(f.getPath());
//			res.load(file, Collections.EMPTY_MAP);
//			WeaverResourceUtil.initResource (res);
			
			//configMetaModel = ASMAMWModel.loadASMAMWModel(match_p, (ASMEMFModel)modelHandler.getMof(), res, null);
			//configMetaModel = ASMAMWModel.loadASMAMWModel(match_p, (ASMEMFModel)modelHandler.getMof(), mmurl, null);

			String plugin = configPath.substring(0, configPath.indexOf("/"));
			String extensionPath = configPath.substring(configPath.indexOf("/") + 1);
			URL urlPlatform = Platform.getBundle(plugin).getEntry(extensionPath);
			try {
				
				//factory
				
				//loads configuration model
//				f = FileLocator.toFileURL(urlPlatform);           
//				file = UtilFile.getStreamForFile(f.getPath());
//				res = rs.createResource(URI.createURI(match));
//				res.load(file, Collections.EMPTY_MAP);
				
				configModel = factory.newModel(configMetaModel);
				
				injector.inject(configModel, urlPlatform.openStream(), Collections.EMPTY_MAP);
				//configModel = ASMAMWModel.loadASMAMWModel(match, configMetaModel, res, null);;
					//ASMAMWModel.loadASMAMWModel(match, configMetaModel, file, null);
				//((Resource)wManager.getWovenMeta().get(0))
			} catch (IOException e1) {
				e1.printStackTrace();
			}

		} catch (Exception e1) {
			e1.printStackTrace();
		}
		
		//sets a grid
		Composite composite = new Composite(container,SWT.BEGINNING);
		{
			GridData gdata = new GridData(GridData.FILL_HORIZONTAL);
			gdata.horizontalSpan=1;
			composite.setLayoutData(gdata);
		}
		{
			GridLayout layout = new GridLayout();
			composite.setLayout(layout);
			layout.numColumns = 2;
			layout.marginWidth = 4;
		}
		EMFModel emfModel = (EMFModel) configModel;
		//the key of the map is the 'name property of the class' + 'attribute name'
		List transfos = WeaverUtil.getElementsByTypeHierarchy("LinkGeneration", emfModel.getResource());		
		createTransformationInterface(composite, transfos, map, TransformationActivator.getDefault().getString("_UI_Linkgen"));

		transfos = WeaverUtil.getElementsByTypeHierarchy("ElementToElement", emfModel.getResource());		
		createTransformationInterface(composite, transfos, map, TransformationActivator.getDefault().getString("_UI_Element"));

		transfos = WeaverUtil.getElementsByTypeHierarchy("Structural", emfModel.getResource());		
		createTransformationInterface(composite, transfos, map,TransformationActivator.getDefault().getString("_UI_Structural"));
		
		transfos = WeaverUtil.getElementsByTypeHierarchy("Filter", emfModel.getResource());		
		createTransformationInterface(composite, transfos, map, TransformationActivator.getDefault().getString("_UI_Filter"));
		
		setSaveFile(composite);
		
	}
	/**
	 * create the transformation interface for a given type
	 * @param aComposite
	 * @param transfos
	 * @param map
	 * @param title
	 */
	private void createTransformationInterface (Composite aComposite, List transfos, Map map, String title) {
		if (transfos.size() == 0)
			return;
		Group composite = new Group (aComposite,SWT.NONE);
		{
			GridData gdata = new GridData(GridData.FILL_HORIZONTAL);
			gdata.horizontalSpan=6;
			composite.setLayoutData(gdata);
		}
		{
			GridLayout layout = new GridLayout();
			composite.setLayout(layout);
			layout.numColumns = 9;
			layout.marginWidth = 4;
		}
		composite.setText(title);
		
		for (Iterator it = transfos.iterator(); it.hasNext();) {
			EObject element = (EObject) it.next();
									
			setTextField(composite, map, 
					(String) element.eGet(element.eClass().getEStructuralFeature("description")), 
					"Name", 
					"shortDesc", element, false);
			
			if (featureExists("weight", element) ) {
				setTextField(composite, map, 
						"", 
						"Weight", 
						"weight", element, true );				
			}
			else 
				createExtraFields(composite);
			
			if (featureExists("threshold", element) ) {
				setTextField(composite, map, 
						"", 
						"Threshold", 
						"threshold", element, true);				
			}
			else 
				createExtraFields(composite);
			setCheckBox(composite, map, "", "Execute","selected", element);
			
			setHelpButton(composite, element);
		}		
	}
	/**
	 * set help button with information about each transformation
	 * @param composite
	 * @param eObj
	 */
	private void setHelpButton(Group composite, EObject eObj) {
		class ASelectionAdapter extends SelectionAdapter {
			public EObject eObj = null;
			public ASelectionAdapter (EObject eObj) {
				this.eObj = eObj;
			}
			public void widgetSelected(SelectionEvent e) {
				MessageDialog.openInformation(editor.getEditorSite().getShell(), 
						TransformationActivator.getDefault().getString("_UI_info_label"),
						TransformationActivator.getDefault().getString("_UI_dependency")+"\n\n"+
						getRefValues(TransformationActivator.getDefault().getString("_UI_metamodel_depend"), eObj,"metamodels")+"\n"+
						getRefValues(TransformationActivator.getDefault().getString("_UI_previous"),eObj, "depends")
						);					
			}
		}
		Button btn = new Button (composite, SWT.PUSH);
		btn.setText("?");
		btn.setToolTipText(TransformationActivator.getDefault().getString("_UI_hint_dependency"));
		btn.addSelectionListener(new ASelectionAdapter(eObj));
	}
	/**
	 * get the values of the references and set them as strings in a pre defined format
	 * @param title
	 * @param eObj
	 * @param featureName
	 * @return
	 */
	private String getRefValues(String title, EObject eObj, String featureName){
		List objs = (List) eObj.eGet(eObj.eClass().getEStructuralFeature(featureName));
		String result = "";
		for (Iterator refs = objs.iterator(); refs.hasNext();) {
			EObject nElement = (EObject) refs.next();
			String name = (String) nElement.eGet(nElement.eClass().getEStructuralFeature("name"));
			result += "-> " + name + "\n";			
		}
		if (! result.equals(""))
			result = title + "\n\n"+result;
		return result;		
	}

	/* 
	 * create empty labels for better organization of fields
	 */
	private void createExtraFields (Composite composite) {
		new Label(composite, SWT.BEGINNING);
		new Label(composite, SWT.BEGINNING);
	}
	/**
	 * tests if a given structural feature exists
	 * @param featName
	 * @param obj
	 * @return
	 */
	private boolean featureExists (String featName, EObject obj){
		return WeaverUtil.getEStructuralFeature(obj, featName) != null;
	}
	/**
	 * create a check box for a boolean field
	 * @param composite
	 * @param map
	 * @param hint
	 * @param name
	 * @param value
	 * @param element
	 */
	private void setCheckBox (Composite composite, Map map, String hint, String name, String featName, EObject element){
		Label label = new Label(composite, SWT.BEGINNING);
		label.setText(name);		
		Object value = element.eGet(element.eClass().getEStructuralFeature(featName));		
		
		Button btn = new Button (composite, SWT.CHECK);
		if (value != null) {
			btn.setSelection(((Boolean)value).booleanValue());
		}
		else
			btn.setSelection(false);
		
		map.put(new Object[]{element, element.eClass().getEStructuralFeature(featName)}, btn);
	}
	/**
	 * creates the "save intermediate models" check
	 * @param composite
	 */
	private void setSaveFile (Composite composite){
		Label label = new Label(composite, SWT.BEGINNING);
		label.setText(TransformationActivator.getDefault().getString("_UI_save_intermediate"));		
		saveFileBtn = new Button (composite, SWT.CHECK);
		saveFileBtn.setSelection(false);
	}
	
	/**
	 * create a text field
	 * @param composite
	 * @param map
	 * @param hint
	 * @param name
	 * @param value
	 * @param element
	 * @param editable
	 */
	private void setTextField(Composite composite, Map map, String hint, String name, String featName, EObject element, boolean editable) {
		Label label = new Label(composite, SWT.BEGINNING);
		label.setText(name);		
		Object value = element.eGet(element.eClass().getEStructuralFeature(featName));

		Text text = new Text(composite, SWT.BORDER | SWT.SINGLE);
		if (value != null)
			text.setText(value.toString());
		else
			text.setText("");
		
		if (!editable) {
			text.setEditable(false);
			Color c = text.getBackground();
			text.setBackground(new Color (c.getDevice(), 255, 255, 255));		
		}		
		text.setToolTipText(hint);
		map.put(new Object[]{element, element.eClass().getEStructuralFeature(featName)}, text);
	}

}
