/*******************************************************************************
 * Copyright (c) 2006 INRIA.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 *******************************************************************************/

package org.eclipse.gmt.weaver.transformation.panel;

import java.util.HashMap;
import java.util.Iterator;

import org.eclipse.gmt.weaver.editors.GlobalWeaverEditor;
import org.eclipse.gmt.weaver.editors.weaving.GenericWeavingPanel;
import org.eclipse.gmt.weaver.plugin.util.WeaverResourceUtil;
import org.eclipse.gmt.weaver.transformation.TransformationActivator;
import org.eclipse.gmt.weaver.transformation.TransformationExtensionManager;
import org.eclipse.gmt.weaver.transformation.config.TConfigSet;
import org.eclipse.gmt.weaver.transformation.config.TransformationConfig;
import org.eclipse.gmt.weaver.transformation.matching.MatchConfigurationDialog;
import org.eclipse.gmt.weaver.transformation.util.TransformationConstant;
import org.eclipse.gmt.weaver.transformation.util.TransformationUtil;
import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.jface.action.IStatusLineManager;
import org.eclipse.jface.action.IToolBarManager;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.jface.action.Separator;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.m2m.atl.core.IModel;
import org.eclipse.m2m.atl.core.emf.EMFModel;
import org.eclipse.m2m.atl.core.launch.ILauncher;


/**
 * Transformation weaving panel extension
 * @author - Marcos Didonet Del Fabro (LINA Labs, INRIA Group)<br>
 * @since 1.0
 */
public class TransformationWeavingPanel extends GenericWeavingPanel {
	// Panel
	
	private HashMap menuManagers = new HashMap();
		
	protected TransformationExtensionManager extensionManager = null;
	/**
	 * Constructs a view panel for a view part.
	 */
	public TransformationWeavingPanel(){
	}
	/* (non-Javadoc)
	 * @see org.eclipse.gmt.weaver.editors.IWeavingPanel#init(org.eclipse.gmt.weaver.editors.GlobalWeaverEditor)
	 */
	public void init(GlobalWeaverEditor editor) {
		super.init(editor);
		extensionManager = new TransformationExtensionManager(editor.getModelManager());
	}	
	
	public void populateMenuAboutToShow(IMenuManager parentMenu){
		super.populateMenuAboutToShow(parentMenu);
		parentMenu.add(new Separator());
		addTransformationExtensions(parentMenu);
	}
	/**
	 * adds menu that configures transformations
	 * @param parentMenu
	 */
	private void addConfigMenuAction(IMenuManager parentMenu, String description, String confModel) {
		 IAction executeGeneralMatch = new MyMenuAction (description, confModel, globalEditor, extensionManager);
		parentMenu.add(executeGeneralMatch);		
	}
	/**
	 * wrapper class to be able to set up variables
	 * @author didonet
	 *
	 */
	class MyMenuAction extends Action {
		private String confModel = "";
		GlobalWeaverEditor globalEditor = null;
		TransformationExtensionManager tem = null;
	    public MyMenuAction (String description, String confModel, GlobalWeaverEditor globalEditor,
	    		TransformationExtensionManager tem){
	    	super (description);
	    	this.confModel = confModel;
	    	this.globalEditor = globalEditor;
	    	this.tem = tem;
	    }
    	public void run() {
    		MatchConfigurationDialog mc = new MatchConfigurationDialog(
    				globalEditor.getWeaverEditorPanel().getViewer().getControl().getShell(),
    				globalEditor,
    				tem,
    				confModel);
    		mc.open();
    	}
		
	}
	/**
	 * Makes contributions
	 * @param menuManager
	 * @param toolBarManager
	 * @param statusLineManager
	 */
	public void makeContributions(IMenuManager menuManager,
			IToolBarManager toolBarManager, IStatusLineManager statusLineManager) {
		
		menuManager.add(new Separator());
		//toolBarManager.update(true);		
		menuManager.update(true);
	}
	
	/**
	 * adds the actions of matching transformations and HOT transformations obtained through the "transformation" extension point
	 * @param menuManager
	 */
	private void addTransformationExtensions(IMenuManager menuManager) {

		//add transformation tasks
		menuManagers.clear();

		for (Iterator tcs = extensionManager.getMatchingExtensions().iterator(); tcs.hasNext();) {
			TransformationConfig tc = (TransformationConfig) tcs.next();
			if (! tc.hasConfigurationModel) {
				IMenuManager mngr = (IMenuManager) menuManagers.get(tc.category);
				if (mngr == null) {
					mngr = new MenuManager(tc.category);
					menuManager.add(mngr);
					menuManagers.put(tc.category, mngr);
				}
				createAction(tc, mngr);
			}
		}

		//add transformation tasks
		for (Iterator tcs = extensionManager.getHotExtensions().iterator(); tcs.hasNext();) {
			TransformationConfig tc = (TransformationConfig) tcs.next();
			if (! tc.hasConfigurationModel) {
				IMenuManager mngr = (IMenuManager) menuManagers.get(tc.category);
				if (mngr == null) {
					mngr = new MenuManager(tc.category);
					menuManager.add(mngr);
					menuManagers.put(tc.category, mngr);				
				}
				createAction(tc, mngr);
			}		
		}

//		add configuration menus
		for (Iterator tcs = extensionManager.getTransformationConfig().iterator(); tcs.hasNext();) {
			TConfigSet tc = (TConfigSet) tcs.next();
			IMenuManager mngr = (IMenuManager) menuManagers.get(tc.category);
			if (mngr == null) {
				mngr = new MenuManager(tc.category);
				menuManager.add(mngr);
				menuManagers.put(tc.category, mngr);				
			}
			addConfigMenuAction(mngr, tc.description, tc.configurationModel);

		}


	}
		
		protected void createAction(TransformationConfig tc, IMenuManager menuManager){
			IAction transform = new TAction( tc );
		    menuManager.add(transform);	    
		}
		/**
		 * Action that executes a transformation in the weaving panel
		 * //the transformations may have two different signatures:
		 * create OUT : ATL from IN : AMW, left: MOF, right: MOF; for the higher-order transformations
		 * create OUT : AMW from IN: AMW, left : MOF, right : MOF; for the matching transformations
		 * @author Marcos Didonet Del Fabro
		 *
		 */    
		class TAction extends Action {
			private TransformationConfig tc = null; 
			public TAction (TransformationConfig tc){
				super(tc.description);
				this.tc = tc;			
			}
			public void run() {
	    		try {
	    			GlobalWeaverEditor editor = globalEditor;
	    			ILauncher launcher = TransformationUtil.setModelsMap(null, editor.getModelManager(), ! tc.type.equals(TransformationConstant.HOT_TRANSFORMATION), false, null, null);	    			
	    			IModel out = TransformationUtil.executeTransfo(launcher, editor.getModelManager(), globalEditor.getXmlFile(), tc);	    			
	    			// in the launch configuration, we assume the models conform to MOF and AMW
	    			TransformationUtil.saveNewModel(tc, out, editor.getModelManager(), editor.getXmlFile(),"");    			
	    			//refreshes the resource and shows confirmation message
	    			WeaverResourceUtil.refreshResourceFolder(editor.getModelManager().getWeavingModel());
	    			MessageDialog.openInformation(editor.getEditorSite().getShell(), tc.description, 
	    					TransformationActivator.getDefault().getString("_UI_model_created") + 
	    					((EMFModel)out).getResource().getURI().toString());

	    		} catch (Exception e) {
	    			e.printStackTrace();
	    		}
			}
		}
	
}

