/*******************************************************************************
 * Copyright (c) 2005 INRIA.

 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Marcos Didonet Del Fabro - initial API and implementation
 *    Guillaume Gueltas
 *******************************************************************************
 *
  * Copyright (c) 2010 UFPR.

 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Marcos Didonet Del Fabro 
 *******************************************************************************/



package org.eclipse.gmt.weaver.core;



import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.EPackage;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.resource.ResourceSet;
import org.eclipse.emf.ecore.resource.impl.ResourceSetImpl;
import org.eclipse.emf.ecore.xmi.impl.XMIResourceFactoryImpl;
import org.eclipse.emf.edit.domain.EditingDomain;
import org.eclipse.gmt.weaver.core.data.WovenModelDescriptor;
import org.eclipse.gmt.weaver.exception.WeaverException;
import org.eclipse.gmt.weaver.plugin.WeaverConstant;
import org.eclipse.gmt.weaver.plugin.WeaverPlugin;
import org.eclipse.gmt.weaver.plugin.util.WeaverResourceUtil;
import org.eclipse.jface.dialogs.ProgressMonitorDialog;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.actions.WorkspaceModifyOperation;

/**
 * This class is the manager of models/metamodels which are used in the weaver 
 * @since 1.0
 */

public class WeaverModelManager {

	//the EMF package registry
	static EPackage.Registry registry = EPackage.Registry.INSTANCE;
	// Helper
	public WeaverModelHelper helper;
	
	// List of URI
	// The init of those URIs is in the constructor
	private URI weavingMetamodelURI = null;
	private URI weavingModelURI = null;
	// Flag to know if we use new model
	private boolean isNewModel = false;
	
	// List of ResourceSet
	private ResourceSet weavingMetamodelRS = null;
	private ResourceSet weavingModelRS = null;
	
	private List resourceSets = new ArrayList();
	
	//private List wovenRessourceSets = new ArrayList();
	//the current wovenResources - it can be a model or metamodel
	private HashMap wovenMetaRelation = new HashMap();
	private List wovenResources = new ArrayList();
	private List wovenResourcesTypes = new ArrayList(); // for knowing if it'as a model (MODEL_TYPE) or a metatmodel (METAMODEL_TYPE)
	
	private WeaverExtensionManager weaverExtensions = null;
	
	private int metamodelType = -1;
	private String metamodelFile ;

	//extension of metamodel (path)
	private List pluginMetamodelExtension;
	private List workspaceMetamodelExtension;
	
	/**
	 * Creates a new model manager for the weaver
	 * @param xml
	 * @param isNewModel
	 * @throws WeaverException
	 */
	public WeaverModelManager(WeaverXMLMetadata xml, boolean isNewModel, int metamodelType) throws WeaverException {
		super();
		helper = new WeaverModelHelper(this);
		weaverExtensions = new WeaverExtensionManager(this);
		this.metamodelType = metamodelType;
		this.metamodelFile = xml.getWeavingEcore();
		
		this.initWeavingMetaModel(xml.getMetamodelExtension(), xml.getLocalMetamodelExtension());
		this.initWeavingModel(xml);
		this.initWoven(xml);
		
		this.isNewModel=isNewModel;
		
	}
	
	/**
	 * Creates a new model manager for the wizard
	 * (only weaving metamodel extension are possible)
	 * @param metamodelExtensions the list (of relative path) of metamodel extension selected (from plugin)
	 * @param localMetaModelExtension the list (of relative path) of metamodel extension selected (from local workspace)
	 * @throws WeaverException 
	 * @throws WeaverException
	 */
	public WeaverModelManager(List metamodelExtensions, List localMetamodelExtension, int metamodelType, String ecoreFile) throws WeaverException{
		super();
		weaverExtensions = new WeaverExtensionManager(this);
		this.metamodelType = metamodelType;
		this.metamodelFile = ecoreFile;
		
		this.initWeavingMetaModel(metamodelExtensions, localMetamodelExtension);		
	}	

	/**
	 * Initialisation of URIs with path of files
	 * And initialisation of chosen extensions for weaving model
	 * @param metamodelExtensions the list (of relative path) of metamodel extension selected (from plugin)
	 * @param localMetaModelExtension the list (of relative path) of metamodel extension selected (from local workspace)
	 * @throws WeaverException 
	 * @throws WeaverException 
	 */
	private void initWeavingMetaModel(List metamodelExtension, List localMetaModelExtension) throws WeaverException {
		// load the selected plugin extensions (choosen in the wizard)
		//weaverExtensions.loadExtensions(metamodelExtension,WeaverConstant.METAMODEL_EXT_ID);
		this.pluginMetamodelExtension = metamodelExtension;
		this.workspaceMetamodelExtension = localMetaModelExtension;
		
		// Init of file path and uris
		this.weavingMetamodelURI = URI.createURI(WeaverConstant.WEAVING_FILE);
		this.getWeavingMetamodel();
	}

	/**
	 * Initialisation of URIs with path of files
	 * And initialisation of choosen extensions for weaving model metamodel
	 * @param xml the xml properties file
	 * @throws WeaverException
	 */
	public void initWeavingModel(WeaverXMLMetadata xml) throws WeaverException {		
		// load the selected extensions (choosen in the wizard)		
		weaverExtensions.loadExtension(xml.getWeavingModelPanel(),WeaverConstant.WEAVING_PANEL_EXT_ID);
		
		// Init of file path and uris
		try{
			this.weavingModelURI = URI.createPlatformResourceURI(xml.getWeavingModelPR());
			//URI.createFileURI(UtilFile.getFileNameFromPath(weaving_metamodel_file));	
		}catch (Exception e){
			throw new WeaverException(IStatus.ERROR,WeaverPlugin.getPlugin().getString("_UI_manager_file_error",xml.getWeavingModelPR()));
		}
	}
	
	/**
	 * Creates the resourceSet and the ressource for woven model / metamodels 
	 * @param xml the xml properties file
	 * @throws WeaverException
	 */
	private void initWoven(WeaverXMLMetadata xml) throws WeaverException {
		// Parse of all models / metamodels
		for(int i = 0;i < xml.getModelsize(); i++){
			WovenModelDescriptor desc = xml.getWovenModelDescriptor(i);
			// we weave model/metamodel or metamodel			
			String metafile = desc.getMetamodelFile();
			
			boolean isMetamodelFile = metafile.indexOf("http") == -1;
			String modelfile = desc.getModelFile();
			boolean isModelExists = modelfile.equals("null") ? false : true;

			// load the selected extensions (choosen in the wizard)
			weaverExtensions.loadExtension(desc.getPanel(),WeaverConstant.WOVEN_PANEL_EXT_ID);	

			try {
				// initialize resource metamodel
				URI uri;
				ResourceSet rs;
				Resource resNoRS = null;
				if (isMetamodelFile){
					uri = URI.createPlatformResourceURI(metafile);
					
					String fullPath = WeaverResourceUtil.getFullPath(metafile);
					rs = WeaverResourceUtil.initializeResourceSet(fullPath, uri);
				}					
				else{
					uri = URI.createURI(metafile);
					rs = WeaverResourceUtil.initializeResourceSet(resourceSets,uri);
					if (rs == null) {// when it is an uri metamodel, it is possible that it does not have resourceset
						EPackage pkg = (EPackage)EPackage.Registry.INSTANCE.getEPackage(uri.toString());		
						resNoRS = pkg.eResource();
					}
				}
				// if we works with models / metamodels
				if(isModelExists){ // model
					// register the mm - not necessary to register mm by URI
					Resource metamodel = null;
					if (rs != null) {
						WeaverResourceUtil.initResource(rs.getResource(uri,false));
						metamodel = (Resource) rs.getResources().get(0);
					}
					else {
						metamodel = resNoRS;
					}
					
					URI modelUri = URI.createPlatformResourceURI(modelfile);
					String fullPath = ResourcesPlugin.getWorkspace().getRoot().getLocation().toString() + modelfile;
					ResourceSet modelRS = WeaverResourceUtil.initializeResourceSet(rs, resNoRS, fullPath,modelUri);	
					resourceSets.add(modelRS);
					Resource model = modelRS.getResource(modelUri,false);
					wovenResources.add(model);
					wovenResourcesTypes.add(WeaverConstant.MODEL_TYPE);
					wovenMetaRelation.put(modelUri, metamodel);
				}
				else{ // metamodels
					resourceSets.add(rs);
					wovenResources.add(rs.getResource(uri,false));	
					wovenResourcesTypes.add(WeaverConstant.METAMODEL_TYPE);
				}				
			} catch (Exception e) {
				throw new WeaverException(IStatus.ERROR,
						WeaverPlugin.getPlugin().getString(
								"_UI_manager_file_error",
								(modelfile.equals("null") ? metafile : modelfile)), e);
			}			
		}
	}	

	// Weaving metamodel
	/**
	 * Returns the resource set of the weaving metamodel
	 * @return ResourceSet
	 * @throws WeaverException
	 */
	public ResourceSet getWeavingMetamodelRS(String fileName, int metamodelType) throws WeaverException {
	    try {
	        if (this.weavingMetamodelRS == null) {	            
	        	if (metamodelType == WeaverConstant.KM3){
	            	if (fileName == null || fileName.equals(""))//take the default values
	            		this.weavingMetamodelRS = getWeavingMetamodelRS();
	            }
	            else
	            	//searches for the xmi file
	            	if (metamodelType == WeaverConstant.XMI && fileName != null){
	            		
	    	            this.weavingMetamodelRS = new ResourceSetImpl();	 

	    	            weavingMetamodelRS.setPackageRegistry(registry);
	    	            weavingMetamodelRS.getResourceFactoryRegistry().getExtensionToFactoryMap().put("xmi", 
	    	            		new XMIResourceFactoryImpl());	    	            
	    	            //weavingMetamodelRS.setResourceFactoryRegistry(ASMEMFModel.getResourceSet().getResourceFactoryRegistry());			
	    	            resourceSets.add(weavingMetamodelRS);
	    	            Resource weavingMetaModel = null;	            	     
	    	            //weavingMetamodelURI = URI.createURI(fileName);
			            weavingMetaModel = this.weavingMetamodelRS.createResource(weavingMetamodelURI);
			            FileInputStream file = new FileInputStream(
			            		ResourcesPlugin.getWorkspace().getRoot().getLocation().toString() + fileName);
			            
			            weavingMetaModel.load(file,Collections.EMPTY_MAP);
			            file.close();				            		            			            			            		
	            	}	            
	        }
	    } catch (IOException e) {
	        e.printStackTrace();
	        throw new WeaverException(IStatus.ERROR,WeaverPlugin.getPlugin().getString("_UI_manager_metamodel_error"),e);
	    }
	    return this.weavingMetamodelRS;
	}	
/**
	 * Returns the resource set of the weaving metamodel
	 * @return ResourceSet
	 * @throws WeaverException
	 */
	public ResourceSet getWeavingMetamodelRS() throws WeaverException {
	    try {
	        if (this.weavingMetamodelRS == null) {
	            this.weavingMetamodelRS = new ResourceSetImpl();
	            resourceSets.add(weavingMetamodelRS);
	            
	            //EcoreFactory f; f.getEcorePackage().geter
	            //weavingMetamodelRS.setPackageRegistry(ASMEMFModel.getResourceSet().getPackageRegistry());
	            //weavingMetamodelRS.setResourceFactoryRegistry(ASMEMFModel.getResourceSet().getResourceFactoryRegistry());			
	            
	            weavingMetamodelRS.setPackageRegistry(registry);
	            weavingMetamodelRS.getResourceFactoryRegistry().getExtensionToFactoryMap().put("xmi", new XMIResourceFactoryImpl());
	            
	            Resource weavingMetaModel = null;	            
	            weavingMetaModel = this.weavingMetamodelRS.createResource(weavingMetamodelURI);
	            // get metamodel stream
	            InputStream abstractMetamodelStream = WeaverModelManager.class.getResourceAsStream(weavingMetamodelURI.toFileString());             
	            // get extensions stream for selected extensions (extensions which have been load in initialization)
	            List km3InputStream = WeaverResourceUtil.loadKm3ExtensionStreamList(
	            		this.getPluginMetamodelExtension(),this.getWorkspaceMetamodelExtension());
	            InputStream km3ExtensionStream = WeaverResourceUtil.addListOfStreams(km3InputStream);
	            // add 2 streams
	            InputStream stream= WeaverResourceUtil.addStream(abstractMetamodelStream,km3ExtensionStream);
	            /*
	            FileOutputStream fs = new FileOutputStream("c:/temp/debugkm3.km3");
	            byte[] buf = new byte[4096];	                      
	            for(int len=-1;(len=stream.read(buf))!=-1;){
	               fs.write(buf,0,len);
	            }
	            fs.close();
	            */

	            // translate from km3 to xmi
	            Resource res = WeaverResourceUtil.km3Stream2EcoreResource(stream,weavingMetamodelURI.toString());
	            // Add contents
	            weavingMetaModel.getContents().addAll(res.getContents()) ;
	            //weavingStreamKM3.close();
	            abstractMetamodelStream.close();
	            
	            // validate metamodel semantic 
	            WeaverResourceUtil.validateWeavingMetamodelSemantic(weavingMetaModel);
	        	WeaverResourceUtil.registerResource(weavingMetaModel);

	            			
	        }
	    } catch (IOException e) {
	        e.printStackTrace();
	        throw new WeaverException(IStatus.ERROR,WeaverPlugin.getPlugin().getString("_UI_manager_metamodel_error"),e);
	    }
	    return this.weavingMetamodelRS;
	}	
	
	
	/**
	 * Returns weaving metamodel resource
	 * @return Resource
	 * @throws WeaverException
	 */
	public Resource getWeavingMetamodel() throws WeaverException {
		return (Resource) this.getWeavingMetamodelRS( metamodelFile, metamodelType ).getResources().get(0);
	}	
	/**
	 * @param file
	 * @return
	 * @throws WeaverException
	 */
	private Resource setResourceFile(IFile file) throws WeaverException {
		URI uri = URI.createPlatformResourceURI(file.getFullPath().toString());
		ResourceSet rs = new ResourceSetImpl();
		
        rs.setPackageRegistry(registry);
        rs.getResourceFactoryRegistry().getExtensionToFactoryMap().put("xmi", new XMIResourceFactoryImpl());	    	            

//		rs.setPackageRegistry(ASMEMFModel.getResourceSet().getPackageRegistry());
//		rs.setResourceFactoryRegistry(ASMEMFModel.getResourceSet().getResourceFactoryRegistry());			
		
        Resource res = rs.createResource(uri); 
        res.getContents().addAll(getWeavingMetamodel().getContents());
        return res;		
	}
	
	/**
	 * Saves the weaving metamodel in file (ecore format)
	 * @param file
	 * @param manager
	 * @param shell
	 * @throws WeaverException
	 */
	public void saveEcoreWeavingMetamodel(final IFile file, Shell shell) throws WeaverException{
				
        final Resource savedRes = setResourceFile(file);
        
		WorkspaceModifyOperation operation =
			new WorkspaceModifyOperation() {
				// This is the method that gets invoked when the operation runs.
				public void execute(IProgressMonitor monitor) throws CoreException {
					try {
						savedRes.save(Collections.EMPTY_MAP);
						getWeavingMetamodel().getContents().addAll(savedRes.getContents());
					} catch (IOException e) {
						throw new CoreException(
								new Status(IStatus.WARNING,
										WeaverPlugin.getPlugin().getString("pluginID"),
										IStatus.OK,
										e.getMessage(),
										e.getCause())
										);
					}
				}
			};

		try {
			new ProgressMonitorDialog(shell).run(true, false, operation);
		} 
		catch (Exception e) {
			throw new WeaverException(IStatus.WARNING,WeaverPlugin.getPlugin().getString("_UI_manager_save_metamodel_error"),e);
		}
	}

	
	
	/**
	 * Saves the weaving metamodel in file (km3 format)
	 * @param file
	 * @param manager
	 * @param shell
	 * @throws WeaverException
	 */
	public void saveKm3WeavingMetamodel(final IFile file, Shell shell) throws WeaverException{
////		WorkspaceModifyOperation operation =
////			new WorkspaceModifyOperation() {
////			// This is the method that gets invoked when the operation runs.
////			public void execute(IProgressMonitor monitor) throws CoreException {
////				if (metamodelType == WeaverConstant.KM3){
////					// get metamodel stream
////					InputStream abstractMetamodelStream = WeaverModelManager.class.getResourceAsStream(
////							weavingMetamodelURI.toFileString());             
////					// get extensions stream for selected extensions (extensions which have been load in initialization)
////					
////					List km3InputStream = WeaverResourceUtil.loadKm3ExtensionStreamList(
////							getPluginMetamodelExtension(),getWorkspaceMetamodelExtension());
////					InputStream km3ExtensionStream = WeaverResourceUtil.addListOfStreams(km3InputStream);
////					// add 2 streams
////					InputStream stream= WeaverResourceUtil.addStream(abstractMetamodelStream,km3ExtensionStream);
////					// create file
////					file.create(stream, true, monitor);						
////				} 
////				else {
////					//gets the resource and saves as a km3 file
////					AtlEMFModelHandler emfHandler = (AtlEMFModelHandler) AtlEMFModelHandler.getDefault(AtlModelHandler.AMH_EMF);
////					ASMModel metamodel;
////					try {
////						metamodel = ASMEMFModel.loadASMEMFModel(
////								"AMW", (ASMEMFModel)emfHandler.getMof(), getWeavingMetamodel(), null);
////						//get the km3 string from the resoruce
////						KM3Projector kp = KM3Projector.getKM3Projector();														
////						String km3 = kp.getStringFromKM3(kp.getKM3FromEcore(metamodel));
////						
////						String name = file.getName();
////						name = name.substring(0, name.length() - file.getFileExtension().length()) + "km3";
////						//saves teh file
////						if(km3 != null) {
////							IWorkspace wks = ResourcesPlugin.getWorkspace();
////							IWorkspaceRoot wksroot = wks.getRoot();
////							IFile fileCreated = wksroot.getFile(file.getFullPath().removeLastSegments(1).append(name));
////							InputStream stream = new ByteArrayInputStream(km3.getBytes()); 
////							
////							if (fileCreated.exists()) {
////								fileCreated.setContents(stream, true, true, null);
////							} else {
////								fileCreated.create(stream, true, null);
////							}
////							try {
////								stream.close();
////							} catch (IOException e1) {
////								e1.printStackTrace();
////							}
////						}		
////						
////					} catch (WeaverException e) {
////						e.printStackTrace();
////					} catch (Exception e) {
////						e.printStackTrace();
////					}					
////				}
////			}
//		};
//		
//		try {
//			new ProgressMonitorDialog(shell).run(true, false, operation);
//		} 
//		catch (Exception e) {
//			e.printStackTrace();
//			throw new WeaverException(IStatus.WARNING,WeaverPlugin.getPlugin().getString("_UI_manager_save_metamodel_error"),e);
//		}		
	}  
	
	
	// Weaving Model
	
	/**
	 * Returns the resource set of the weaving model
	 * @param editingDomain the editing domain
	 * @return ResourceSet
	 * @throws WeaverException
	 */
	public ResourceSet getWeavingModelRS(EditingDomain editingDomain) throws WeaverException{
		try {
			if (this.weavingModelRS==null) {		
				//the factories must be the same to be able to load model and metamodel
				editingDomain.getResourceSet().setPackageRegistry(weavingMetamodelRS.getPackageRegistry());
				editingDomain.getResourceSet().setResourceFactoryRegistry(weavingMetamodelRS.getResourceFactoryRegistry());
				this.weavingModelRS = editingDomain.getResourceSet();
				resourceSets.add(weavingModelRS);
				if (isNewModel){					
					editingDomain.createResource(weavingModelURI.toString());								
				}				
				else{					
					editingDomain.loadResource(weavingModelURI.toString());		
				}				
			}
		} catch (Exception e) {
			e.printStackTrace();
			   throw new WeaverException(IStatus.ERROR,WeaverPlugin.getPlugin().getString("_UI_manager_model_error"),e);
		}
		return this.weavingModelRS;	    
	}
	
	/**
	 * Returns the resource set of the weaving model or null
	 * @return ResourceSet
	 */
	public ResourceSet getWeavingModelRS() {
		return this.weavingModelRS;
	}	
	
	/**
	 * return weaving metamodel resource
	 * @param editingDomain the editing domain
	 * @return Resource
	 * @throws WeaverException
	 */	
	public Resource getWeavingModel(EditingDomain editingDomain) throws WeaverException {	    
		return this.getWeavingModelRS(editingDomain).getResource(weavingModelURI, false);
	}	
	
	/**
	 * Returns weaving metamodel resource
	 * @return Resource
	 */
	public Resource getWeavingModel() {	    
		return this.getWeavingModelRS().getResource(weavingModelURI, false);
	}	
	
	// Woven model
	
	/**
	 * Returns the ressource for the id
	 * @return Resource
	 */

	public Resource getWovenResource(URI uri) {
		ResourceSet rs = null;
		try {
			rs = WeaverResourceUtil.initializeResourceSet(resourceSets,uri);
		} catch (WeaverException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}
		
		return (Resource) rs.getResources().get(0); 
		//getWovenRS(id).getResource(uri,true);		
	}
	
	
	
	/**
	 * Returns the ressource for the id
	 * @return Resource
	 */
	public String getWovenResourceType(int id) {
		return (String) this.wovenResourcesTypes.get(id);
	}		
	
    // GETTER AND SETTER

	/**
	 * @return Returns the weavingModelURI.
	 */
	public URI getWeavingModelURI() {
		return weavingModelURI;
	}
	/**
	 * @param weavingModelURI The weavingModelURI to set.
	 */
	public void setWeavingModelURI(URI weavingModelURI) {
		this.weavingModelURI = weavingModelURI;
	}
	/**
	 * @return Returns the helper.
	 */
	public WeaverModelHelper getHelper() {
		return helper;
	}
	/**
	 * @return Returns the isNewModel.
	 */
	public boolean isNewModel() {
		return isNewModel;
	}
    /**
     * Returns the manager of extensions
     * @return WeaverExtensions
     */
    public WeaverExtensionManager getWeaverExtensions() {
        return weaverExtensions;
    }  
	/**
	 * @return Returns the wovenResources.
	 */
	public List getWovenResources() {
		return wovenResources;
	}
	/**
	 * @param wovenResources The wovenResources to set.
	 */
	public void setWovenResources(List wovenResources) {
		this.wovenResources = wovenResources;
	}
	/**
	 * Returns list with MODEL_TYPE for model and METAMODEL_TYPE for metamodel
	 * @return Returns the wovenResourcesTypes.
	 */
	public List getWovenResourcesTypes() {
		return wovenResourcesTypes;
	}
	/**
	 * Sets list with true for MODEL_TYPE and METAMODEL_TYPE for metamodel
	 * @param wovenResourcesTypes The wovenResourcesTypes to set.
	 */
	public void setWovenResourcesTypes(List wovenResourcesTypes) {
		this.wovenResourcesTypes = wovenResourcesTypes;
	}
	/**
	 * @param helper The helper to set.
	 */
	public void setHelper(WeaverModelHelper helper) {
		this.helper = helper;
	}
	/**
	 * @return Returns the pluginMetamodelExtension.
	 */
	public List getPluginMetamodelExtension() {
		return pluginMetamodelExtension;
	}
	/**
	 * @param pluginMetamodelExtension The pluginMetamodelExtension to set.
	 */
	public void setPluginMetamodelExtension(List pluginMetamodelExtension) {
		this.pluginMetamodelExtension = pluginMetamodelExtension;
	}
	/**
	 * @return Returns the workspaceMetamodelExtension.
	 */
	public List getWorkspaceMetamodelExtension() {
		return workspaceMetamodelExtension;
	}
	/**
	 * @param workspaceMetamodelExtension The workspaceMetamodelExtension to set.
	 */
	public void setWorkspaceMetamodelExtension(List workspaceMetamodelExtension) {
		this.workspaceMetamodelExtension = workspaceMetamodelExtension;
	}
	public void resetWeavingMetamodel (){
		weavingMetamodelRS = null;
	}

	public Resource getWovenMeta(URI uri) {
		return (Resource) wovenMetaRelation.get(uri);
	}
}
