/*******************************************************************************
 * Copyright (c) 2005 INRIA.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Marcos Didonet Del Fabro - initial API and implementation
 *    Guillaume Gueltas
 *******************************************************************************/


package org.eclipse.gmt.weaver.core;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.OutputKeys;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;

import org.eclipse.core.runtime.IStatus;
import org.eclipse.gmt.weaver.core.data.WovenModelDescriptor;
import org.eclipse.gmt.weaver.exception.WeaverException;
import org.eclipse.gmt.weaver.plugin.WeaverPlugin;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.Text;
import org.xml.sax.SAXException;

/**
 * This class create a XML properties file.
 * 
 * Creates the metadata file in the path of the prop file.
 * 
 * @since 1.0
 */
public class WeaverXMLMetadata{
	
	// List of WeaverModelDescritors
	private List wovenModels = new ArrayList();
	// Other descriptors
	private String weavingModelPR;
	private String weavingModelPanel;
	private String wModelName;
	private List metamodelExtension = new ArrayList(); // List of String
	private List localMetamodelExtension = new ArrayList();
	
	private String weavingEcore;
	
	private File file;
	
	/**
	 * Creates a instance of XMLCOnfigFile
	 * @param aFile
	 * @param aFolder
	 * @throws IOException
	 */
	public WeaverXMLMetadata(File aFile) throws IOException{
		file=aFile;		
		File folder = file.getParentFile();
		if(!folder.exists())
			folder.mkdirs();
	}
	
	/**
	 * Creates the file 
	 * @param models
	 * @param weavingModelPR
	 * @param weavingModelPanel
	 * @param wModelName
	 * @param propertyExtension
	 * @param itemProviderExtension
	 * @throws IOException
	 * @throws TransformerException 
	 * @throws ParserConfigurationException 
	 */
	public void createXMLConfigFile(List models,
			String weavingModelPR,
			String weavingModelPanel,
			String wModelName,
			List metamodelExtension,
			List localMetamodelExtension, String weavingEcore) throws ParserConfigurationException, TransformerException{
		// Init of properties
		this.wovenModels = models;
		this.weavingModelPanel=weavingModelPanel;
		this.weavingModelPR=weavingModelPR;
		this.wModelName=wModelName;
		this.metamodelExtension = metamodelExtension;
		this.localMetamodelExtension=localMetamodelExtension;
		this.weavingEcore = weavingEcore;
		
		// Create XML
		this.createXML();
	}
	
	/**
	 * Creates the XML File (Write)
	 * @throws ParserConfigurationException 
	 * @throws TransformerException 
	 * @throws IOException
	 */
	private void createXML() throws ParserConfigurationException, TransformerException{
		//Document document = new XmlDocument();
		DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
		DocumentBuilder builder = factory.newDocumentBuilder();
		Document document = builder.newDocument();  // Create from whole cloth
		
		Element root   = document.createElement(WeaverPlugin.getPlugin().getString("_XML_element_base"));
		document.appendChild(root);
		
		// weaving
		Element weaving = document.createElement(WeaverPlugin.getPlugin().getString("_XML_element_weaving_base"));
		Element weavingRP = document.createElement(WeaverPlugin.getPlugin().getString("_XML_element_weaving_0"));
		Element weavingPanel = document.createElement(WeaverPlugin.getPlugin().getString("_XML_element_weaving_1"));
		Element wmodel = document.createElement(WeaverPlugin.getPlugin().getString("_XML_element_weaving_2"));
		root.appendChild(weaving);
		weaving.appendChild(weavingRP);
		weaving.appendChild(weavingPanel);
		weaving.appendChild(wmodel);
		Text weavingRPT  = document.createTextNode(this.getWeavingModelPR());
		weavingRP.appendChild(weavingRPT);
		Text weavingPanelT  = document.createTextNode(this.getWeavingModelPanel());
		weavingPanel.appendChild(weavingPanelT);
		Text wmodelT  = document.createTextNode(this.getWModelName());
		wmodel.appendChild(wmodelT);		
		
		// metamodel extension
		Element metamodel = document.createElement(WeaverPlugin.getPlugin().getString("_XML_element_metamodel_base"));
		root.appendChild(metamodel);
		
		Element pluginMetamodel = document.createElement(WeaverPlugin.getPlugin().getString("_XML_element_metamodel_0"));
		metamodel.appendChild(pluginMetamodel);
		Element workspaceMetamodel = document.createElement(WeaverPlugin.getPlugin().getString("_XML_element_metamodel_1"));
		metamodel.appendChild(workspaceMetamodel);
						
		Iterator iter1 = metamodelExtension.iterator();
		while(iter1.hasNext()){
			String file = (String) iter1.next();
			Element metamodelRp = document.createElement(WeaverPlugin.getPlugin().getString("_XML_element_metamodel_2"));
			pluginMetamodel.appendChild(metamodelRp);
			Text metamodelRpT  = document.createTextNode(file);
			metamodelRp.appendChild(metamodelRpT);
		}
		
		Iterator iter = localMetamodelExtension.iterator();
		while(iter.hasNext()){
			String file = (String) iter.next();
			Element metamodelRp = document.createElement(WeaverPlugin.getPlugin().getString("_XML_element_metamodel_2"));
			workspaceMetamodel.appendChild(metamodelRp);
			Text metamodelRpT  = document.createTextNode(file);
			metamodelRp.appendChild(metamodelRpT);
		}

		//ecore files
		Element completeMetamodel = document.createElement(WeaverPlugin.getPlugin().getString("_XML_element_metamodel_complete"));
		metamodel.appendChild(completeMetamodel);		
		Text weavingEcoreT  = document.createTextNode(this.getWeavingEcore());
		completeMetamodel.appendChild(weavingEcoreT);
		
		// woven model
		Iterator iter2 = wovenModels.iterator();
		while(iter2.hasNext()){
				
			WovenModelDescriptor desc = (WovenModelDescriptor) iter2.next();
			
			Element woven  = document.createElement(WeaverPlugin.getPlugin().getString("_XML_element_woven_base"));
			woven.setAttribute(WeaverPlugin.getPlugin().getString("_XML_element_woven_attribute_0"),desc.getId());
			Element wovenMFile = document.createElement(WeaverPlugin.getPlugin().getString("_XML_element_woven_0"));
			Element wovenMMFile = document.createElement(WeaverPlugin.getPlugin().getString("_XML_element_woven_1"));
			Element wovenReference = document.createElement(WeaverPlugin.getPlugin().getString("_XML_element_woven_5"));
			Element wovenModelName = document.createElement(WeaverPlugin.getPlugin().getString("_XML_element_woven_2"));
			Element wovenPanel = document.createElement(WeaverPlugin.getPlugin().getString("_XML_element_woven_3"));
			Element wovenModelRef = document.createElement(WeaverPlugin.getPlugin().getString("_XML_element_woven_4"));
			root.appendChild(woven);
			woven.appendChild(wovenMFile);
			woven.appendChild(wovenMMFile);
			woven.appendChild(wovenReference);
			woven.appendChild(wovenModelName);
			woven.appendChild(wovenPanel);
			woven.appendChild(wovenModelRef);
			Text wovenMFileT  = document.createTextNode(desc.getModelFile());
			wovenMFile.appendChild(wovenMFileT);
			Text wovenMMFileT  = document.createTextNode(desc.getMetamodelFile());
			wovenMMFile.appendChild(wovenMMFileT);			
			Text wovenRefT  = document.createTextNode(desc.getReferenceName());
			wovenReference.appendChild(wovenRefT);			
			Text wovenModelNameT  = document.createTextNode(desc.getNameProperty());
			wovenModelName.appendChild(wovenModelNameT);
			Text wovenPanelT  = document.createTextNode(desc.getPanel());
			wovenPanel.appendChild(wovenPanelT);		
			Text wovenModelRefT  = document.createTextNode(desc.getWModelRef());
			wovenModelRef.appendChild(wovenModelRefT);	
		}	
		// write
		/*FileOutputStream fos = new FileOutputStream(file);
		fos.flush();
		((XmlDocument)document).write(fos);
		fos.close();*/
		
		TransformerFactory tf = TransformerFactory.newInstance();
		// set all necessary features for your transformer -> see OutputKeys
		Transformer t = tf.newTransformer();
		t.setOutputProperty(OutputKeys.INDENT, "yes");
		t.transform(new DOMSource(document), new StreamResult(file));
	}
	
	/**
	 * Loads XML file (Parse)
	 * @throws ParserConfigurationException
	 * @throws SAXException
	 * @throws IOException
	 * @throws WeaverException 
	 */
	public void loadXMLConfigFile() throws ParserConfigurationException, SAXException, IOException, WeaverException{
        DocumentBuilderFactory factory = 
                                DocumentBuilderFactory.newInstance();
        DocumentBuilder builder = factory.newDocumentBuilder();
        if (!file.exists())
        	throw new WeaverException(IStatus.ERROR,WeaverPlugin.getPlugin().getString("_UI_config_file_not_found_exception"));
        FileInputStream fis = new FileInputStream(file);
        Document doc = builder.parse(fis);

        // parsing
        Element weaver = doc.getDocumentElement();
        int i = 1;
        
        // weaving
        Node weaving = weaver.getChildNodes().item(i);
        i+=2;
        
        Node weavingPath = weaving.getChildNodes().item(1);
        Text weavingPathT = (Text)weavingPath.getChildNodes().item(0);
        this.weavingModelPR=weavingPathT.getData();
        
        Node weavingPanel = weaving.getChildNodes().item(3);
        Text weavingPanelT = (Text)weavingPanel.getChildNodes().item(0);
        this.weavingModelPanel=weavingPanelT.getData();
        
        Node wmodel = weaving.getChildNodes().item(5);
        Text wmodelT = (Text)wmodel.getChildNodes().item(0);
        this.wModelName=wmodelT.getData();
        
        // weaving metamodel extension
        Node metamodel = weaver.getChildNodes().item(i);
        i+=2;
                        
        Node pluginMetamodel = metamodel.getChildNodes().item(1);
        int size = pluginMetamodel.getChildNodes().getLength();
        for (int a=1;a<size;a+=2){
	        Node metamodelRp = pluginMetamodel.getChildNodes().item(a);
	        Text metamodelRpT = (Text)metamodelRp.getChildNodes().item(0);
	        this.metamodelExtension.add(metamodelRpT.getData());
        }  
        Node pworkspaceMetamodel = metamodel.getChildNodes().item(3);
        size = pworkspaceMetamodel.getChildNodes().getLength();
        for (int a=1;a<size;a+=2){
	        Node metamodelRp = pworkspaceMetamodel.getChildNodes().item(a);
	        Text metamodelRpT = (Text)metamodelRp.getChildNodes().item(0);
	        this.localMetamodelExtension.add(metamodelRpT.getData());
        }        
        
        Node ecoreMetamodel = metamodel.getChildNodes().item(5);
        Text weavingEcoreT = (Text)ecoreMetamodel.getChildNodes().item(0);
        if (weavingEcoreT == null)
        	weavingEcore = "";
        else
        	weavingEcore = weavingEcoreT.getData();
        
        // woven models
        int lenght = weaver.getChildNodes().getLength();
        int j;
        for (j=i;j<lenght;j+=2){
        	
        	WovenModelDescriptor desc = new WovenModelDescriptor();
	        Node woven = weaver.getChildNodes().item(j);
	        desc.setId(woven.getAttributes().item(0).getNodeValue());
	        
	        Node wovenMFile = woven.getChildNodes().item(1);
	        Text wovenMFileT = (Text)wovenMFile.getChildNodes().item(0);
	        desc.setModelFile(wovenMFileT.getData());	        	        	       
	        
	        Node wovenMMFile = woven.getChildNodes().item(3);
	        Text wovenMMFileT = (Text)wovenMMFile.getChildNodes().item(0);
	        desc.setMetamodelFile(wovenMMFileT.getData());	        
	        
	        Node wovenRef = woven.getChildNodes().item(5);
	        Text wovenRefT = (Text)wovenRef.getChildNodes().item(0);
	        desc.setReferenceName(wovenRefT.getData());
	        
	        Node wovenTitle = woven.getChildNodes().item(7);
	        Text wovenTitleT = (Text)wovenTitle.getChildNodes().item(0);
	        desc.setNameProperty(wovenTitleT.getData());
	        
	        Node wovenPanel = woven.getChildNodes().item(9);
	        Text wovenPanelT = (Text)wovenPanel.getChildNodes().item(0);
	        desc.setPanel(wovenPanelT.getData());
	        
	        Node wovenModelRef = woven.getChildNodes().item(11);
	        Text wovenModelRefT = (Text)wovenModelRef.getChildNodes().item(0);
	        desc.setWModelRef(wovenModelRefT.getData());
	        
	        this.addWovenModelDescriptor(desc);
        }
        i=j+2;
        
	}
	
	
	/**
	 * Returns the number of model/metamodel
	 * @return
	 */
	public int getModelsize(){
		return wovenModels.size();
	}	
	/**
	 * Adds a model/metamodel descriptor
	 * @param desc
	 */
	public void addWovenModelDescriptor(WovenModelDescriptor desc){
		this.wovenModels.add(desc);
	}
	/**
	 * Gets the model/metamodel descriptor
	 * @param desc
	 */
	public WovenModelDescriptor getWovenModelDescriptor(int id){
		return (WovenModelDescriptor) this.wovenModels.get(id);
	}
	/**
	 * Returns list of model/metamodel descriptors
	 * @return Returns the wovenModels.
	 */
	public List getWovenModels() {
		return wovenModels;
	}
	/**
	 * Returns the weaving panel extension id
	 * @return Returns the weavingModelPanel.
	 */
	public String getWeavingModelPanel() {
		return weavingModelPanel;
	}
	/**
	 * Returns the weaving relative path 
	 * @return Returns the weavingModelPR.
	 */
	public String getWeavingModelPR() {
		return weavingModelPR;
	}
	/**
	 * Returns the WModel
	 * @return Returns the wModelName.
	 */
	public String getWModelName() {
		return wModelName;
	}
	/**
	 * Returns the list of metamodel extensions id
	 * @return Returns the metamodelExtension.
	 */
	public List getMetamodelExtension() {
		return metamodelExtension;
	}
	/**
	 * Returns the list of metamodel extensions id
	 * @return Returns the metamodelExtension.
	 */
	public List getLocalMetamodelExtension() {
		return localMetamodelExtension;
	}
	/**
	 * @return Returns the file.
	 */
	public File getFile() {
		return file;
	}
	/**
	 * the if the weaving is saved as an ecore model, not KM3
	 * @return
	 */
	public String getWeavingEcore() {
		return weavingEcore;
	}

	public void setWeavingEcore(String weavingEcore) {
		this.weavingEcore = weavingEcore;
	}

}