/*******************************************************************************
 * Copyright (c) 2005 INRIA.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Marcos Didonet Del Fabro - initial API and implementation
 *    Guillaume Gueltas
 *******************************************************************************/


package org.eclipse.gmt.weaver.core.data;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.core.runtime.IExtension;


/**
 * Descriptor of an extension point. Contains this extension point ID (an unique Integer pre defined) 
 * and a list of the extensions which use this extension point. 
 * They will be used in the weaver application plug-ins (IExtension). 
 * @since 1.0
 */
public class WeaverExtensionPointDescriptor {
    
    private String extensionPointID;
    private int extensionLocalIdentifier;
    // the selected extension 
    private List selectedExtensions = new ArrayList();
    
    /**
     * Creates a descriptor of an extension point
     * @param extensionLocalIdentifier
     * @param extensionPointID
     */
    public WeaverExtensionPointDescriptor(int extensionLocalIdentifier, String extensionPointID){
        this.extensionLocalIdentifier = extensionLocalIdentifier;
        this.extensionPointID = extensionPointID;
    }    
    
    /**
     * Gets extension
     * @param extension the extension unique identifier defined in the xml file
     * @return the extension descriptor
     * @see IExtension
     */
    private IExtension getExtension(IExtension extension){
        if (selectedExtensions.contains(extension))
            return extension; 
        return null;
    }        
    /**
     * adds a new extension object into the selected extensions list
     * @param extension the extension selected
     */
    public void addSelectedExtension (IExtension extension){
        if (getExtension(extension) == null)
        	selectedExtensions.add(extension);
    }
    /**
     * removes the extension from the selected extensions list
     * @param uniqueIdentifier
     */
    public void removeSelectedExtension (IExtension extension){
        if (selectedExtensions.contains(extension))
        	selectedExtensions.remove(extension);
    }

    /**
     * Gets local identifier
     * @return extensionLocalIdentifier
     */
    public int getExtensionLocalIdentifier() {
        return extensionLocalIdentifier;
    }
    /**
     * Gets identifier for extension point
     * @return extensionPointID
     */
    public String getExtensionPointID() {
        return extensionPointID;
    }
    /**
     * gets selected extensions
     * @return selectedExtensions
     */
    public List getSelectedExtensions() {
        return selectedExtensions;
    }
}