/*******************************************************************************
 * Copyright (c) 2005 INRIA.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Marcos Didonet Del Fabro - initial API and implementation
 *    Guillaume Gueltas
 *******************************************************************************/


package org.eclipse.gmt.weaver.core.data;

import org.eclipse.emf.common.util.URI;

/**
 * This class contents data configuration of the model/metamodel
 * @since 1.0
 */
public class WovenModelDescriptor {
	
	private String id=""; // From 0 to n
	private String modelFile="";
	private String metamodelFile="";
	private String referenceName="";
	private String nameProperty="";
	private String panel="";
	private String wModelRef="";

	/**
	 * Return true if we work with models, false if we work with metamodels
	 * @return true or false
	 */
	public boolean isModelExists(){
		return (!modelFile.equals("null"));
	}
	
	/**
	 * Returns the model file path
	 * @return Returns the modelfile.
	 */
	public String getModelFile() {
		return modelFile;
	}
	/**
	 * Sets the model file path
	 * @param modelfile The modelfile to set.
	 */
	public void setModelFile(String file) {
		this.modelFile = file;
	}
	/**
	 * Returns the metamodel file path 
	 * @return Returns the metamodelFile.
	 */
	public String getMetamodelFile() {
		return metamodelFile;
	}
	/**
	 * Sets the metamodel file path
	 * @param metamodelFile The metamodelFile to set.
	 */
	public void setMetamodelFile(String metamodelFile) {
		this.metamodelFile = metamodelFile;
	}	
	/**
	 * Returns the id of the descriptor
	 * @return Returns the id.
	 */
	public String getId() {
		return id;
	}
	/**
	 * Sets the id of the descriptor
	 * @param id The id to set.
	 */
	public void setId(String id) {
		this.id = id;
	}
	/**
	 * Returns the name of the model/metamodel
	 * @return Returns the name.
	 */
	public String getReferenceName() {
		return referenceName;
	}
	/**
	 * Set the model/metamodel name
	 * @param name The name to set.
	 */
	public void setReferenceName(String name) {
		this.referenceName = name;
	}
	/**
	 * Returns the woven panel extension id
	 * @return Returns the panel.
	 */
	public String getPanel() {
		return panel;
	}
	/**
	 * Set the woven panel extension id
	 * @param panel The panel to set.
	 */
	public void setPanel(String panel) {
		this.panel = panel;
	}

	/**
	 * @return Returns the wModelRef.
	 */
	public String getWModelRef() {
		return wModelRef;
	}
	/**
	 * @param modelRef The wModelRef to set.
	 */
	public void setWModelRef(String modelRef) {
		wModelRef = modelRef;
	}
	
	/**
	 * @return Returns the woven model name.
	 */
	public String getNameProperty() {
		return nameProperty;
	}
	/**
	 * @param title The title to set.
	 */
	public void setNameProperty(String title) {
		this.nameProperty = title;
	}	
	/**
	 * converts a string into a file uri or an http uri
	 * @param string uri
	 * @return
	 */
	private URI getURIFromString (String uri){
		if (uri.indexOf("http") == -1)
			return URI.createPlatformResourceURI(uri);
		else
			return URI.createURI(uri);			
	}
	/**
	 * returns the URI of the given model or metamodel
	 * @return
	 */
	public URI getURI (){
		if (isModelExists()) 
			return getURIFromString(getModelFile());
		else
			return getURIFromString(getMetamodelFile());	
	}
}
