/*******************************************************************************
 * Copyright (c) 2005 INRIA.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Marcos Didonet Del Fabro - initial API and implementation
 *    Guillaume Gueltas
 *******************************************************************************/


package org.eclipse.gmt.weaver.editors;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.EventObject;
import java.util.Iterator;
import java.util.List;

import javax.xml.parsers.ParserConfigurationException;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.NullProgressMonitor;
import org.eclipse.core.runtime.QualifiedName;
import org.eclipse.emf.common.command.BasicCommandStack;
import org.eclipse.emf.common.command.CommandStackListener;
import org.eclipse.emf.common.notify.AdapterFactory;
import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.edit.domain.AdapterFactoryEditingDomain;
import org.eclipse.emf.edit.domain.EditingDomain;
import org.eclipse.emf.edit.domain.IEditingDomainProvider;
import org.eclipse.emf.edit.provider.ComposedAdapterFactory;
import org.eclipse.emf.edit.provider.resource.ResourceItemProviderAdapterFactory;
import org.eclipse.emf.edit.ui.provider.AdapterFactoryContentProvider;
import org.eclipse.gmt.weaver.core.WeaverModelManager;
import org.eclipse.gmt.weaver.core.WeaverXMLMetadata;
import org.eclipse.gmt.weaver.core.data.WovenModelDescriptor;
import org.eclipse.gmt.weaver.editors.interfaces.ITreeViewerSelection;
import org.eclipse.gmt.weaver.editors.interfaces.IWSelectionChangedListener;
import org.eclipse.gmt.weaver.editors.interfaces.IWeaverInitializer;
import org.eclipse.gmt.weaver.editors.interfaces.IWeavingPanel;
import org.eclipse.gmt.weaver.editors.interfaces.IWovenModelPanel;
import org.eclipse.gmt.weaver.exception.WeaverException;
import org.eclipse.gmt.weaver.plugin.WeaverConstant;
import org.eclipse.gmt.weaver.plugin.WeaverPlugin;
import org.eclipse.gmt.weaver.plugin.util.WeaverResourceUtil;
import org.eclipse.gmt.weaver.plugin.util.WeaverUtil;
import org.eclipse.gmt.weaver.providers.WeaverAdapterFactory;
import org.eclipse.jface.dialogs.InputDialog;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.dialogs.ProgressMonitorDialog;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.ISelectionProvider;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.CTabFolder;
import org.eclipse.swt.custom.SashForm;
import org.eclipse.swt.events.ControlAdapter;
import org.eclipse.swt.events.ControlEvent;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.ui.IActionBars;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.IEditorSite;
import org.eclipse.ui.IFileEditorInput;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.actions.WorkspaceModifyOperation;
import org.eclipse.ui.dialogs.SaveAsDialog;
import org.eclipse.ui.internal.ide.IDEWorkbenchMessages;
import org.eclipse.ui.part.FileEditorInput;
import org.eclipse.ui.part.MultiPageEditorPart;
import org.eclipse.ui.views.properties.IPropertySheetPage;
import org.eclipse.ui.views.properties.PropertySheetPage;
import org.xml.sax.SAXException;

/**
 * This class is the editor of the  model weaver 
 * @since 1.0
 */
public class GlobalWeaverEditor extends MultiPageEditorPart implements
	IEditingDomainProvider, ISelectionProvider {
	
	// Instance of model manager (see init())
	private WeaverModelManager modelManager;
    private IWeavingPanel weaverEditorPanel;
    private List wovenModelPanels = new ArrayList();
    
    // editingDomain
    private ComposedAdapterFactory adapterFactory;
    private AdapterFactoryEditingDomain editingDomain;
    
    // xml config file
    private WeaverXMLMetadata xmlFile;
    
    // property sheet
    private PropertySheetPage propertySheetPage;
   
	/**
	 * This creates a weaver editor
	 */
	public GlobalWeaverEditor() {
		super();
	}
	
	/**
	 * Creates the page of the editor.
	 */
	protected void createPages() {		
	    try{
	    	
	    	Composite container = getContainer();
			final SashForm topSashForm = new SashForm(container,SWT.HORIZONTAL);

			String wModelName = null;
			List wModelRefClassName = new ArrayList();		
			List wReferenceNames = new ArrayList();
			List wModelRefNameProperty = new ArrayList();
			List wModelPath = new ArrayList();
			
		    for(int i = 0; i < xmlFile.getModelsize(); i++){
		    	WovenModelDescriptor desc = xmlFile.getWovenModelDescriptor(i);
		    	// Metamodel or Model/Metamodel
				//boolean modelExist = (desc.getModelFile().equals("null")) ? false : true;
		    	
		    	// Woven Model Panel
			    Object modelPanel = modelManager.getWeaverExtensions().getWovenPanelImplementationClass(desc.getPanel());
			    IWovenModelPanel modelViewerPanel = ((IWovenModelPanel)modelPanel);
			    
			    modelViewerPanel.createControl(topSashForm);
			    modelViewerPanel.setTitle(desc.getNameProperty(), WeaverPlugin.getPlugin().getImage("MLogo"));
			    Resource wovenResource = null;
			    			    
			    wovenResource = (Resource) modelManager.getWovenResource(desc.getURI());
				modelViewerPanel.setInput(wovenResource);
				// this operation is only made for models (this is the opposite to weaving metamodels)
				//if(modelManager.getWovenResourceType(i).equals(WeaverConstant.MODEL_TYPE)){
				wovenResource = modelViewerPanel.initializeModelResource(wovenResource,xmlFile, desc.getId() );
				//}
			    
			    wovenModelPanels.add(modelViewerPanel);
			    wModelRefClassName.add(desc.getWModelRef());
			    wReferenceNames.add(desc.getReferenceName());
			    wModelRefNameProperty.add(desc.getNameProperty());
			    //if (modelExist)//takes the path of the models to save it int othe ref value
			    wModelPath.add(WeaverUtil.getIdentifierAdapter(wovenResource).getID().toString());
			    //else  //	wModelPath.add(desc.getMetamodelFile());			    
			    
			    // to edit properties of woven model
			   ((TreeViewer)modelViewerPanel.getViewer()).addSelectionChangedListener( new ISelectionChangedListener(){
	
			    	public void selectionChanged(SelectionChangedEvent event) {
			    	        if ( getCreatePropertySheetPage().getControl() != null){
			    	        	if (! getCreatePropertySheetPage().getControl().isDisposed()) {
			    	        		AdapterFactory newFactory = ((IWovenModelPanel)getGlobalWeaverEditor().getWeaverModelPanels().get(0)).getAdapterFactory();
			    	        		getCreatePropertySheetPage().setPropertySourceProvider(new AdapterFactoryContentProvider(newFactory));
			    	        		getCreatePropertySheetPage().selectionChanged(getGlobalWeaverEditor(), 
			    	        				event.getSelection());			    	        	
			    	        	}
			    	        }
			    	}} 
			    	    );   	    			    
			    // Weaving Model Panel
			    if (i==0){
			    	
				    WeaverAdapterFactory wElementProviderAdapterFactory = 
		            	new WeaverAdapterFactory(modelManager.getWeaverExtensions());
			        List factories = new ArrayList();
			        factories.add(new ResourceItemProviderAdapterFactory());
			        factories.add(wElementProviderAdapterFactory);
			        adapterFactory = new ComposedAdapterFactory(factories);
			        wElementProviderAdapterFactory.setComposedFactory(adapterFactory);			        
			        
			        // initialize weaving metamodel
			        //fixed code to be able to choose the metamodel as xmi or km3 in the wizards
			        //modelManager.resetWeavingMetamodel();			        			        
			        //Resource WMM = (Resource)modelManager.getWeavingMetamodelRS(
			        	//"C:/Documents and Settings/didonet/Mes documents/workspace_transfos/Match/metamodels/test.ecore",
			        		//WeaverConstant.XMI).getResources().get(0);
			        editingDomain = createEditingDomain(adapterFactory);
			        Resource WMM = modelManager.getWeavingMetamodel();			       
			        WeaverResourceUtil.initResource(WMM);			        
			        			        
			        modelManager.getWeaverExtensions().getAllProviderImplementationIcon();
			        
			        // initialize editing domain and resource			        
			        modelManager.getWeavingModel(editingDomain);
			        
				    Object weavingPanel = modelManager.getWeaverExtensions().getWeavingPanelImplementationClass(xmlFile.getWeavingModelPanel());
				    weaverEditorPanel = (IWeavingPanel) weavingPanel;
				    weaverEditorPanel.init(this);
				    weaverEditorPanel.createControl(topSashForm);
				    weaverEditorPanel.setTitle(WeaverPlugin.getPlugin().getString("_UI_weaving_model"), WeaverPlugin.getPlugin().getImage("WLogo"));
				    weaverEditorPanel.setInput(editingDomain.getResourceSet()); 
				    wModelName = xmlFile.getWModelName();
				    // to edit properties of weaving model 
				    weaverEditorPanel.getViewer().addSelectionChangedListener(
				    		new ISelectionChangedListener(){
				    			public void selectionChanged(SelectionChangedEvent event) {
				    				AdapterFactory newFactory = getGlobalWeaverEditor().getAdapterFactory();
				    				getCreatePropertySheetPage().setPropertySourceProvider(new AdapterFactoryContentProvider(newFactory));
				    				getCreatePropertySheetPage().selectionChanged(getGlobalWeaverEditor(), 
				    						event.getSelection());
				    			}
				    		}

				    );
			    }		
			    weaverEditorPanel.addWeaverSelectionChangedListener((IWSelectionChangedListener) modelViewerPanel);
			    modelViewerPanel.initializeModelManager(this.getModelManager());
			    modelViewerPanel.initializeEditingDomain(this.getEditingDomain());
			    modelViewerPanel.getViewer().addSelectionChangedListener(
			    		new ISelectionChangedListener(){
							public void selectionChanged(SelectionChangedEvent event) {
								wovenPanelSelectionChanged(event);		
							}
			    		}
			    );
		    }
		    // WModel Eclass choosen in wizard
		    EClass wModel = WeaverUtil.getEClassByName(modelManager.getWeavingMetamodel(),wModelName);
		    if (wModel != null){
			    for (Iterator i = modelManager.getWeaverExtensions().getInitializingClasses().iterator(); i.hasNext();){
			        IWeaverInitializer extInit = (IWeaverInitializer)i.next();
			        extInit.initWeaverMetamodel(modelManager,this.getEditingDomain(),
			        		wModelName, wModelRefClassName, wReferenceNames, 
			        		wModelRefNameProperty, wModelPath, 
			        		modelManager.isNewModel());	        	
			    }		    	
		    }		    	   	
		    else{// if there is no WModel, asks for a root element to be created
		    	InputDialog setRootElement = new InputDialog(getSite().getShell(),
		    			WeaverPlugin.getPlugin().getString("_UI_Type_root_element"),
		    			WeaverPlugin.getPlugin().getString("_UI_Root_element"),"",null);
		    	setRootElement.open();
		    	if (! setRootElement.getValue().equals(""))
		    		modelManager.helper.createWeavingElementInRes(setRootElement.getValue(), editingDomain);
		    }				    		   		    
		    
	    	this.saveResource();//must save always because it updates the ref attribute of the WModelRef elements
        	this.refreshState();        	
        	WeaverResourceUtil.refreshResourceFolder(this.getModelManager().getWeavingModel());
		    
		    int pageIndex = addPage(topSashForm);
		    setPageText(pageIndex, WeaverPlugin.getPlugin().getString("_UI_weaver_page"));	
		    setActivePage(pageIndex);		
		    		    		    
		}
	    catch (Exception e) {
	    	MessageDialog.openError(this.getSite().getShell(), WeaverPlugin.getPlugin().getString("_UI_error_label"), e.getMessage());
	    	e.printStackTrace();
	    	//return;
		}
	    
	    getContainer().addControlListener
	    (new ControlAdapter()
	    		{
	    	boolean guard = false;
	    	public void controlResized(ControlEvent event){
	    		if (!guard){
	    			guard = true;
	    			hideTabs();
	    			guard = false;
	    		}}
	    		});			    
	}

	public EObject getModelRef(String className,String propertyName, String propertyValue){
	        Resource res = this.modelManager.getWeavingModel();
	        for (Iterator i = res.getAllContents(); i.hasNext(); ){
	            Object obj = i.next();
	            if (obj instanceof EObject){
	                EObject eObj = (EObject)obj;
	                Resource mm=null;
	                try{
	                	mm = this.modelManager.getWeavingMetamodel();
	                }catch(Exception e) {}
	                if (WeaverUtil.isInstanceOf(eObj, className,mm)) {
	                    String value =  (String)eObj.eGet( eObj.eClass(). getEStructuralFeature(propertyName));
	                    if (value != null)
	                        if (value.equals(propertyValue))
	                            return eObj;                                
	                }                
	            }
	        }
	        return null;        
	    }  	

	/**
	 * If there is just one page in the multi-page editor part, this hides
	 * the single tab at the bottom.
	 */
	protected void hideTabs() {
		if (getPageCount() <= 1) {
			setPageText(0, "");
			if (getContainer() instanceof CTabFolder) {
				((CTabFolder) getContainer()).setTabHeight(1);
				Point point = getContainer().getSize();
				getContainer().setSize(point.x, point.y + 6);
			}
		}
	}

    /**
     * Creates the editing domain. Its is responsible for the command stack and
     * creation of new objects
     * @param factory
     * @return the editing domain
     */
    public AdapterFactoryEditingDomain createEditingDomain(
            ComposedAdapterFactory factory) {
        // ResourceSet 
        BasicCommandStack commandStack = new BasicCommandStack();
        
        // Add a listener to set the most recent command's affected objects to
        // be the selection of the viewer with focus.
        commandStack.addCommandStackListener(new CommandStackListener() {
            public void commandStackChanged(final EventObject event) {
                getContainer().getDisplay()
                        .asyncExec(new Runnable() {
                            public void run() {
                                firePropertyChange(IEditorPart.PROP_DIRTY);
                                // Try to select the affected objects.
                                if (getCreatePropertySheetPage() != null) {
                                	getCreatePropertySheetPage().refresh();
                                    //used to refresh the selections menu
                                    //fakes a selection event
                                    getActionBarContributor().selectionChanged(
                                    		new SelectionChangedEvent(getWeaverEditorPanel().getViewer(),getSelection()));
                                }
                            }
                        });
            }
        });

        // Create the editing domain with a special command stack.
        editingDomain = new AdapterFactoryEditingDomain(factory, commandStack);
        return editingDomain;
    }	
	
	/**
	 * The <code>MultiPageEditorExample</code> implementation of this method
	 * checks that the input is an instance of <code>IFileEditorInput</code>.
	 */
	public void init(IEditorSite site, IEditorInput editorInput)
	throws PartInitException {
		if (!(editorInput instanceof IFileEditorInput))
			throw new PartInitException(WeaverPlugin.getPlugin().getString("_UI_wizard_part_exception_1"));
		super.init(site, editorInput);
		setPartName(editorInput.getName());
//		site.setSelectionProvider(this);
		//site.getPage().getWorkbenchWindow().getWorkbench().addWindowListener(windowListener);
		
		// Init of ModelManager
		IFile file = ((IFileEditorInput)editorInput).getFile();
		initModelManager(file);
	}

	/**
	 * Initialize the model manager for this input file
	 * @param file the editor input file
	 * @throws PartInitException
	 */
	private void initModelManager(IFile editorFile) throws PartInitException {
		File propertiesFile = getConfigFile(editorFile);
		
		try {			
			// Config file			
			xmlFile = new WeaverXMLMetadata(propertiesFile);		
			
			xmlFile.loadXMLConfigFile();
			
			// metadata persistent property for tell if new model
			IProject project = editorFile.getProject();
			QualifiedName qualifiedNameNewModelWM = new QualifiedName(editorFile.getFullPath().toString(),WeaverPlugin.getPlugin().getString("_UI_weaving_model"));
			boolean isNewModel = (new Boolean(project.getPersistentProperty(qualifiedNameNewModelWM))).booleanValue();
			project.setPersistentProperty(qualifiedNameNewModelWM, Boolean.toString(false));
			
			//initialize model Manager
			int metamodelType;
			if (xmlFile.getWeavingEcore().equals(""))				
				metamodelType = WeaverConstant.KM3;
			else
				metamodelType = WeaverConstant.XMI;
			modelManager = new WeaverModelManager(xmlFile,isNewModel,metamodelType);
		} catch (IOException e1) {
			e1.printStackTrace();
			throw new PartInitException(WeaverPlugin.getPlugin().getString("_UI_wizard_part_exception_1"),e1);
		} catch (ParserConfigurationException e) {
			throw new PartInitException(WeaverPlugin.getPlugin().getString("_UI_wizard_part_exception_2"),e);
		} catch (SAXException e) {
			e.printStackTrace();
			throw new PartInitException(WeaverPlugin.getPlugin().getString("_UI_wizard_part_exception_2"),e);
		}catch (WeaverException e) {
			if (e.getMessage().length() > 0)
				MessageDialog.openError(this.getSite().getShell(), WeaverPlugin.getPlugin().getString("_UI_error_label"), e.getMessage());
			e.printStackTrace();
			throw new PartInitException(WeaverPlugin.getPlugin().getString("_UI_wizard_part_exception_5"),e);		
		}catch (CoreException e) {
			e.printStackTrace();
			throw new PartInitException(WeaverPlugin.getPlugin().getString("_UI_wizard_part_exception_3"),e);
		}
	}
	
	/**
	 * get the config file (% input editor file)
	 * @param file the editor input file
	 */	
	public File getConfigFile(IFile file){
		
		//this method is also called in the wizard
		File propertiesFile = WeaverUtil.getPropertyFilePath(file).toFile();
		return propertiesFile;
	}
	
	
	/**
	 * Updates the xml config file (metadata properties file)
	 * @param file the config file
	 * @throws WeaverException
	 */
	public void updateXMLConfigFile(IFile file) throws WeaverException{
		File propertiesFile = getConfigFile(file);
		try {
			// Config file
			WeaverXMLMetadata xmlFileTemp = new WeaverXMLMetadata(propertiesFile);
			xmlFileTemp.createXMLConfigFile(xmlFile.getWovenModels(),
					file.getFullPath().toString(),
					xmlFile.getWeavingModelPanel(),
					xmlFile.getWModelName(),
					xmlFile.getMetamodelExtension(),
					xmlFile.getLocalMetamodelExtension(),
					xmlFile.getWeavingEcore());
			xmlFile = xmlFileTemp;
		} catch (Exception e1) {
			e1.printStackTrace();
			throw new WeaverException(IStatus.ERROR,WeaverPlugin.getPlugin().getString("_UI_wizard_write_error")+e1.getMessage());
		}
	}
    
	/**
	 * The <code>MultiPageEditorPart</code> implementation of this 
	 * <code>IWorkbenchPart</code> method disposes all nested editors.
	 * Subclasses may extend.
	 */
	public void dispose() {
		// dispose of Woven Panel
		Iterator iter = wovenModelPanels.iterator();
		while (iter.hasNext()){
			IWovenModelPanel wovenModelPanel = (IWovenModelPanel )iter.next();
			if(wovenModelPanel!=null){
				wovenModelPanel.dispose();
				weaverEditorPanel.removeWeaverSelectionChangedListener((IWSelectionChangedListener) wovenModelPanel);
			}
		}
		
		// dispose of Weaving Panel
		if (weaverEditorPanel!= null)weaverEditorPanel.dispose();
		
		// dispose of actionBarContribuor
		if (getActionBarContributor().getActiveEditor() == this) {
			getActionBarContributor().setActiveEditor(null);
		}
		
		// dispose of composedAdapterFactory
		if (adapterFactory!=null)
			adapterFactory.dispose();
		
		// dipose of propertysheet
		if(propertySheetPage != null) propertySheetPage.dispose();
		
		super.dispose();
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.ui.ISaveablePart#isDirty()
	 */
	public boolean isDirty() {
		if ((this.weaverEditorPanel !=null)&&(getEditingDomain() != null))
			return ((BasicCommandStack)getEditingDomain().getCommandStack()).isSaveNeeded();
		else
			return false;
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.ui.part.EditorPart#isSaveAsAllowed()
	 */
	public boolean isSaveAsAllowed() {
		return true;		
	}	
	
	/**
	 * Saves the editing domain resource (weaving model)
	 * @param resource
	 */
	protected void saveResource(){
		try {
			Resource savedResource = (Resource) getEditingDomain().getResourceSet().getResources().get(0);
			savedResource.save(Collections.EMPTY_MAP);
		}
		catch (Exception exception) {
			exception.printStackTrace();
		}
	}
	
	/**
	 * Refreshs the necessary state
	 */
	protected void refreshState(){
		this.isDirty();
		((BasicCommandStack)getEditingDomain().getCommandStack()).saveIsDone();
		firePropertyChange(IEditorPart.PROP_DIRTY);
	}	

	/* (non-Javadoc)
	 * @see org.eclipse.ui.ISaveablePart#doSave(org.eclipse.core.runtime.IProgressMonitor)
	 */
	public void doSave(IProgressMonitor progressMonitor) {
		WorkspaceModifyOperation operation =
			new WorkspaceModifyOperation() {
				// This is the method that gets invoked when the operation runs.
				public void execute(IProgressMonitor monitor) {
					// Save the resource to the file system.
					saveResource();
				}
			};

		try {
			// This runs the options, and shows progress.
			new ProgressMonitorDialog(getSite().getShell()).run(true, false, operation);

			// Refresh the necessary state.
			this.refreshState();

		}
		catch (Exception exception) {
			// Something went wrong that shouldn't.
			exception.printStackTrace();
		}		
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.ui.ISaveablePart#doSaveAs()
	 */
	public void doSaveAs() {
		SaveAsDialog saveAsDialog= new SaveAsDialog(getSite().getShell());
		saveAsDialog.open();
		IPath path= saveAsDialog.getResult();
		if (path != null) {
			// create file
			final IFile file = ResourcesPlugin.getWorkspace().getRoot().getFile(path);
			if (file != null) {
				this.doSaveAs(URI.createPlatformResourceURI(file.getFullPath().toString()), new FileEditorInput(file));

						IWorkbenchPage page = PlatformUI.getWorkbench().getActiveWorkbenchWindow().getActivePage();
						try {
							updateXMLConfigFile(file);
						} catch (WeaverException e) {
							e.printStackTrace();
							MessageDialog.openError(									
									page.getWorkbenchWindow().getShell(),
									IDEWorkbenchMessages.OpenWithMenu_dialogTitle, //$NON-NLS-1$
									e.getMessage());
						}	
			}
		}
	}

	/**
	 * Saves the input (resource) with this uri
	 * @param uri
	 * @param editorInput
	 */
	protected void doSaveAs(URI uri, IEditorInput editorInput) {
		((Resource) this.getEditingDomain().getResourceSet().getResources().get(0)).setURI(uri);
		modelManager.setWeavingModelURI(uri);
		setInput(editorInput);
		setPartName(editorInput.getName());
		IProgressMonitor progressMonitor =
			getActionBars().getStatusLineManager() != null ?
				getActionBars().getStatusLineManager().getProgressMonitor() :
				new NullProgressMonitor();
		this.doSave(progressMonitor);
	}	
	
	/* (non-Javadoc)
	 * @see org.eclipse.ui.IWorkbenchPart#setFocus()
	 */
	public void setFocus(){
		super.setFocus();
	}
	
    /* (non-Javadoc)
     * @see org.eclipse.core.runtime.IAdaptable#getAdapter(java.lang.Class)
     */
    public Object getAdapter(Class key) {
        if (key.equals(IPropertySheetPage.class)) {                        
            return  this.getCreatePropertySheetPage();
        }        
        else {
            return super.getAdapter(key);
        }
    }
    
    /* (non-Javadoc)
     * @see org.eclipse.ui.part.WorkbenchPart#firePropertyChange(int)
     */
    public void firePropertyChange(final int propertyId){
        super.firePropertyChange(propertyId);
    }
    
	/**
	 * Creates a new property sheet page
	 * @return the property sheet page
	 */
	public PropertySheetPage getCreatePropertySheetPage() {
	    if (propertySheetPage == null) { 
	        propertySheetPage = new PropertySheetPage();
	        propertySheetPage.setPropertySourceProvider(new AdapterFactoryContentProvider(this.adapterFactory));
	    }
	    return propertySheetPage;
	}   	
	
	/**
	 * Returns this instance of GlobalWeaverEditor
	 * @return
	 */
	public GlobalWeaverEditor getGlobalWeaverEditor(){
		return this;
	}	
        
	/**
	 * Returns the action bar contribuor (for global editor part)
	 * @return
	 */
	public GlobalWeaverEditorContributor getActionBarContributor() {
		return (GlobalWeaverEditorContributor) getEditorSite().getActionBarContributor();
	}
	
	/**
	 * Returns the actions bars (for global editor part)
	 * @return
	 */
	public IActionBars getActionBars() {
		return getActionBarContributor().getActionBars();
	}	    
    
    /* (non-Javadoc)
     * @see org.eclipse.emf.edit.domain.IEditingDomainProvider#getEditingDomain()
     */
    public EditingDomain getEditingDomain(){	      
    	return editingDomain;
    }

    /* (non-Javadoc)
     * @see org.eclipse.jface.viewers.ISelectionProvider#addSelectionChangedListener(org.eclipse.jface.viewers.ISelectionChangedListener)
     */
    public void addSelectionChangedListener(ISelectionChangedListener listener) {
    	getWeaverEditorPanel().getViewer().addSelectionChangedListener(listener);
    }

    /* (non-Javadoc)
     * @see org.eclipse.jface.viewers.ISelectionProvider#getSelection()
     */
    public ISelection getSelection() {
        return getWeaverEditorPanel().getViewer().getSelection();        
    }

    /* (non-Javadoc)
     * @see org.eclipse.jface.viewers.ISelectionProvider#removeSelectionChangedListener(org.eclipse.jface.viewers.ISelectionChangedListener)
     */
    public void removeSelectionChangedListener(ISelectionChangedListener listener) {
    	getWeaverEditorPanel().getViewer().removeSelectionChangedListener(listener);
    }

    /* (non-Javadoc)
     * @see org.eclipse.jface.viewers.ISelectionProvider#setSelection(org.eclipse.jface.viewers.ISelection)
     */
    public void setSelection(ISelection selection) {
    	getWeaverEditorPanel().getViewer().setSelection(selection);
    }
    
    /**
     * Returns the model manager
     * @return 
     */
    public WeaverModelManager getModelManager() {
    	return modelManager;
    }
	/**
	 * Returns the adapterFactory
	 * @return 
	 */
	public ComposedAdapterFactory getAdapterFactory() {
		return adapterFactory;
	}    
	/**
	 * @return Returns the weaverEditorPanel.
	 */
	public IWeavingPanel getWeaverEditorPanel() {
		return weaverEditorPanel;
	}
	/**
	 * @return Returns the weaverModelPanels.
	 */
	public List getWeaverModelPanels() {
		return wovenModelPanels;
	}
	/**
	 * @return Returns the xmlFile.
	 */
	public WeaverXMLMetadata getXmlFile() {
		return xmlFile;
	}
	//TODO verify this method - it has performance problems (maybe the selection is done several times)
	private void wovenPanelSelectionChanged (SelectionChangedEvent e){		
		Object source = ((StructuredSelection)e.getSelection()).getFirstElement();		
		if (source instanceof EObject){
			Resource res = null;
			EObject eSource = null; 
			List list = new ArrayList();
			res = modelManager.getWeavingModel();
			if (res != null) {
				for (Iterator i = res.getAllContents(); i.hasNext(); ){
					Object obj = i.next();
					if (obj instanceof EObject){
						EObject eObj = (EObject) obj;
						try {
							if (WeaverUtil.isInstanceOf(eObj,"WLinkEnd",modelManager.getWeavingMetamodel())){
								EObject elementRef = (EObject) eObj.eGet( WeaverUtil.getEStructuralFeature(eObj,"element") );
								String ID="";
								if (elementRef != null){
									ID = (String) elementRef.eGet( WeaverUtil.getEStructuralFeature(elementRef,"ref") );
									eSource = (EObject) source;
									//checks the model uri as well									
									String aURI = WeaverUtil.getIdentifierAdapter(eSource.eResource()).getID().toString();
									EObject modelRef = (EObject)elementRef.eGet( WeaverUtil.getEStructuralFeature(elementRef, WeaverConstant.MODEL_REF_FEATURE) );
									String modelID = (String) modelRef.eGet( WeaverUtil.getEStructuralFeature(elementRef,"ref") );
									//takes the adapters from the source
									if (modelID.equals(aURI)){												
										if (WeaverUtil.getIdentifierAdapter(eSource)==null){																					
											if ( ID.equals( (String)eSource.eResource().getURIFragment(eSource) ))
												list.add(eObj);
										}
										else
											if (ID.equals((String)WeaverUtil.getIdentifierAdapter(eSource).getID())){
												list.add(eObj);
											}
									}
								}									
							}
						} catch (WeaverException e1) {						
							e1.printStackTrace();
						}							
					}								
				}
				((ITreeViewerSelection)weaverEditorPanel.getViewer()).fireSelections(false);	
				if (list.size()>0)
					weaverEditorPanel.getViewer().setSelection(new StructuredSelection(list));
				else
					weaverEditorPanel.getViewer().setSelection(null);
				((ITreeViewerSelection)weaverEditorPanel.getViewer()).fireSelections(true);
			}
		}
		
	}
}
