/*******************************************************************************
 * Copyright (c) 2005 INRIA.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Marcos Didonet Del Fabro - initial API and implementation
 *    Guillaume Gueltas
 *******************************************************************************/


package org.eclipse.gmt.weaver.editors.actions;

import java.util.Iterator;

import org.eclipse.emf.common.notify.impl.NotificationImpl;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.InternalEObject;
import org.eclipse.emf.ecore.impl.ENotificationImpl;
import org.eclipse.emf.edit.ui.action.CreateChildAction;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.ISelectionProvider;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.ui.IEditorPart;

//Find a better way to generate notifications when using editing domain.

/**
 * Child Action
 * @since 1.0
 */
public class WeaverChildAction extends CreateChildAction {
	ISelectionProvider selectionProvider;	
	
	/**
	 * This creates a child action
	 * @param editorPart
	 * @param selection
	 * @param descriptor
	 * @param selectionProvider
	 */
	public WeaverChildAction(IEditorPart editorPart, ISelection selection, Object descriptor, ISelectionProvider selectionProvider) {
		super(editorPart, selection, descriptor);
		this.selectionProvider = selectionProvider;		
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.jface.action.IAction#run()
	 */
	public void run(){
      editingDomain.getCommandStack().execute(command);
      for (Iterator objs = command.getAffectedObjects().iterator(); objs.hasNext();){
      	EObject eObj = (EObject) objs.next();
      	// default name
      	
      	if (selectionProvider != null)
      		selectionProvider.setSelection(new StructuredSelection (eObj));	      	
      	//fakes a notification
      	NotificationImpl notification = new ENotificationImpl((InternalEObject)eObj,
      			NotificationImpl.EVENT_TYPE_COUNT + 1, null, null, null);	      	
      	eObj.eNotify(notification);	      	
      }
	}		
}   	
