/*******************************************************************************
 * Copyright (c) 2005 INRIA.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Marcos Didonet Del Fabro - initial API and implementation
 *    Guillaume Gueltas
 *******************************************************************************/


package org.eclipse.gmt.weaver.editors.interfaces;

import org.eclipse.emf.ecore.resource.ResourceSet;
import org.eclipse.gmt.weaver.editors.GlobalWeaverEditor;
import org.eclipse.jface.action.IMenuListener;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.ui.IPropertyListener;

/**
 * Interface for weaving panel extension
 * @since 1.0
 */
public interface IWeavingPanel extends IMenuListener, IPropertyListener, Listener, IWeaverMenuAdapter {
	/**
	 * Initialize the panel with the reference of the Global Editor
	 * @param editor
	 */
	public void init(GlobalWeaverEditor editor);
    /**
     * Create the viewer
     * @param parent
     */
	public Viewer createViewer(Composite parent);	
    /**
     * Create the control
     * @param parent
     */
	public void createControl(Composite parent);
    /**
     * Get the viewer
     * @return
     */
	public Viewer getViewer();
	/**
     * Sets the ressource set input
     * @param input the ressource set
     */
	public void setInput(ResourceSet input);
    /**
     * Sets the panel title for ContentViewer
     * @param object
     */
	public void setTitle(Object object);
    /**
     * Set the panel title
     * @param title
     * @param image
     */
	public void setTitle(String title, Image image);
	/**
	 * Dispose the panel
	 */
	public void dispose();
			
	/**
	 * adds a new selection listener.  
	 * @param listener
	 */
	public void addWeaverSelectionChangedListener(IWSelectionChangedListener listener);
	/**
	 * removes the selection listener
	 * @param listener
	 */
	public void removeWeaverSelectionChangedListener(IWSelectionChangedListener listener);
	public void setViewersSelection(ISelection aSelection);
	
	public GlobalWeaverEditor getGlobalEditor(); 

}
