/*******************************************************************************
 * Copyright (c) 2005 INRIA.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Marcos Didonet Del Fabro - initial API and implementation
 *    Guillaume Gueltas
 *******************************************************************************/


package org.eclipse.gmt.weaver.editors.interfaces;

import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.edit.domain.EditingDomain;
import org.eclipse.emf.edit.provider.ComposedAdapterFactory;
import org.eclipse.gmt.weaver.core.WeaverModelManager;
import org.eclipse.gmt.weaver.core.WeaverXMLMetadata;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.widgets.Composite;

/**
 * Interface for woven panel extension
 * @since 1.0
 */
public interface IWovenModelPanel extends IWSelectionChangedListener {
	/**
	 * Sets reference to the model manager
	 * @param manager
	 * @param editingDomain
	 */
	public void initializeModelManager(WeaverModelManager manager);
   
	/**
	 * sets reference to the editing domain
	 * @param editingDomain
	 */
	public void initializeEditingDomain(EditingDomain editingDomain);
	/**
	 * initialize model
	 * @param manager
	 * @param editingDomain
	 */
	public Resource initializeModelResource(Resource input, WeaverXMLMetadata xmlFile, String modelID);
   
	/**
     * Creates the viewer
     * @param parent
     */
	public Viewer createViewer(Composite parent);
    /**
     * Creates the control
     * @param parent
     */
	public void createControl(Composite parent);
    /**
     * Sets the resource set input
     * @param input the resource set
     */
	public void setInput(Object input);
    /**
     * Sets the panel title for ContentViewer
     * @param object
     */
	public void setTitle(Object object);
    /**
     * Sets the panel title
     * @param title
     * @param image
     */
	public void setTitle(String title, Image image);
	/**
	 * Disposes the panel
	 */
	public void dispose();
    /**
     * Gets the viewer
     * @return
     */
	public Viewer getViewer();
	/**
	 * Gets the adapter factory
	 * @return Returns the adapterFactory.
	 */
	public ComposedAdapterFactory getAdapterFactory();

}
