/*******************************************************************************

 * Copyright (c) 2005 INRIA.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Marcos Didonet Del Fabro - initial API and implementation
 *******************************************************************************/


package org.eclipse.gmt.weaver.editors.weaving;

import java.net.MalformedURLException;
import java.net.URL;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;

import org.eclipse.emf.common.ui.CommonUIPlugin;
import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.EReference;
import org.eclipse.emf.ecore.EStructuralFeature;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.resource.ResourceSet;
import org.eclipse.emf.ecore.resource.impl.ResourceImpl;
import org.eclipse.emf.ecore.util.EcoreUtil;
import org.eclipse.emf.edit.ui.action.EditingDomainActionBarContributor;
import org.eclipse.emf.edit.ui.dnd.LocalTransfer;
import org.eclipse.emf.edit.ui.dnd.ViewerDragAdapter;
import org.eclipse.emf.edit.ui.provider.AdapterFactoryContentProvider;
import org.eclipse.emf.edit.ui.provider.AdapterFactoryLabelProvider;
import org.eclipse.gmt.weaver.editors.GlobalWeaverEditor;
import org.eclipse.gmt.weaver.editors.interfaces.IWSelectionChangedListener;
import org.eclipse.gmt.weaver.editors.interfaces.IWeavingPanel;
import org.eclipse.gmt.weaver.exception.WeaverException;
import org.eclipse.gmt.weaver.plugin.WeaverPlugin;
import org.eclipse.gmt.weaver.plugin.util.WeaverInterfaceUtil;
import org.eclipse.gmt.weaver.plugin.util.WeaverUtil;
import org.eclipse.gmt.weaver.providers.IIdentifierAdapter;
import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.action.IContributionManager;
import org.eclipse.jface.action.IMenuListener;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.jface.action.IStatusLineManager;
import org.eclipse.jface.action.IToolBarManager;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.jface.action.Separator;
import org.eclipse.jface.action.ToolBarManager;
import org.eclipse.jface.dialogs.InputDialog;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.jface.viewers.ContentViewer;
import org.eclipse.jface.viewers.IBaseLabelProvider;
import org.eclipse.jface.viewers.ILabelProvider;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.CLabel;
import org.eclipse.swt.custom.SashForm;
import org.eclipse.swt.custom.ViewForm;
import org.eclipse.swt.dnd.DND;
import org.eclipse.swt.dnd.Transfer;
import org.eclipse.swt.events.DisposeEvent;
import org.eclipse.swt.events.DisposeListener;
import org.eclipse.swt.events.MouseAdapter;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.events.MouseListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.MenuItem;
import org.eclipse.swt.widgets.ToolBar;
import org.eclipse.swt.widgets.ToolItem;
import org.eclipse.swt.widgets.Tree;
import org.eclipse.ui.IWorkbenchPart;

/**
 * @since AMW2006
 */

public abstract class GenericWeavingPanel implements IWeavingPanel {
	// Panel
	protected Viewer viewer;
	protected Composite container;
	boolean isActive;
	protected CLabel titleLabel;
	protected ToolBar actionBar;
	private Image expandAllImage;
	protected ToolBarManager toolBarManager;
	protected MenuManager menuManager;
	protected Image pullDownImage;
	protected ToolBar systemBar;
	protected ViewForm control;
	private WeavingPanelDndManager weavingDndManager; 
	///menu
	private Collection wLinkEndActions;
	private List resourceActions;
	private List resourceActionsDescription;
	// GlobalEditor Reference
	protected GlobalWeaverEditor globalEditor= null;  
	
	// Current Selection
	protected ISelection editorSelection;
	
	// Listener for other selection models
	private List listeners = new ArrayList();  
	
	// Mouse listener
	protected MouseListener mouseListener = new MouseAdapter(){
		public void mouseDown(MouseEvent e){
			requestActivation();
		}
		public void mouseDoubleClick(MouseEvent e){
			if (e.getSource() == titleLabel){
				doMaximize();
			}
		} 
	};

	
	/**
	 * Constructs a view panel for a view part.
	 */
	public GenericWeavingPanel(){	
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.gmt.weaver.editors.IWeavingPanel#init(org.eclipse.gmt.weaver.editors.GlobalWeaverEditor)
	 */
	public void init(GlobalWeaverEditor editor) {
		this.globalEditor = editor;
	}	
	
	/* (non-Javadoc)
	 * @see org.eclipse.gmt.weaver.editors.IWeavingPanel#createViewer(org.eclipse.swt.widgets.Composite)
	 */
	public Viewer createViewer(Composite parent) {
		TreeViewer v = new SelectionTreeViewer(new Tree(parent,SWT.MULTI));
		v.addSelectionChangedListener(		       
				new ISelectionChangedListener() {
					public void selectionChanged(SelectionChangedEvent event) {
						wSelectionChanged(event.getSelection());
					}
				});
		
		//add drag and drop support		
		Transfer[] transfers = new Transfer[] { LocalTransfer.getInstance() };
		v.addDragSupport(DND.DROP_COPY | DND.DROP_LINK, transfers, new ViewerDragAdapter(
				v));
		//the viewer is passed as parameter because it was not yet associated with the panel
		weavingDndManager = new WeavingPanelDndManager(this, (Viewer) v);
		v.addDropSupport(DND.DROP_COPY | DND.DROP_LINK, transfers, weavingDndManager);
		
		// Provider
		v.setContentProvider(new AdapterFactoryContentProvider(globalEditor.getAdapterFactory()));
		v.setLabelProvider(new AdapterFactoryLabelProvider(globalEditor.getAdapterFactory()));
		return v;
	}
	
	
	/* (non-Javadoc)
	 * @see org.eclipse.gmt.weaver.editors.IWeavingPanel#createControl(org.eclipse.swt.widgets.Composite)
	 */
	public void createControl(Composite parent){
		if (getControl() == null){
			container = parent;
			
			// Create view form.    
			control = new ViewForm(parent, SWT.NONE);
			control.addDisposeListener
			(new DisposeListener()
					{
				public void widgetDisposed(DisposeEvent event){
					dispose();
				}
					});
			control.marginWidth = 0;
			control.marginHeight = 0;
			
			// Create a title bar.
			createTitleBar();
			
			viewer = createViewer(control);
			control.setContent(viewer.getControl());
			
			control.setTabList(new Control [] { viewer.getControl() });
			
			// When the pane or any child gains focus, notify the workbench.
			control.addListener(SWT.Activate, this);
			hookFocus(control);
			hookFocus(viewer.getControl());
			
			
			// add Menu for the viewer
			this.createContextMenu();
			this.makeContributions(this.getMenuManager(), null,
					((EditingDomainActionBarContributor) globalEditor.getEditorSite()
							.getActionBarContributor()).getActionBars()
							.getStatusLineManager());
		}
	}

	/**
	 * Selection changed
	 * @param selection
	 */
	public void wSelectionChanged(ISelection selection) {
		editorSelection = selection;   
		//Object obj =  ((StructuredSelection) selection).getFirstElement();
		//((TreeViewer)viewer).expandToLevel(obj,1);
		setViewersSelection(selection);
	}
	
	/**
	 * Verifies if the current selection is a WLink or WLinkEnd to select
	 * correct elements in the woven models
	 * @param aSelection
	 */
	public void setViewersSelection(ISelection aSelection) {
		StructuredSelection selection = (StructuredSelection) aSelection;
		Object obj = selection.getFirstElement();
		//((TreeViewer)viewer).expandToLevel(obj,2);
		if (obj instanceof EObject) {
			EObject selected = (EObject) obj;
			List selectedIDs = new ArrayList();
			List selectedModelURIs = new ArrayList();
			//set WLinkEnd selection behavior
			if (globalEditor.getModelManager().getHelper().isInstanceOfMetaModel(selected, "WLinkEnd")) {
				List selectedLink = new ArrayList();
				selectedLink.add(selected);
				setWLinkEndSelection(selectedLink,selectedIDs, selectedModelURIs);
			} 
			//set WLinkEnd selection behavior
			else if (globalEditor.getModelManager().getHelper().isInstanceOfMetaModel(selected, "WLink")) {
				setWLinkSelection(selected,selectedIDs,selectedModelURIs);            	
			}
			else if (globalEditor.getModelManager().getHelper().isInstanceOfMetaModel(selected, "WAssociationEnd")) {
				setWAssociationEndSelection(selected,true,selectedIDs,selectedModelURIs);
			}
			else if (globalEditor.getModelManager().getHelper().isInstanceOfMetaModel(selected, "WAssociation")) {
				setWAssociationSelection(selected, selectedIDs,selectedModelURIs);
			}      
			setSelectionChanged(selectedIDs, selectedModelURIs, selected);
		}
	}
	/**
	 * Sets selection behavior for WAssociation elements
	 * @param selected the selected wAssociation
	 * @param selectedURIs a list with all selected objects. (The selection must be done in one pass otherwise it will not work correctly)
	 */
	public void setWAssociationSelection(EObject selected, List selectedIDs, List modelURIs){
		Object listRef = selected.eGet(selected.eClass()
				.getEStructuralFeature("associationEnd"));        
		if (listRef != null) {
			if (((List) listRef).size() > 0){
				List wLinks = new ArrayList();
				
				wLinks.add(selected);                
				for (Iterator i = ((List) listRef).iterator(); i.hasNext();) {
					EObject wAssociationEnd = (EObject) i.next();             	
					//gets the link of each association
					EObject wLink = (EObject) wAssociationEnd.eGet(wAssociationEnd.eClass()
							.getEStructuralFeature("relatedLink"));
					if (wLink != null){
						setWLinkSelection(wLink, selectedIDs, modelURIs);
						wLinks.add(wLink);
					}            	
				}
				//sets weaver selection
				ISelection selection = new StructuredSelection(wLinks);
				getViewer().setSelection(selection);                                    
			}
		}
		
	}
	/**
	 * Sets selection behavior for WAssociationEnd elements
	 * @param selected the selected wAssociationEnd
	 * @param selectCurrentObject if the current association end will be also selecte
	 * @param selectedURIs a list with all selected objects. (The selection must be done in one pass otherwise it will not work correctly)
	 */
	public void setWAssociationEndSelection(EObject selected, boolean selectCurrentObject, List selectedIDs, List modelURIs){
		Object linkRef = selected.eGet(selected.eClass()
				.getEStructuralFeature("relatedLink"));
		if (linkRef != null) {            		    		
			List listOfSelectedObjects = new ArrayList();
			if (selectCurrentObject)
				listOfSelectedObjects.add(selected);
			//selects linkrefs on weaver editor
			listOfSelectedObjects.add(linkRef);        	
			ISelection selection = new StructuredSelection(listOfSelectedObjects);
			getViewer().setSelection(selection, true);            

			setWLinkSelection((EObject)linkRef, selectedIDs, modelURIs);    					
		}    	    	
	}
	/**
	 * Sets selection behavior for WLinks elements
	 * @param selected
	 * @param selectedList
	 */
	public void setWLinkSelection(EObject selected, List selectedList, List modelURIs){
		List references = selected.eClass().getEAllReferences();
		List wLinkEnds = new ArrayList();
		for (Iterator refs = references.iterator(); refs.hasNext();){
			EReference ref = (EReference) refs.next();
			
			try {			
				if ( WeaverUtil.getEClassByName(globalEditor.getModelManager().getWeavingMetamodel(), "WLinkEnd").isSuperTypeOf((EClass)ref.getEType())) { // && !ref.isDerived()){
				//if ( globalEditor.getModelManager().helper.isInstanceOfMetaModel(ref.getEType(),"WLinkEnd") && !ref.isDerived()){
					Object linkEnds = selected.eGet(selected.eClass()
							.getEStructuralFeature(ref.getName()));
					if (linkEnds != null) {					
						if (linkEnds instanceof EObject){
							if (globalEditor.getModelManager().helper.isInstanceOfMetaModel((EObject) linkEnds,"WLinkEnd"))
								wLinkEnds.add(linkEnds);						
						}											
						else{
							if (linkEnds instanceof List)
								if (((List) linkEnds).size() > 0){						
									for (Iterator i = ((List) linkEnds).iterator(); i.hasNext();) {
										EObject wLinkEnd = (EObject) i.next(); 
										if (globalEditor.getModelManager().helper.isInstanceOfMetaModel(wLinkEnd,"WLinkEnd"))
											wLinkEnds.add(wLinkEnd);
									}
								}						
						}
						if (wLinkEnds.size() > 0)
							setWLinkEndSelection(wLinkEnds,selectedList, modelURIs);            					
					}				
				}
			} catch (WeaverException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
			 
		}
	}
	
	/**
	 * Gets the WElementRefs uri associated with this WLinkEnd
	 * @param wLinkEnd
	 * @param selectedList
	 */
	public void setWLinkEndSelection(List wLinkEnds, List selectedList, List modelURIs) {
		for (Iterator i = wLinkEnds.iterator(); i .hasNext();)
		{
			EObject wLinkEnd = (EObject) i.next();
			Object aObj = wLinkEnd.eGet(wLinkEnd.eClass().getEStructuralFeature(
			"element"));
			if (aObj != null) {// a wLinkEnd might not be yet associated with an
				EObject wElementRef = (EObject) aObj;
				String ref = (String) wElementRef.eGet(wElementRef.eClass()
						.getEStructuralFeature("ref"));
				selectedList.add(ref);
				//
				EObject eModelRef = (EObject) wElementRef.eGet(wElementRef.eClass()
						.getEStructuralFeature("modelRef"));
				String modelRef = (String) eModelRef.eGet(eModelRef.eClass()
						.getEStructuralFeature("ref"));		
				modelURIs.add(modelRef);
			}            
		}
	}        
	
	/**
	 * Makes contributions
	 * @param menuManager
	 * @param toolBarManager
	 * @param statusLineManager
	 */
	protected void makeContributions(IMenuManager menuManager,
			IToolBarManager toolBarManager, IStatusLineManager statusLineManager) {
		
		menuManager.add(new Separator());
		//toolBarManager.update(true);		
		menuManager.update(true);
	}

	/* (non-Javadoc)
	 * @see org.eclipse.jface.action.IMenuListener#menuAboutToShow(org.eclipse.jface.action.IMenuManager)
	 */    
	
	public void menuAboutToShow(IMenuManager manager) {
		((IMenuListener) globalEditor.getEditorSite().getActionBarContributor())
		.menuAboutToShow(manager);
		//creates the actions to add references for new WElementRefs
		this.populateMenuAboutToShow(manager);		
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.gmt.weaver.editors.IWeaverMenuAdapter#populateMenuAboutToShow(org.eclipse.jface.action.IMenuManager)
	 */
	public void populateMenuAboutToShow(IMenuManager parentMenu){		
		if (resourceActions != null) {
			
			IMenuManager submenuManagerResource = new MenuManager(WeaverPlugin.getPlugin()
					.getString("_UI_New_element"));
			parentMenu.insertBefore("additions", submenuManagerResource);
			
			submenuManagerResource.add(new Separator("additions"));
			Iterator resActionIt = resourceActions.iterator();
			for (Iterator i = resourceActionsDescription.iterator(); i
			.hasNext();) {
				MenuManager submenuManagerElements = new MenuManager((String) i.next());
				populateManager(submenuManagerElements, (Collection) resActionIt
						.next(), null);
				submenuManagerResource.insertBefore("additions", submenuManagerElements);
			}
		}
	}
	
	/**
	 * Creates context menu
	 */
	protected void createContextMenu() {
		MenuManager contextMenu = new MenuManager("#PopUp");
		contextMenu.add(new Separator("additions"));
		contextMenu.setRemoveAllWhenShown(true);
		contextMenu.addMenuListener(this);
		Menu menu = contextMenu.createContextMenu(this.getViewer().getControl());
		this.getViewer().getControl().setMenu(menu);
		globalEditor.getSite().registerContextMenu(contextMenu, getViewer());
		
		
	}
	/* (non-Javadoc)
	 * @see org.eclipse.gmt.weaver.editors.IWeavingPanel#addWeaverSelectionChangedListener(org.eclipse.gmt.weaver.editors.IWSelectionChangedListener)
	 */
	public void addWeaverSelectionChangedListener(
			IWSelectionChangedListener listener) {
		listeners.add(listener);
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.gmt.weaver.editors.IWeavingPanel#removeWeaverSelectionChangedListener(org.eclipse.gmt.weaver.editors.IWSelectionChangedListener)
	 */
	public void removeWeaverSelectionChangedListener(IWSelectionChangedListener listener) {
		listeners.remove(listener);
	}           
	/**
	 * set selection changed and pass a list of uris and the current selected obj
	 * @param ids
	 * @param currentSelection
	 */
	public void setSelectionChanged(List ids, List modelURIs, EObject currentSelection) {
		Iterator iter = listeners.iterator();
		while (iter.hasNext()) {
			IWSelectionChangedListener next = (IWSelectionChangedListener) iter.next();
			
			next.selectEObject(ids, modelURIs, currentSelection);
		}
	}        
	
	
	/* (non-Javadoc)
	 * @see org.eclipse.gmt.weaver.editors.IWeavingPanel#getViewer()
	 */
	public Viewer getViewer(){
		return viewer;
	}
	
	/**
	 * Sets the ressource set
	 * 
	 * @param input
	 */
	public void setInput(ResourceSet input) {
		viewer.setInput(input);
		((TreeViewer) viewer).expandToLevel(3);
	}
	
	/**
	 * Gets the control.
	 */
	public Control getControl(){
		return control;
	}
	
	/**
	 * Gets the view form.
	 */
	protected ViewForm getViewForm(){
		return control;
	}
	
	/**
	 * @see Listener
	 */
	public void handleEvent(Event event){
		if (event.type == SWT.Activate){
			requestActivation();
		}
	}
	
	/**
	 * Hook focus on a control.
	 */
	public void hookFocus(Control ctrl){
		ctrl.addMouseListener(mouseListener);
	}
	
	/**
	 * Notify the workbook page that the part pane has
	 * been activated by the user.
	 */
	protected void requestActivation(){
		control.getContent().setFocus();
		showFocus(true);
	}
	
	/**
	 * Sets focus to this part.
	 */
	public void setFocus(){
		requestActivation();
		control.getContent().setFocus();
	}
	
	
	/**
	 * Creates the menu manager.
	 */
	private void createMenuManager(){
		menuManager = new MenuManager("Pane Menu");
		if (systemBar != null){
			createPulldownMenu();
		}
	}
	
	/**
	 * Creates a pulldown menu on the action bar.
	 */
	private void createPulldownMenu(){
		if (systemBar != null){
			ToolItem ti = new ToolItem(systemBar, SWT.PUSH, 0);
			try{
				pullDownImage = 
					ImageDescriptor.createFromURL
					(new URL(CommonUIPlugin.INSTANCE.getImage("full/ctool16/ViewPullDown").toString())).createImage();
				ti.setImage(pullDownImage);
				ti.addSelectionListener
				(new SelectionAdapter() 
						{
					public void widgetSelected(SelectionEvent e) 
					{
						showViewMenu();
					}
						});
			}
			catch (MalformedURLException exception){
			}
		}
	}
	
	/**
	 * Creates a title bar for the pane which includes
	 * the view icon and title to the far left, and
	 * the close X icon to the far right.
	 * The middle part is reserved for the view part to
	 * add a menu and tools.
	 */
	protected void createTitleBar(){
		// Only do this once.
		if (titleLabel == null) {
			// Title.  
			titleLabel = new CLabel(control, SWT.SHADOW_NONE);
			hookFocus(titleLabel);
			titleLabel.setAlignment(SWT.LEFT);
			titleLabel.setBackground(null, null);
			titleLabel.addMouseListener
			(new MouseAdapter() 
					{
				public void mouseDown(MouseEvent e) 
				{
					if (e.button == 3)
					{
						showTitleLabelMenu(e);
					}
				}
					});
			
			
			updateTitles();
			control.setTopLeft(titleLabel);
			
			// Listen to title changes.
			// getViewPart().addPropertyListener(this);
			
			// Action bar
			actionBar = new ToolBar(control, SWT.FLAT | SWT.WRAP);
			ToolItem ti = new ToolItem(actionBar, SWT.PUSH, 0);
	        expandAllImage = ImageDescriptor.createFromURL((URL)WeaverPlugin.getPlugin().getImageObj("icons/expandAll.gif")).createImage();
			ti.setImage(expandAllImage);
	        ti.setSelection(false);
	        ti.setToolTipText(WeaverPlugin.getPlugin().getString("_UI_panel_action_expand"));
	        ti.addSelectionListener(new SelectionAdapter(){
	             public void widgetSelected(SelectionEvent e){
	             	if(viewer != null){
	             		((TreeViewer)viewer).expandAll();
	             	}
	             }
	        });
	        setCountElementButton();
	        setSearchElementButton();
			//hookFocus(actionBar);
			control.setTopCenter(actionBar);
			/*
			// System bar.  
			systemBar = new ToolBar(control, SWT.FLAT | SWT.WRAP);
			hookFocus(systemBar);
			if (menuManager != null && !menuManager.isEmpty()){
				createPulldownMenu();
			}
			control.setTopRight(systemBar);*/
		}
	}
	
	private void setCountElementButton(){      
    	ToolItem ti = new ToolItem(actionBar, SWT.PUSH, 0);
        expandAllImage = ImageDescriptor.createFromURL((URL)WeaverPlugin.getPlugin().getImageObj("icons/count.gif")).createImage();
		ti.setImage(expandAllImage);
        ti.setSelection(false);
        ti.setToolTipText(WeaverPlugin.getPlugin().getString("_UI_panel_action_count"));        
        ti.addSelectionListener(new SelectionAdapter(){
             public void widgetSelected(SelectionEvent e){
             	if(viewer != null){            		
             		
             		MessageDialog msgDialog = new MessageDialog(viewer.getControl().getShell(),
             				WeaverPlugin.getPlugin().getString("_UI_element_count_title"),null,
             				WeaverPlugin.getPlugin().getString("_UI_element_count_text") + 
             				WeaverUtil.countElements(((Resource)((ResourceSet) getViewer().getInput()).getResources().get(0))),
             				0,
             				new String[]{WeaverPlugin.getPlugin().getString("_UI_confirm_button")},0);
             		msgDialog.open();
             	}
             }
        });          	
    }
	    
    private void setSearchElementButton (){
    	ToolItem ti = new ToolItem(actionBar, SWT.PUSH, 0);
        expandAllImage = ImageDescriptor.createFromURL((URL)WeaverPlugin.getPlugin().getImageObj("icons/search.gif")).createImage();
		ti.setImage(expandAllImage);
        ti.setSelection(false);
        ti.setToolTipText(WeaverPlugin.getPlugin().getString("_UI_panel_action_search_element"));
        ti.addSelectionListener(new SelectionAdapter(){
        	public void widgetSelected(SelectionEvent e){
        		if(viewer != null){            	             		
        			InputDialog msgDialog = new InputDialog(viewer.getControl().getShell(),
        					WeaverPlugin.getPlugin().getString("_UI_search_element_title"),
        					WeaverPlugin.getPlugin().getString("_UI_search_element_caption"),"",null);             		
        			if (msgDialog.open() == org.eclipse.jface.window.Window.OK){
        				boolean found = false;
        				List list = new ArrayList();
        				for (Iterator i = ((Resource)((ResourceSet) getViewer().getInput()).getResources().get(0)).getAllContents(); i.hasNext();){             	    		
        					Object obj = i.next();
        					if (obj instanceof EObject){             	    				
        						EStructuralFeature feature = ((EObject)obj).eClass().getEStructuralFeature("name");
        						if (feature != null){//search the Name prop             	    					
        							Object name =((EObject)obj).eGet(feature);
        							if (name !=null && name instanceof String){
            							found = ((String)name).toLowerCase().indexOf(msgDialog.getValue().toLowerCase()) >= 0;
            							if (found){
            								list.add(obj);
            								//TODO open every chosen object, one by one 
            								//trick to solve multiple selection problem (to be redone) 
            								getViewer().setSelection(new StructuredSelection(obj));
            								//break; 
            							}        								
        							}
        						}
        					}
        				}
        				if (list.size()==0){//!found) {
		             		MessageDialog aDialog = new MessageDialog(viewer.getControl().getShell(),
		             				WeaverPlugin.getPlugin().getString("_UI_element_not_found"),null,
		             				WeaverPlugin.getPlugin().getString("_UI_element_not_found"),
		             				0,
		             				new String[]{WeaverPlugin.getPlugin().getString("_UI_confirm_button")},0);
		             		aDialog.open();						        									
						}        					
        			}
        		}
        	}
        });               
        
    }	
	
	/**
	 * Maximizes panel
	 */
	protected void doMaximize(){
		Control child = control;
		for (Control parent = control.getParent(); parent instanceof SashForm; parent = parent.getParent()){
			SashForm sashForm = (SashForm)parent;
			if (sashForm.getMaximizedControl() == null){
				sashForm.setMaximizedControl(child);
			}
			else{
				sashForm.setMaximizedControl(null);
			}
			child = parent;
		}
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.gmt.weaver.editors.IWeavingPanel#dispose()
	 */
	public void dispose() {
		if ((control != null) && (!control.isDisposed())) {
			control.removeListener(SWT.Activate, this);
			control = null;
		}
		
		if (pullDownImage != null) {
			pullDownImage.dispose();
			pullDownImage = null;
		}
        if(expandAllImage!= null){
        	expandAllImage.dispose();
        }
        if(actionBar != null){
        	actionBar.dispose();
        }
	}
	
	/**
	 * Returns the menu manager
	 * @return 
	 */
	public MenuManager getMenuManager(){
		if (menuManager == null){
			createMenuManager();
		}		
		return menuManager;
	}
	
	/**
	 * Returns the toolbar manager
	 * @return
	 */
	public ToolBarManager getToolBarManager(){
		if (toolBarManager == null){
			//toolBarManager = new PanelToolBarManager(actionBar);
		}
		return toolBarManager;
	}
	
	/**
	 * Indicates that a property has changed.
	 *
	 * @param source the object whose property has changed
	 * @param propId the id of the property which has changed; property ids
	 *   are generally defined as constants on the source class
	 */
	public void propertyChanged(Object source, int propId){
		if (propId == IWorkbenchPart.PROP_TITLE){
			updateTitles();
		}
	}
	
	/**
	 * Indicates focus in part.
	 */
	public void showFocus(boolean inFocus){
		if (inFocus != isActive){
			isActive = inFocus;
			
			if (titleLabel != null){
				if (inFocus){
					titleLabel.update();
					titleLabel.redraw();
				}
				else{
					titleLabel.update();
					titleLabel.redraw();
				}
			}
		}
	}
	
	/**
	 * Show the context menu for this window.
	 */
	private void showViewMenu(){
		Menu aMenu = menuManager.createContextMenu(getControl());
		Point topLeft = new Point(0, 0);
		topLeft.y += systemBar.getBounds().height;
		topLeft = systemBar.toDisplay(topLeft);
		aMenu.setLocation(topLeft.x, topLeft.y);
		aMenu.setVisible(true);
	}
	
	/* (non-Javadoc)
	 * @see java.lang.Object#toString()
	 */
	public String toString(){
		String label = "disposed";
		if((titleLabel != null) && (!titleLabel.isDisposed()))
			label = titleLabel.getText();
		
		return getClass().getName() + "@" + Integer.toHexString(hashCode()) + 
		"(" + label + ")";
	}
	
	/**
	 * Updates action bars
	 */
	public void updateActionBars(){
		if (menuManager != null){
			menuManager.updateAll(false);
		}
		
		if (toolBarManager != null)	{
			getToolBarManager().update(false);
		}
	}
	
	/**
	 * Updates the title attributes.
	 */
	public void updateTitles(){
		titleLabel.update();
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.gmt.weaver.editors.IWeavingPanel#setTitle(java.lang.Object)
	 */
	public void setTitle(Object object){
		if (viewer != null){
			if (viewer instanceof ContentViewer){
				IBaseLabelProvider labelProvider = ((ContentViewer)viewer).getLabelProvider();
				if (labelProvider instanceof ILabelProvider){
					if (object == null){
						titleLabel.setImage(null);
						titleLabel.setText("");
					}
					else{
						titleLabel.setImage(((ILabelProvider)labelProvider).getImage(object));
						titleLabel.setText(((ILabelProvider)labelProvider).getText(object));
					}
				}
			}
		}
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.gmt.weaver.editors.IWeavingPanel#setTitle(java.lang.String, org.eclipse.swt.graphics.Image)
	 */
	public void setTitle(String title, Image image){
		if (titleLabel != null){
			titleLabel.setImage(image);
			titleLabel.setText(title);
		}
	}
	
	/**
	 * Show the title label menu
	 * @param e the event
	 */
	private void showTitleLabelMenu(MouseEvent e){
		Menu menu = new Menu(titleLabel);		
		
		SashForm sashForm = (SashForm)control.getParent();
		boolean isMaximized = sashForm.getMaximizedControl() != null;
		
		MenuItem restoreItem = new MenuItem(menu, SWT.NONE);
		restoreItem.setText(CommonUIPlugin.INSTANCE.getString("_UI_Restore_menu_item"));
		restoreItem.addSelectionListener
		(new SelectionAdapter() 
				{
			public void widgetSelected(SelectionEvent selectionEvent) 
			{
				doMaximize();
			}
				});
		restoreItem.setEnabled(isMaximized);
		
		MenuItem maximizeItem = new MenuItem(menu, SWT.NONE);
		maximizeItem.setText(CommonUIPlugin.INSTANCE.getString("_UI_Maximize_menu_item"));
		maximizeItem.addSelectionListener
		(new SelectionAdapter() 
				{
			public void widgetSelected(SelectionEvent selectionEvent) 
			{
				doMaximize();
			}
				});
		maximizeItem.setEnabled(!isMaximized);
		
		Point point = new Point(e.x, e.y);
		point = titleLabel.toDisplay(point);
		menu.setLocation(point.x, point.y);
		menu.setVisible(true);
	}		
	
	
	
	/* (non-Javadoc)
	 * @see org.eclipse.gmt.weaver.editors.IWeaverMenuAdapter#createActions(org.eclipse.emf.ecore.EObject)
	 */
	public void createActions(EObject eObj){
		resourceActions = new ArrayList();
		resourceActionsDescription = new ArrayList();
		if ( globalEditor.getModelManager().getHelper().isInstanceOfMetaModel(eObj,"WLinkEnd")) {
			createWLinkEndActions(eObj);
		} else {
			resourceActions = null;
		}
	}
	/**
	 * Creates the actions with the weaving elements for the current WLinkEnd
	 * object
	 * @param wLinkEnd
	 */
	public void createWLinkEndActions(EObject wLinkEnd) {
		
		wLinkEndActions = null;
		
	
		for (Iterator i = globalEditor.getModelManager().getWovenResources().iterator(); i
		.hasNext();) {
			wLinkEndActions = new ArrayList();
			ResourceImpl res = (ResourceImpl) i.next();
			
			IIdentifierAdapter adapter = WeaverUtil.getIdentifierAdapter(res);
			
			for (Iterator j = res.getAllContents(); j.hasNext();) {
				Object eWovenObj = j.next();
				if (eWovenObj instanceof EObject) {//for each object creates
					// an action for creating awElementRef
					Action newElement = 
						new WElementRefAction((EObject) eWovenObj, wLinkEnd);
					IIdentifierAdapter adapterObj = WeaverUtil.getIdentifierAdapter((EObject)eWovenObj);
					String ID = null;
					if (adapterObj != null)
						ID  = adapterObj.getID().toString();
					else
						ID = EcoreUtil.getURI((EObject)eWovenObj).toString();
					newElement.setId(ID);
					wLinkEndActions.add(newElement);
				}
			}
			resourceActions.add(wLinkEndActions);
			resourceActionsDescription.add(adapter.getID().toString());
		}
	}
	
	/**
	 * Adds elements to the menu
	 * @param manager
	 * @param actions
	 * @param contributionID
	 */
	protected void populateManager(IContributionManager manager,
			Collection actions, String contributionID) {
		if (actions != null) {
			for (Iterator i = actions.iterator(); i.hasNext();) {
				IAction action = (IAction) i.next();
				if (contributionID != null) {
					manager.insertBefore(contributionID, action);
				} else {
					manager.add(action);
				}
			}
		}
	}    
	
	/**
	 * Tool bar manager
	 */
	class PanelToolBarManager extends ToolBarManager 
	{
		public PanelToolBarManager(ToolBar paneToolBar) 
		{
			super(paneToolBar);
		}
		
		/**
		 *  EATM I have no idea how this is supposed to be called.
		 */
		protected void relayout(ToolBar toolBar, int oldCount, int newCount) 
		{
			// remove/add the action bar from the view so to avoid
			// having an empty action bar participating in the view's
			// layout calculation (and maybe causing an empty bar to appear)
			if (newCount < 1) 
			{
				if (control.getTopCenter() != null)
				{
					control.setTopCenter(null);
				}
			}
			else 
			{
				toolBar.layout();
				if (control.getTopCenter() == null)
				{
					control.setTopCenter(toolBar);
				}
			}
			Composite parent= toolBar.getParent();
			parent.layout();
			if (parent.getParent() != null){
				parent.getParent().layout();
			}
		}    
	}		
	/**
	 * creates an action to modify the selected wlinkend URI 
	 */
	class WElementRefAction extends Action {
		
		private EObject eWovenObj = null;
		
		private EObject wLinkEnd = null;
		
		public WElementRefAction(EObject eWovenObj,
				EObject wLinkEnd) {
			
			super( eWovenObj.eClass().getName()+" "+WeaverInterfaceUtil.getNameProperty(eWovenObj) );
			this.eWovenObj = eWovenObj;
			this.wLinkEnd = wLinkEnd;
		}
		
		public void run() {
			try {	            
				setNewElementProperties();
			} catch (Exception exception) {
			}
		}
		
		/**
		 * sets the element property of the WLinkEnd object associated with the
		 * current action
		 */
		public void setNewElementProperties() {
			WeaverInterfaceUtil.setLinkEndValueProperty(eWovenObj,wLinkEnd,globalEditor.getModelManager(),globalEditor.getEditingDomain());
			viewer.setSelection(new StructuredSelection(wLinkEnd),true);
		}
	}

	/**
	 * @return Returns the globalEditor.
	 */
	public GlobalWeaverEditor getGlobalEditor() {
		return globalEditor;
	}
	/**
	 * @param globalEditor The globalEditor to set.
	 */
	public void setGlobalEditor(GlobalWeaverEditor globalEditor) {
		this.globalEditor = globalEditor;
	}
	/**
	 * @return Returns the listeners.
	 */
	public List getListeners() {
		return listeners;
	}
	/**
	 * @param listeners The listeners to set.
	 */
	public void setListeners(List listeners) {
		this.listeners = listeners;
	}
	/**
	 * @return Returns the resourceActions.
	 */
	public List getResourceActions() {
		return resourceActions;
	}
	/**
	 * @param resourceActions The resourceActions to set.
	 */
	public void setResourceActions(List resourceActions) {
		this.resourceActions = resourceActions;
	}
	/**
	 * @return Returns the resourceActionsDescription.
	 */
	public List getResourceActionsDescription() {
		return resourceActionsDescription;
	}
	/**
	 * @param resourceActionsDescription The resourceActionsDescription to set.
	 */
	public void setResourceActionsDescription(List resourceActionsDescription) {
		this.resourceActionsDescription = resourceActionsDescription;
	}
	/**
	 * @return Returns the weavingDndManager.
	 */
	public WeavingPanelDndManager getWeavingDndManager() {
		return weavingDndManager;
	}
	/**
	 * @param weavingDndManager The weavingDndManager to set.
	 */
	public void setWeavingDndManager(WeavingPanelDndManager weavingDndManager) {
		this.weavingDndManager = weavingDndManager;
	}
	/**
	 * @return Returns the wLinkEndActions.
	 */
	public Collection getWLinkEndActions() {
		return wLinkEndActions;
	}
	/**
	 * @param linkEndActions The wLinkEndActions to set.
	 */
	public void setWLinkEndActions(Collection linkEndActions) {
		wLinkEndActions = linkEndActions;
	}
	/**
	 * @return Returns the editorSelection.
	 */
	public ISelection getEditorSelection() {
		return editorSelection;
	}
	/**
	 * @param editorSelection The editorSelection to set.
	 */
	public void setEditorSelection(ISelection editorSelection) {
		this.editorSelection = editorSelection;
	}
	/**
	 * @return Returns the titleLabel.
	 */
	public CLabel getTitleLabel() {
		return titleLabel;
	}
	/**
	 * @param titleLabel The titleLabel to set.
	 */
	public void setTitleLabel(CLabel titleLabel) {
		this.titleLabel = titleLabel;
	}
}

 
