/*******************************************************************************
 * Copyright (c) 2005 INRIA.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Marcos Didonet Del Fabro - initial API and implementation
 *******************************************************************************/


package org.eclipse.gmt.weaver.editors.weaving;

import org.eclipse.gmt.weaver.editors.interfaces.ITreeViewerSelection;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Tree;

/**
 * This class is used to create the woven and weaving model trees. It is used to avoid
 * problem with the selection listeners when selecting elements in all panels.
 * @since AMW2006
 */

public class SelectionTreeViewer extends TreeViewer implements ITreeViewerSelection {
    	private boolean isCurrentPanel = true;
    	
    	public SelectionTreeViewer (Composite parent, int style){
    		super(parent, style);
    	}

    	public SelectionTreeViewer(Tree tree){
    		super(tree);
    	}
    	/**
    	 * sets fireSelection to True if the selection listeners events should be called
    	 */
    	public void fireSelections(boolean fireSelection){
    		isCurrentPanel = fireSelection;
    	}
    	/**
    	 * overrides parent method. Verifies if the isCurrentPanel property is true
    	 */
    	protected void updateSelection(ISelection selection) {
    		SelectionChangedEvent event = new SelectionChangedEvent(this, selection);
    		if (isCurrentPanel)
    			fireSelectionChanged(event);
    	}    	
    }

