/*******************************************************************************
 * Copyright (c) 2005 INRIA.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Marcos Didonet Del Fabro - initial API and implementation
 *    Guillaume Gueltas
 *******************************************************************************/


package org.eclipse.gmt.weaver.exception;

import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;

/**
 * This class is an Exception based on CoreException for the weaver
 * @since 1.0 
 */
public class WeaverException extends CoreException {

	/**
	 * <code>serialVersionUID</code>
	 */
	private static final long serialVersionUID = 3977014067670233906L;

	/**
	 * Creates a new exception for the weaver
	 * @param status the status
	 */
	public WeaverException(IStatus status) {
		super(status);
	}
	
	/**
	 * Creates a new exception for the weaver
	 * @param status the status (IStatus.OK, IStatus.INFO, IStatus.WARNING, IStatus.ERROR, IStatus.CANCEL)
	 */
	public WeaverException(int status,String message) {
		super(new Status(status, "org.eclipse.gmt.weaver", IStatus.OK, message, null));
	}	

	/**
	 * Creates a new exception for the weaver
	 * @param status the status (IStatus.OK, IStatus.INFO, IStatus.WARNING, IStatus.ERROR, IStatus.CANCEL)
	 */
	public WeaverException(int status,String message,Exception exception) {
		super(new Status(status, "org.eclipse.gmt.weaver", IStatus.OK, message+"\n"+exception.getMessage(), exception));
	}		
	
	/**
	 * Creates a new exception for the weaver
	 * @param status the status (IStatus.OK, IStatus.INFO, IStatus.WARNING, IStatus.ERROR, IStatus.CANCEL)
	 */
	public WeaverException(int status,Exception exception) {
		super(new Status(status, "org.eclipse.gmt.weaver", IStatus.OK, exception.getMessage(), exception));
	}		
}
