/*******************************************************************************
 * Copyright (c) 2005 INRIA.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Marcos Didonet Del Fabro - initial API and implementation
 *    Guillaume Gueltas
 *******************************************************************************/


package org.eclipse.gmt.weaver.plugin;

import java.io.IOException;
import java.io.InputStream;
import java.net.MalformedURLException;
import java.net.URL;
import java.text.MessageFormat;
import java.util.HashMap;
import java.util.Map;
import java.util.MissingResourceException;
import java.util.ResourceBundle;

import org.eclipse.core.runtime.Platform;
import org.eclipse.emf.common.util.WrappedException;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.jface.resource.ImageRegistry;
import org.eclipse.swt.graphics.Image;
import org.eclipse.ui.plugin.AbstractUIPlugin;
import org.osgi.framework.BundleContext;

/**
 * The main plugin class to be used in the desktop.
 * @since 1.0
 */
public class WeaverPlugin extends AbstractUIPlugin {
	//The shared instance.
	private static WeaverPlugin plugin;
	//Resource bundle.
	private ResourceBundle resourceBundle;
	public Map images = new HashMap();	
	
	
	/**
	 * Get the singleton instance.
	 */
	public static WeaverPlugin getPlugin(){
		if(plugin == null)
			new WeaverPlugin();
		return plugin;
	}	
	
	/**
	 * Returns the singleton instance of the Eclipse plugin.
	 * <!-- begin-user-doc -->
	 * <!-- end-user-doc -->
	 * @return the singleton instance.
	 * @generated
	 */
	
	public WeaverPlugin() {
		super();
		plugin = this;
		try {
			resourceBundle = ResourceBundle.getBundle("/");
		} catch (MissingResourceException x) {
			resourceBundle = null;
		}
	}
	
	/**
	 * This method is called upon plug-in activation
	 */
	public void start(BundleContext context) throws Exception {
		super.start(context);
	}
	
	/**
	 * This method is called when the plug-in is stopped
	 */
	public void stop(BundleContext context) throws Exception {
		super.stop(context);
	}
	
	/**
	 * Returns the shared instance.
	 */
	public static WeaverPlugin getDefault() {
		return plugin;
	}
	
	/**
	 * Returns the string from the plugin's resource bundle,
	 * or 'key' if not found.
	 */
	public static String getResourceString(String key) {
		ResourceBundle bundle = WeaverPlugin.getDefault().getResourceBundle();
		try {
			return (bundle != null) ? bundle.getString(key) : key;
		} catch (MissingResourceException e) {
			return key;
		}
	}
	
	/**
	 * Returns the plugin's resource bundle,
	 */
	public ResourceBundle getResourceBundle() {
		return resourceBundle;
	}
	
	public String getString(String key){
		return Platform.getResourceBundle(getBundle()).getString(key);
	}
	
	public String getString(String key, Object s1){
		return MessageFormat.format(getString(key), new Object [] { s1 });
	}
	
	public String getString(String key, Object s1, Object s2){
		return MessageFormat.format(getString(key), new Object [] { s1, s2 });
	}
	
	public String getString(String key, Object[] array){
		return MessageFormat.format(getString(key), array);
	}	  
	
	/**
	 * This gets a .gif from the icons folder.
	 */
	public ImageDescriptor getImageDescriptor(String key){
		try{
			ImageDescriptor imageDescriptor = ImageDescriptor.createFromURL(new URL(getBundle().getEntry("/"), "icons/" + key + ".gif"));
			return imageDescriptor;
		}
		catch (MalformedURLException exception){
			System.out.println("Failed to load image for '" + key + "'");
			exception.printStackTrace();
		}
		
		return null;
	}
	
	
	/**
	 * This get a .gif from the image registry, which caches the icons folder.
	 */
	public Image getImage(String key){
		ImageRegistry imageRegistry = getImageRegistry();
		Image image = imageRegistry.get(key); 
		if (image == null){
			imageRegistry.put(key, getImageDescriptor(key));
			image = imageRegistry.get(key);
		}
		
		return image;
	}
	/* (non-Javadoc)
	 * @see org.eclipse.emf.common.util.ResourceLocator#getImage(java.lang.String)
	 */
	public Object getImageObj(String key) {
		Object result = (URL)images.get(key);
	    if (result == null){
	        try{
	        	result = doGetImage(key);
	        }catch (MalformedURLException exception){
	        	throw new WrappedException(exception);
	        }catch (IOException exception){
	        	result = null;
	        }
	        images.put(key, result);
	    }
	    return result;
	}
	
	/**
	 * Does the work of fetching the image associated with the key.
	 * It ensures that the image exists.
	 * @param key the key of the image to fetch.
	 * @exception IOException if an image doesn't exist.
	 * @return the description of the image associated with the key.
	 */
	protected Object doGetImage(String key) throws IOException{
		URL url = new URL(getBundle().getEntry("/"),key);
		InputStream inputStream = url.openStream(); 
		inputStream.close();
		return url;
	}		
	
	
}
