/*******************************************************************************
 * Copyright (c) 2005 INRIA.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Marcos Didonet Del Fabro - initial API and implementation
 *    Guillaume Gueltas
 *******************************************************************************/


package org.eclipse.gmt.weaver.plugin.util;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.InputStream;


/**
 * This class contains convenient static methods for working with IO files
 * @since 1.0
 */
public class UtilFile {
	
	/**
	 * returns the stream for the file pathname
	 * @param fileName : String 
	 * @return InputStream
	 * @throws FileNotFoundException
	 */
	public static InputStream getStreamForFile(String fileName) throws FileNotFoundException{
		FileInputStream fis  = new FileInputStream(fileName);
		return fis;
	}
	
	/**
	 * Returns true if file exists (false if file doesn't exist)
	 * @param path : String
	 * @return boolean
	 */
	public static boolean testIfFileExist(String filePath){
		File file = new File(filePath);
		return file.exists();
	}
	
	/**
	 * Returns the name of the file
	 * @param path : String
	 * @return String
	 */
	public static String getFileNameFromPath(String path){
		File file = new File(path);
		return file.getName();
	}	
	
	/**
	 * Gets the extension of a file 
	 * @param filename
	 * @return
	 */
	public static String getFileExtension(String filename){
		String ext=null;
		if (filename!=null || filename.length()>0){
			int dotLoc = filename.lastIndexOf('.');
			if (dotLoc != -1) {
				ext = filename.substring(dotLoc + 1);
			}		
		}
		return ext;
	}
	
	/**
	 * Returns the file (corresponding to this pathname)
	 * @param pathname
	 * @return
	 */
	public static File getFile(String pathname){
		//if (pathname== null) 
			//throw new FileException(this.getClass(),"");
		File f = new File(pathname);
		//boolean file = f.isFile();
		//if(file == false)
			//throw new FileException(this.getClass(),"");
		return f;
	}

	/**
	 * Returns the directory (corresponding to this pathname)
	 * @param pathname
	 * @return
	 */
	public static File getRepertory(String pathname){
		//if (pathname== null) 
			//throw new FileException(this.getClass(),"le repertoire a un chemin d'acces null");
		File f = new File(pathname);
		//if(f.isDirectory() == false)
			//throw new FileException(this.getClass(),"le repertoire "+pathname+" n'existe pas");
		return f;
	}

}
