/*******************************************************************************
 * Copyright (c) 2005 INRIA.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Marcos Didonet Del Fabro - initial API and implementation
 *******************************************************************************/


package org.eclipse.gmt.weaver.providers;

import org.eclipse.emf.common.notify.Notification;
import org.eclipse.emf.common.notify.Notifier;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.gmt.weaver.plugin.WeaverConstant;

/**
 * Handles ID generation for an XMI resource.
 * 
 * @since 2.0
 */

public class ResourceIdentifierAdapter implements IIdentifierAdapter {
	
	private Object ID = null;
	private Resource resource;
		
	
	public ResourceIdentifierAdapter(Resource resource){
		this.resource = resource;
	}
	/* (non-Javadoc)
	 * @see org.eclipse.gmt.weaver.providers.IIdentifierAdapter#setID(java.lang.Object)
	 */
	public void setID(Object obj) {
		if (obj == null)
			ID = getID();	
		else
			ID = obj;
	}
	/* (non-Javadoc)
	 * @see org.eclipse.gmt.weaver.providers.IIdentifierAdapter#getID()
	 */
	public Object getID() {		
		if (ID == null){		
			if (resource.getURI().toString().indexOf(WeaverConstant.RESOURCE_STRING) == -1)
				ID = resource.getURI().toString();
				//ID = modelNumber + WeaverConstant.ID_TOKEN + resource.getURI().toString();
			else
				ID = //modelNumber + WeaverConstant.ID_TOKEN +  
				resource.getURI().toString().substring( WeaverConstant.RESOURCE_STRING.length());				
		} 		
		return ID;
	}
	/* (non-Javadoc)
	 * @see org.eclipse.gmt.weaver.providers.IIdentifierAdapter#getID()
	 */
	//TODO verify this method
	public Object getID2() {
		if (ID == null){		
			if (resource.getURI().toString().indexOf(WeaverConstant.RESOURCE_STRING) == -1)
				ID = resource.getURI().toString();
			else
				ID = resource.getURI().toString().substring( WeaverConstant.RESOURCE_STRING.length());				
		}		
		return ID;
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.emf.common.notify.Adapter#notifyChanged(org.eclipse.emf.common.notify.Notification)
	 */
	public void notifyChanged(Notification notification) {
		//does nothing. not used for this adapter
	}
	/* (non-Javadoc)
	 * @see org.eclipse.emf.common.notify.Adapter#getTarget()
	 */
	public Notifier getTarget() {		
		return null;
	}
	/* (non-Javadoc)
	 * @see org.eclipse.emf.common.notify.Adapter#setTarget(org.eclipse.emf.common.notify.Notifier)
	 */
	public void setTarget(Notifier newTarget) {
		//not used for the resources		
	}
	/* (non-Javadoc)
	 * @see org.eclipse.emf.common.notify.Adapter#isAdapterForType(java.lang.Object)
	 */
	public boolean isAdapterForType(Object type) {
		//does nothing. not used for this adapter
		return false;
	}
	
}