/*******************************************************************************
 * Copyright (c) 2005 INRIA.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Marcos Didonet Del Fabro - initial API and implementation
 *    Guillaume Gueltas
 *******************************************************************************/


package org.eclipse.gmt.weaver.providers;


import java.util.ArrayList;
import java.util.Collection;

import org.eclipse.emf.common.notify.Adapter;
import org.eclipse.emf.common.notify.Notifier;
import org.eclipse.emf.common.notify.impl.AdapterFactoryImpl;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.edit.provider.ComposedAdapterFactory;
import org.eclipse.emf.edit.provider.IEditingDomainItemProvider;
import org.eclipse.emf.edit.provider.IItemLabelProvider;
import org.eclipse.emf.edit.provider.IItemPropertySource;
import org.eclipse.emf.edit.provider.IStructuredItemContentProvider;
import org.eclipse.emf.edit.provider.ITableItemLabelProvider;
import org.eclipse.emf.edit.provider.ITreeItemContentProvider;
import org.eclipse.gmt.weaver.core.WeaverExtensionManager;
import org.eclipse.gmt.weaver.editors.interfaces.IWeaverItemProvider;
import org.eclipse.gmt.weaver.plugin.WeaverPlugin;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.ui.internal.Workbench;


/**
 * An extensible adapter factory for the model weaver
 * It allows to creates the adapter for the current object. 
 * Every object might have a different adapter if needed 
 * @since 1.0
 */

public class WeaverAdapterFactory extends AdapterFactoryImpl {    

    protected Collection supportedTypes = new ArrayList();
    private ComposedAdapterFactory composedFactory = null;
    private WeaverExtensionManager weaverExtensions = null;
    
    /**
     * This creates a new weaver extension manager
     * @param we the Weaver Extension Manager
     */
    public WeaverAdapterFactory(WeaverExtensionManager we) {
    	    	
    	//reflectiveItemProviderAdapter = new WovenElementItemProvider(this);
    	
        supportedTypes.clear();
        supportedTypes.add(IItemLabelProvider.class);
        supportedTypes.add(ITreeItemContentProvider.class);
        supportedTypes.add(IItemPropertySource.class);
        supportedTypes.add(IStructuredItemContentProvider.class);
        supportedTypes.add(IEditingDomainItemProvider.class);
        supportedTypes.add(ITableItemLabelProvider.class);       
        supportedTypes.add(IIdentifierAdapter.class);
        weaverExtensions = we;
    }    
    
    /**
     * Sets the composed adapter factory
     * @param factory
     */
    public void setComposedFactory (ComposedAdapterFactory factory){
        this.composedFactory = factory;
    }
    
    /* (non-Javadoc)
     * @see org.eclipse.emf.common.notify.impl.AdapterFactoryImpl#createAdapter(org.eclipse.emf.common.notify.Notifier)
     */
    public Adapter createAdapter(Notifier target) {
        try{
	    	if (target instanceof EObject){
	            EObject eObj = (EObject)target;
	            //EClass eClass = eObj.eClass();
	            //String name = eClass.getName();
	        	Object provider = weaverExtensions.getProviderImplementationClass(eObj);
	        	if (provider != null){
	        		((IWeaverItemProvider)provider).setAdapterFactory(composedFactory);
	        		return (Adapter)provider;
	        	}
	            return new WeaverItemProvider(composedFactory);            
	        } 
        }catch(Exception e){
        	MessageDialog.openError(        			
        			Workbench.getInstance().getDisplay().getActiveShell(), WeaverPlugin.getPlugin().getString("_UI_error_label"), e.getMessage());
        }
        return null; 
    }
    
    /* (non-Javadoc)
     * @see org.eclipse.emf.common.notify.AdapterFactory#isFactoryForType(java.lang.Object)
     */
    public boolean isFactoryForType(Object type) {
        return type instanceof EObject || supportedTypes.contains(type);
    }

    /* (non-Javadoc)
     * @see org.eclipse.emf.common.notify.AdapterFactory#adapt(org.eclipse.emf.common.notify.Notifier, java.lang.Object)
     */
    public Adapter adapt(Notifier notifier, Object type) {
        return super.adapt(notifier, this);
    }
      
}
