/*******************************************************************************
 * Copyright (c) 2005 INRIA.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Marcos Didonet Del Fabro - initial API and implementation
 *    Guillaume Gueltas
 *******************************************************************************/


package org.eclipse.gmt.weaver.providers;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.eclipse.emf.common.notify.AdapterFactory;
import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.EPackage;
import org.eclipse.emf.ecore.EStructuralFeature;
import org.eclipse.emf.ecore.util.EcoreUtil;
import org.eclipse.emf.edit.provider.IEditingDomainItemProvider;
import org.eclipse.emf.edit.provider.IItemLabelProvider;
import org.eclipse.emf.edit.provider.IItemPropertySource;
import org.eclipse.emf.edit.provider.IStructuredItemContentProvider;
import org.eclipse.emf.edit.provider.ITreeItemContentProvider;
import org.eclipse.emf.edit.provider.ReflectiveItemProvider;
import org.eclipse.gmt.weaver.plugin.util.WeaverUtil;


/**
 * This class is the default Item Provider.
 * It is used when there is no adaptedClassName is defined for base extension
 * @since 1.0
 */

public class WeaverItemProvider extends ReflectiveItemProvider 
	implements      
		IEditingDomainItemProvider,
		IStructuredItemContentProvider,
		ITreeItemContentProvider,
		IItemLabelProvider,
		IItemPropertySource 
    {
    

    /**
     * This creates a default item provider
     * @param adapterFactory
     */
    public WeaverItemProvider(AdapterFactory adapterFactory){         
        super(adapterFactory);               
    }    
           
    /**
     * given a feature name and an object name, it returns true if it matches the feature values  
     * @param eObj
     * @param eFeature
     * @param aName
     * @param aObjName
     * @return
     */
    protected boolean isChoosenFeature(EObject eContainerObj, EStructuralFeature eFeature, String aFeatName, String aObjName){       
        return eContainerObj.eClass().getName().equals(aObjName) &&
        				eFeature.getName().equals(aFeatName);
    }
    
    /* (non-Javadoc)
     * @see org.eclipse.emf.common.notify.Adapter#isAdapterForType(java.lang.Object)
     */
    public boolean isAdapterForType(Object type){
      return  type instanceof WeaverAdapterFactory;
    }         
    /* (non-Javadoc)
     * @see org.eclipse.emf.edit.provider.ReflectiveItemProvider#getAllConcreteSubclasses(org.eclipse.emf.ecore.EClass)
     */
    protected List getAllConcreteSubclasses(EClass eClass){            
        allRoots = new ArrayList();
        allEClasses = new ArrayList();
        
        EPackage pkg = (EPackage)EcoreUtil.getRootContainer(eClass);
        allRoots = WeaverUtil.getAllPackages(pkg.eResource());  
                        
        //get all packages
        for (Iterator pkgs = allRoots.iterator(); pkgs.hasNext();){
            EPackage apkg = (EPackage)pkgs.next();
            //get packages elements
            for (Iterator i = apkg.eAllContents() ;i.hasNext();){
                EObject eObj = (EObject)i.next();
                if (eObj instanceof EClass){//gets concrete and child elements
                    if ( !((EClass)eObj).isAbstract() && eClass.isSuperTypeOf((EClass)eObj) && !allEClasses.contains(eObj))                        
                        allEClasses.add((EClass) eObj);
                }
            }
        }
        return allEClasses;
    }    
}

    
    
    


