/*******************************************************************************
 * Copyright (c) 2005 INRIA.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Marcos Didonet Del Fabro - initial API and implementation
 *    Guillaume Gueltas
 *******************************************************************************/


package org.eclipse.gmt.weaver.providers;

import org.eclipse.emf.common.notify.Notification;
import org.eclipse.emf.common.notify.Notifier;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.xmi.impl.XMIResourceImpl;

/**
 * Handles ID generation for an XMI resource.
 * 
 * @since 1.1
 */

public class XMIIdentifierAdapter implements IIdentifierAdapter {
	
	private EObject target;	
	private Object ID = null;
	
	public XMIIdentifierAdapter(EObject target){
		this.target = target;		
	}		
	/* (non-Javadoc)
	 * @see org.eclipse.gmt.weaver.providers.IIdentifierAdapter#setID(java.lang.Object)
	 */
	public void setID(Object obj) {		
		if (target != null ){
			if (target.eResource() instanceof XMIResourceImpl){
				XMIResourceImpl xmiRes =  (XMIResourceImpl) target.eResource();
				xmiRes.setID(target,(String)obj);
				ID = (String)obj;					
			}			
		}
	}

	/* (non-Javadoc)
	 * @see org.eclipse.gmt.weaver.providers.IIdentifierAdapter#getID()
	 */
	public Object getID() {
		if (ID == null){
			if (target.eResource() instanceof XMIResourceImpl){
				XMIResourceImpl xmiRes =  (XMIResourceImpl) target.eResource();
				ID = xmiRes.getID(target);				
			}			
		}
		return ID;
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.emf.common.notify.Adapter#notifyChanged(org.eclipse.emf.common.notify.Notification)
	 */
	public void notifyChanged(Notification notification) {
		//does nothing. not used for this adapter
	}
	/* (non-Javadoc)
	 * @see org.eclipse.emf.common.notify.Adapter#getTarget()
	 */
	public Notifier getTarget() {		
		return target;
	}
	/* (non-Javadoc)
	 * @see org.eclipse.emf.common.notify.Adapter#setTarget(org.eclipse.emf.common.notify.Notifier)
	 */
	public void setTarget(Notifier newTarget) {
		ID = null;
		target = (EObject) newTarget;		
	}
	/* (non-Javadoc)
	 * @see org.eclipse.emf.common.notify.Adapter#isAdapterForType(java.lang.Object)
	 */
	public boolean isAdapterForType(Object type) {
		//does nothing. not used for this adapter
		return false;
	}
	
}