/*******************************************************************************
 * Copyright (c) 2005 INRIA. - C3SML UFPR

 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Marcos Didonet Del Fabro - initial API and implementation
 *    Guillaume Gueltas
 *******************************************************************************/


package org.eclipse.gmt.weaver.wizards;

import java.lang.reflect.InvocationTargetException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.gmt.weaver.plugin.WeaverPlugin;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.operation.IRunnableWithProgress;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.jface.viewers.ColumnWeightData;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.TableLayout;
import org.eclipse.jface.wizard.IWizardPage;
import org.eclipse.jface.wizard.WizardPage;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.swt.widgets.TableItem;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.dialogs.ResourceSelectionDialog;

/**
 * First page of the wizard (metamodel extension, ...)
 * @since 1.0 
 */
public class WeaverFirstWizardPage extends WizardPage {
	protected WeaverWizard parentWizard;
	private String extensionKM3String = WeaverPlugin.getPlugin().getString("_UI_extension_2");
	protected ISelection selection;
	// Resources in RessourceDialiog
	private List resources = new ArrayList();
	
	private List pluginExtensions= new ArrayList();
	private List loadedPluginExtensions= new ArrayList();
	private List pluginPathExtensions= new ArrayList();
	private List loadedPluginPathExtensions= new ArrayList();
	
	private List workspaceExtensions= new ArrayList();
	private List loadedWorkspaceExtensions= new ArrayList();
	
	private Table metamodelTable;
	private Table loadedMetamodelTable;
	private Label labelkm3;
	private Button buttonkm3;
	private Button add;
	private Button remove;
	
	private Label ecoreMeta;
	private Button browseweaving;	
	private Text newWeavingText; 
	
	private Button loadEcore;
	private Button loadKM3; 
	
	
	Composite compositeEcore;
	
	/**
	 * Creates a new first wizard page
	 * @param parent the wizard parent of this page
	 * @param pageName the name of the page
	 * @param selection the current object selection
	 */
	public WeaverFirstWizardPage(WeaverWizard parent, String pageName, ISelection selection) {
		super(pageName);
		this.selection = selection;
		this.parentWizard = parent;
		setPageComplete(false);
	}		
	
	/**
	 * Creates a new first wizard page
	 * @param parent the wizard parent of this page
	 * @param pageName the name of the page
	 * @param title the title for this wizard page,
	 *   or <code>null</code> if none
	 * @param titleImage the image descriptor for the title of this wizard page,
	 *   or <code>null</code> if none
	 */
	public WeaverFirstWizardPage(WeaverWizard parent,String pageName, String title,
			ImageDescriptor titleImage) {
		super(pageName, title, titleImage);
		this.parentWizard = parent;
		setPageComplete(false);
	}

	/* (non-Javadoc)
	 * @see org.eclipse.jface.dialogs.IDialogPage#createControl(org.eclipse.swt.widgets.Composite)
	 */
	public void createControl(Composite parent) {
	    		
		Composite composite = new Composite(parent, SWT.NONE);
		{
		    GridLayout layout = new GridLayout();
		    composite.setLayout(layout);
		    layout.numColumns = 2;
		    layout.verticalSpacing = 12;		    
		
		    GridData data = new GridData();
		    data.verticalAlignment = GridData.FILL;
		    data.grabExcessVerticalSpace = true;
		    data.horizontalAlignment = GridData.FILL;
		    composite.setLayoutData(data);
		}
		
		loadEcore = new Button(composite, SWT.RADIO);
		{
			loadEcore.setText(WeaverPlugin.getPlugin().getString("_UI_wizard_load_ecore"));
		    GridData data = new GridData();
		    data.horizontalAlignment = GridData.BEGINNING;
		    data.horizontalSpan = 2;
		    loadEcore.setLayoutData(data);	
		    loadEcore.addSelectionListener(new SelectionAdapter() {
				public void widgetSelected(SelectionEvent e) {
					cleanData();			
					newWeavingText.setFocus();
				}
			});	
		};
		
		compositeEcore = new Composite(composite, SWT.NONE);
		{
		    GridLayout layout = new GridLayout();
		    compositeEcore.setLayout(layout);
		    layout.numColumns = 3;
		    layout.verticalSpacing = 6;		   
		    	
		    GridData data = new GridData();
		    data.verticalAlignment = GridData.FILL;
		    data.grabExcessVerticalSpace = true;
		    data.horizontalAlignment = GridData.FILL;
		    data.horizontalSpan = 2;
		    compositeEcore.setLayoutData(data);
		}
				
//		 Line to enter the name		
		ecoreMeta = new Label(compositeEcore, SWT.NULL);
		ecoreMeta.setText(WeaverPlugin.getPlugin().getString("_UI_wizard_enter_model_name"));
				
		newWeavingText = new Text(compositeEcore, SWT.BORDER | SWT.SINGLE);
		GridData gd = new GridData(GridData.FILL_HORIZONTAL);		
		newWeavingText.setLayoutData(gd);
		newWeavingText.addModifyListener(new ModifyListener() {
			public void modifyText(ModifyEvent e) {
				setPageComplete(ecoreOK());
			}
		});		
		
		browseweaving = new Button(compositeEcore,SWT.PUSH);
		{
		    GridData data = new GridData();
		    data.horizontalAlignment = GridData.CENTER;
		    browseweaving.setLayoutData(data);
		}
		
		browseweaving.setText(WeaverPlugin.getPlugin().getString("_UI_wizard_button_browse"));
		browseweaving.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				handleWorkspaceFileBrowse(newWeavingText);	
				setPageComplete(ecoreOK());
			}
		});			

		//		 First line
		Label labelLine = new Label(composite, SWT.SEPARATOR | SWT.HORIZONTAL);
		{
		    GridData data = new GridData();
		    data.horizontalAlignment = GridData.FILL;
		    data.horizontalSpan = 2;
		    labelLine.setLayoutData(data);			
		}
		//TODO - change enabled/disabled features 
		loadKM3 = new Button(composite, SWT.RADIO);
		{
			loadKM3.setText(WeaverPlugin.getPlugin().getString("_UI_wizard_load_KM3"));
		    GridData data = new GridData();
		    data.horizontalAlignment = GridData.BEGINNING;
		    data.horizontalSpan = 2;
		    //TODO
		    loadKM3.setEnabled(true);
		    //loadKM3.setSelection(true);
		    loadEcore.addSelectionListener(new SelectionAdapter() {
				public void widgetSelected(SelectionEvent e) {
					cleanData();					
				}
			});	
		    loadEcore.setSelection(true);
		};
		
		labelkm3 = new Label(composite,SWT.NONE);
		labelkm3.setText(WeaverPlugin.getPlugin().getString("_UI_wizard_metamodel_extension"));
		{
		    GridData data = new GridData();
		    data.horizontalAlignment = GridData.CENTER;		    
		    labelkm3.setLayoutData(data);
		}
		
		buttonkm3 = new Button(composite,SWT.PUSH);
		{
		    GridData data = new GridData();
		    data.horizontalAlignment = GridData.CENTER;
		    buttonkm3.setLayoutData(data);
		}
		buttonkm3.setText(WeaverPlugin.getPlugin().getString("_UI_wizard_button_browse_mm"));
		buttonkm3.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				handleFileBrowse();
				setPageComplete(extensionsOK());
			}
		});	

        metamodelTable = new Table(composite, SWT.CHECK | SWT.BORDER);
        {
          GridData data = new GridData();
          data.verticalAlignment = GridData.FILL;
          data.grabExcessHorizontalSpace = true;
          data.grabExcessVerticalSpace = true;
          data.horizontalAlignment = GridData.FILL;
          data.horizontalSpan = 2;
          data.heightHint = 90;
          metamodelTable.setLayoutData(data);
        }

        metamodelTable.setHeaderVisible(true);
        metamodelTable.setLinesVisible(true);
        {
          TableLayout layout = new TableLayout();

          TableColumn locationColumn = new TableColumn(metamodelTable, SWT.NONE);
          layout.addColumnData(new ColumnWeightData(1, true));
          locationColumn.setText(WeaverPlugin.getPlugin().getString("_UI_wizard_mm_table_column1"));
          
          locationColumn.setResizable(true);

          TableColumn sourceColumn = new TableColumn(metamodelTable, SWT.NONE);
          layout.addColumnData(new ColumnWeightData(1, true));
          sourceColumn.setText(WeaverPlugin.getPlugin().getString("_UI_wizard_mm_table_column2"));
          sourceColumn.setWidth(80);
          sourceColumn.setResizable(true);

          metamodelTable.setLayout(layout);
        }
        
		add = new Button(composite,SWT.PUSH);
		{
		    GridData data = new GridData();
		    data.horizontalAlignment = GridData.CENTER;
		    add.setLayoutData(data);
		}
		add.setText(WeaverPlugin.getPlugin().getString("_UI_wizard_button_load_mm"));
		add.addSelectionListener(new SelectionListener(){
			
			public void widgetSelected(SelectionEvent e) {
				try{
					TableItem[] array = metamodelTable.getItems();
					for(int i=0;i<array.length;i++){
						if(array[i].getChecked()){
							if(workspaceExtensions.contains(array[i].getText(0))){
								workspaceExtensions.remove(array[i].getText(0));
								loadedWorkspaceExtensions.add(array[i].getText(0));
							}
							else{
								int index = pluginExtensions.indexOf(array[i].getText(0));
								pluginExtensions.remove(index);
								String path = (String) pluginPathExtensions.get(index);
								pluginPathExtensions.remove(index);
								
								loadedPluginExtensions.add(array[i].getText(0));
								loadedPluginPathExtensions.add(path);
							}
						}
					}
					redrawTables();
					//click in the load metamodel button					
					setPageComplete(extensionsOK());
					//dialogChanged();
				}catch(Exception ex){}
			}

			public void widgetDefaultSelected(SelectionEvent e) {
			}
		});
		 
		remove = new Button(composite,SWT.PUSH);
		{
		    GridData data = new GridData();
		    data.horizontalAlignment = GridData.CENTER;
		    remove.setLayoutData(data);
		}
		remove.setText(WeaverPlugin.getPlugin().getString("_UI_wizard_button_remove_mm"));
		remove.addSelectionListener(new SelectionListener(){

			public void widgetSelected(SelectionEvent e) {
				try{
					TableItem[] array = loadedMetamodelTable.getItems();
					for(int i=0;i<array.length;i++){
						if(array[i].getChecked()){
							if(loadedWorkspaceExtensions.contains(array[i].getText(0))){
								loadedWorkspaceExtensions.remove(array[i].getText(0));
								workspaceExtensions.add(array[i].getText(0));
							}
							else{
								int index = loadedPluginExtensions.indexOf(array[i].getText(0));
								loadedPluginExtensions.remove(index);
								String path = (String) loadedPluginPathExtensions.get(index);
								loadedPluginPathExtensions.remove(index);
								
								pluginExtensions.add(array[i].getText(0));
								pluginPathExtensions.add(path);
							}
						}
					}
					redrawTables();
					//remove the extension
					setPageComplete(extensionsOK());
					//dialogChanged();
				}catch(Exception ex){}
			}
			public void widgetDefaultSelected(SelectionEvent e) {
			}
		});        
        
        
        loadedMetamodelTable = new Table(composite, SWT.CHECK | SWT.BORDER);
        {
          GridData data = new GridData();
          data.verticalAlignment = GridData.FILL;
          data.grabExcessHorizontalSpace = true;
          data.grabExcessVerticalSpace = true;
          data.horizontalAlignment = GridData.FILL;
          data.horizontalSpan = 2;
          data.heightHint = 90;
          loadedMetamodelTable.setLayoutData(data);
        }

        loadedMetamodelTable.setHeaderVisible(true);
        loadedMetamodelTable.setLinesVisible(true);
        {
          TableLayout layout = new TableLayout();

          TableColumn locationColumn = new TableColumn(loadedMetamodelTable, SWT.NONE);
          layout.addColumnData(new ColumnWeightData(1, true));
          locationColumn.setText("Location");
          locationColumn.setResizable(true);

          TableColumn sourceColumn = new TableColumn(loadedMetamodelTable, SWT.NONE);
          layout.addColumnData(new ColumnWeightData(1, true));
          sourceColumn.setText("Source");
          sourceColumn.setWidth(80);
          sourceColumn.setResizable(true);

          loadedMetamodelTable.setLayout(layout);
        }
        initControls();
		//dialogChanged();
        setPageComplete(extensionsOK());
		setControl(composite);
	}
	
	/**
	 * Opens the browse file
	 */	
	private void handleFileBrowse() {
        ResourceSelectionDialog resourceSelectionDialog = 
            new ResourceSelectionDialog
            (getShell(), 
            ResourcesPlugin.getWorkspace().getRoot(),
            WeaverPlugin.getPlugin().getString("_UI_wizard_select_metamodel_ext"));
        resourceSelectionDialog.setInitialSelections(resources.toArray());
        resourceSelectionDialog.open();
        if (resourceSelectionDialog.getReturnCode() == ResourceSelectionDialog.OK){
	        Object [] result = resourceSelectionDialog.getResult();
	        if(result != null){
	        	resources.clear();
	        	workspaceExtensions.clear();
	        	loadedWorkspaceExtensions.clear();
	        	initControls();
	        	int lenght = result.length;
	        	for(int i=0; i<lenght; i++){
	        		if((result[i] != null)&&
	        				(result[i] instanceof IFile) &&
	        				(((IFile)result[i]).exists()) &&
	        				(((IFile)result[i]).getFileExtension().equals(extensionKM3String))){
	        			resources.add((IFile)result[i]);
	        			workspaceExtensions.add(((IFile)result[i]).getFullPath().toString());
	        		}
	        	}
	        }
	        else{
	        	resources.clear();
	        	workspaceExtensions.clear();
	        	loadedWorkspaceExtensions.clear();
	        	initControls();
	        }
	        redrawTables();	    
        }
	}	

	/**	
	 * Initializes all controls of this page with extensions found
	 */
	private void initControls(){
    	loadedPluginExtensions.clear();
    	loadedPluginPathExtensions.clear();
		pluginExtensions.clear();
		pluginPathExtensions.clear();
		for(Iterator iter = parentWizard.getMetamodelExtensionNames().iterator();iter.hasNext();){
			String[] array = (String[]) iter.next();
			pluginExtensions.add(array[1]);
			pluginPathExtensions.add(array[0]);
		}
		redrawTables();
		enableEcore( true ); //loadEcore.getSelection());
		enableKM3( false ); //! loadEcore.getSelection());
	}	
	
	/**
	 * redraws the tables
	 */
	private void redrawTables(){
		metamodelTable.removeAll();
		for(Iterator iter = pluginExtensions.iterator(); iter.hasNext();){
			TableItem ligne = new TableItem(metamodelTable,SWT.NONE);
			ligne.setText(0,(String)iter.next());
			ligne.setText(1,"Plug-in");
		}
		for(Iterator iter = workspaceExtensions.iterator(); iter.hasNext();){
			TableItem ligne = new TableItem(metamodelTable,SWT.NONE);
			ligne.setText(0,(String)iter.next());
			ligne.setText(1,"Workspace");
		}
		loadedMetamodelTable.removeAll();
		for(Iterator iter = loadedPluginExtensions.iterator(); iter.hasNext();){
			TableItem ligne = new TableItem(loadedMetamodelTable,SWT.NONE);
			ligne.setText(0,(String)iter.next());
			ligne.setText(1,"Plug-in");
			ligne.setChecked(true);
		}
		for(Iterator iter = loadedWorkspaceExtensions.iterator(); iter.hasNext();){
			TableItem ligne = new TableItem(loadedMetamodelTable,SWT.NONE);
			ligne.setText(0,(String)iter.next());
			ligne.setText(1,"Workspace");
			ligne.setChecked(true);
		}
	}
	
	private boolean ecoreOK() {
		return loadEcore.getSelection() && newWeavingText.getText().length() > 0;
	}
	private boolean extensionsOK(){
		return loadKM3.getSelection() && (loadedPluginExtensions.size() + loadedWorkspaceExtensions.size()) > 0;
	}
	/**
	 * Ensures that both fields are set
	 * initialize wmodel for next page
	 */
	private void dialogChanged() {
		// management of task
		IRunnableWithProgress op = new IRunnableWithProgress() {
			public void run(IProgressMonitor monitor) throws InvocationTargetException {
				try{
					parentWizard.initializeWeavingResource(
							getPluginMmExtensions(),getWorkspaceMmExtensions(),monitor, 
							loadEcore.getSelection(), newWeavingText.getText());
				} catch (CoreException e) {
					throw new InvocationTargetException(e);
				} finally {
					monitor.done();
				}
			}
		};
		
		// test if one or many extensions are selected
		// and run the research of subtypes of wmodel
		if ( (loadEcore.getSelection() && ecoreOK()) || (loadKM3.getSelection() && extensionsOK()) ) { 
		
		//if((loadedPluginExtensions.size() + loadedWorkspaceExtensions.size()) >0){
			
			setMessage(WeaverPlugin.getPlugin().getString("_UI_wizard_first_description"));
			setPageComplete(true);
			try {
				getContainer().run(false, false, op);
			} catch (InvocationTargetException e1) {
				setPageComplete(false);
				e1.printStackTrace();
				Throwable realException = e1.getTargetException();
				MessageDialog.openError(getShell(), WeaverPlugin.getPlugin().getString("_UI_error_label"), realException.getMessage());
			} catch (InterruptedException e1) {
				setPageComplete(false);
				e1.printStackTrace();
			}
		}
		else{
			setMessage(WeaverPlugin.getPlugin().getString("_UI_wizard_select_metamodel_ext"));
			setPageComplete(false);
		}
	}
	
	/**
	 * Returns the collection of metamodel extensions selected (from an plugin extension)
	 * @return the java.util list of metamodel extensions selected
	 */
	public Collection getPluginMmExtensions() {
		
		if(loadedPluginExtensions.size() != loadedPluginPathExtensions.size()){
			System.out.println("ERROR PATH / FILE");
		}
		
		Collection fullPath = new ArrayList();
		int size = loadedPluginExtensions.size();
		for(int i = 0;i<size;i++){		
			fullPath.add(((String)loadedPluginPathExtensions.get(i))+'/'+((String)loadedPluginExtensions.get(i)));
			//takes only the extension name, the path will be calculated automatically
		}
		return fullPath;
	}
	
	/**
	 * Returns the collection of metamodel extensions selected (from workspace files)
	 * @return the java.util list of metamodel extensions selected
	 */
	public Collection getWorkspaceMmExtensions() {
		return loadedWorkspaceExtensions;
	}	
	/**
	 * @return Returns the loadedMetamodelTable.
	 */
	public Table getLoadedMetamodelTable() {
		return loadedMetamodelTable;
	}
	/**
	 * @param loadedMetamodelTable The loadedMetamodelTable to set.
	 */
	public void setLoadedMetamodelTable(Table loadedMetamodelTable) {
		this.loadedMetamodelTable = loadedMetamodelTable;
	}
	/**
	 * @return Returns the metamodelTable.
	 */
	public Table getMetamodelTable() {
		return metamodelTable;
	}
	/**
	 * @param metamodelTable The metamodelTable to set.
	 */
	public void setMetamodelTable(Table metamodelTable) {
		this.metamodelTable = metamodelTable;
	}
	/**
	 * @return Returns the parentWizard.
	 */
	public WeaverWizard getParentWizard() {
		return parentWizard;
	}
	/**
	 * @param parentWizard The parentWizard to set.
	 */
	public void setParentWizard(WeaverWizard parentWizard) {
		this.parentWizard = parentWizard;
	}

	/*
	 *  (non-Javadoc)
	 * @see org.eclipse.jface.wizard.IWizardPage#canFlipToNextPage()
	 */
    public boolean canFlipToNextPage() {
        return isPageComplete();
    }
    
    private void cleanData(){
    	enableEcore(loadEcore.getSelection());
    	enableKM3(! loadEcore.getSelection());
    	if (loadEcore.getSelection()){
    		initControls();
    		setPageComplete(ecoreOK());    		
    	}    		
    	else{
    		newWeavingText.setText("");
    		setPageComplete(extensionsOK());    		
    	}    		
    }   
    
    private void enableKM3(boolean enabled) {
    	metamodelTable.setEnabled(enabled);
    	loadedMetamodelTable.setEnabled(enabled);
    	labelkm3.setEnabled(enabled);
    	buttonkm3.setEnabled(enabled);
    	add.setEnabled(enabled);
    	remove.setEnabled(enabled);		
	}

	private void enableEcore(boolean enabled){
    	ecoreMeta.setEnabled(enabled);
    	browseweaving.setEnabled(enabled);	
    	newWeavingText.setEnabled(enabled); 
    }

    /*
     *  (non-Javadoc)
     * @see org.eclipse.jface.wizard.IWizardPage#getNextPage()
     */
    public IWizardPage getNextPage() {
        if (parentWizard == null)
            return null;        
        dialogChanged();
        if (isPageComplete())
        	return parentWizard.getNextPage(this);
        else
        	return null;
    }
	/**
	 * Opens the browse file - the file must be in the workspace
	 */	
	private void handleWorkspaceFileBrowse(Text aText) {
        ResourceSelectionDialog resourceSelectionDialog = 
            new ResourceSelectionDialog            
            (getShell(), 
            ResourcesPlugin.getWorkspace().getRoot(),
            WeaverPlugin.getPlugin().getString("_UI_wizard_button_browse"));
        //resourceSelectionDialog.setInitialSelections(resources.toArray());
        resourceSelectionDialog.open();
        if (resourceSelectionDialog.getReturnCode() == ResourceSelectionDialog.OK){
	        Object [] result = resourceSelectionDialog.getResult();
	        if(result != null){//always takes the first selected element (must be improved to allow only single selection	        	
	        	aText.setText(((IFile)result[0]).getFullPath().toString());
        }
        }
	}	

	    
}
