/*******************************************************************************
 * Copyright (c) 2005 INRIA.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Marcos Didonet Del Fabro - initial API and implementation
 *    Guillaume Gueltas
 *******************************************************************************/


package org.eclipse.gmt.weaver.wizards;

import java.util.Iterator;
import java.util.List;

import org.eclipse.core.resources.IContainer;
import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.Path;
import org.eclipse.gmt.weaver.exception.WeaverException;
import org.eclipse.gmt.weaver.plugin.WeaverConstant;
import org.eclipse.gmt.weaver.plugin.WeaverPlugin;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.wizard.WizardPage;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.dialogs.ContainerSelectionDialog;
import org.eclipse.ui.dialogs.ResourceSelectionDialog;

/**
 * Second page of the wizard (weaving data)
 * @since 1.0
 */
public class WeaverSecondWizardPage extends WizardPage {
	protected WeaverWizard parentWizard;
	// First line
	private Text projectText;
	// Line "new model" 
	private Text newModelFileText;
	private Label newModelLabel1;
	private Button newModelButton;
	// Line "existing model"
	private Text existingModelFileText;
	private Label existingModelLabel1;
	private Button browseModelButton;
	private Button existingModelButton;
	private Label infoLabel;

	// Line "extension"
	private Combo extensionEditorCombo;
	private String editorExtensionSelected;
	// Line "wmodel"
	private Combo wmodelCombo;
	private String wmodelSelected;
	
	// Text label for messages
	private String existingWeavingModelString = WeaverPlugin.getPlugin().getString("_UI_existing_weaving_model");
	private String newWeavingModelString = WeaverPlugin.getPlugin().getString("_UI_new_weaving_model");
	private String projectString = WeaverPlugin.getPlugin().getString("_UI_container");
	private String extensionXMIString = WeaverPlugin.getPlugin().getString("_UI_extension_1");
	private String extensionKM3String = WeaverPlugin.getPlugin().getString("_UI_extension_2");
	private String extensionEcoreString = WeaverPlugin.getPlugin().getString("_UI_extension_3");
	private String extensionAMWString = WeaverConstant.FILE_EXT;

	private ISelection selection;
	
	/**
	 * Creates a new second wizard page
	 * @param parent the wizard parent of this page
	 * @param pageName the name of the page
	 * @param selection the current object selection
	 */
	public WeaverSecondWizardPage(WeaverWizard parent, String pageName, ISelection selection) {
		super(pageName);
		this.selection = selection;
		this.parentWizard = parent;
		setPageComplete(false);
	}	
	
	/**
	 * Creates a new second wizard page
	 * @param parent the wizard parent of this page
	 * @param pageName the name of the page
	 * @param title the title for this wizard page,
	 *   or <code>null</code> if none
	 * @param titleImage the image descriptor for the title of this wizard page,
	 *   or <code>null</code> if none
	 */
	public WeaverSecondWizardPage(WeaverWizard parent,String pageName, String title,
			ImageDescriptor titleImage) {
		super(pageName, title, titleImage);
		this.parentWizard = parent;
		setPageComplete(false);
	}

	/* (non-Javadoc)
	 * @see org.eclipse.jface.dialogs.IDialogPage#createControl(org.eclipse.swt.widgets.Composite)
	 */
	public void createControl(Composite parent) {
		Composite container = new Composite(parent, SWT.NULL);
		GridLayout layout = new GridLayout();
		container.setLayout(layout);
		layout.numColumns = 3;
		layout.verticalSpacing = 9;
		
		// griddata 
		GridData gd1 = new GridData(GridData.FILL_HORIZONTAL);
		gd1.horizontalSpan = 3;
		GridData gd2 = new GridData(GridData.FILL_HORIZONTAL);
		gd2.horizontalSpan = 3;
		GridData gd3 = new GridData(GridData.FILL_HORIZONTAL);
		gd3.horizontalSpan = 3;
		GridData gd4 = new GridData(GridData.BEGINNING);
		gd4.horizontalSpan = 3;
		GridData gd5 = new GridData(GridData.BEGINNING);
		gd5.horizontalSpan = 3;
		GridData gd6 = new GridData(GridData.BEGINNING);
		gd6.horizontalSpan = 3;
		
		// First line
		this.createProjectLine(container);
		Label labelLine = new Label(container, SWT.SEPARATOR | SWT.HORIZONTAL);
		labelLine.setLayoutData(gd1);

		// Second line and third line
		Label label = new Label(container,SWT.BEGINNING);
		label.setText(WeaverPlugin.getPlugin().getString("_UI_wizard_question_weaving_model"));
		label.setLayoutData(gd5);
		this.createNewModelLine(container);		
		this.createExistingModelLine(container);	
		labelLine = new Label(container, SWT.SEPARATOR | SWT.HORIZONTAL);
		labelLine.setLayoutData(gd2);
		
		// fourth line
		label = new Label(container,SWT.BEGINNING);
		label.setText(WeaverPlugin.getPlugin().getString("_UI_wizard_question_weaving_panel"));
		label.setLayoutData(gd4);
		this.createExtensionEditorLine(container);
		labelLine = new Label(container, SWT.SEPARATOR | SWT.HORIZONTAL);
		labelLine.setLayoutData(gd3);
		
		// lastline
		label = new Label(container,SWT.BEGINNING);
		label.setText(WeaverPlugin.getPlugin().getString("_UI_wizard_question_wmodel"));
		label.setLayoutData(gd6);
		this.createWModelLine(container);
		
		initialize();
		initControls();
		dialogChanged();
		setControl(container);
	}

	/**
	 * Creates the project line (for choose a working project and store the weaving model)
	 * @param container the container
	 */
	private void createProjectLine(Composite container) {

		Label label = new Label(container, SWT.NULL);
		label.setText("&"+WeaverPlugin.getPlugin().getString("_UI_wizard_question_projectname"));
		projectText = new Text(container, SWT.BORDER | SWT.SINGLE);
		GridData gd = new GridData(GridData.FILL_HORIZONTAL);
		projectText.setLayoutData(gd);
		projectText.addModifyListener(new ModifyListener() {
			public void modifyText(ModifyEvent e) {
				dialogChanged();
			}
		});

		Button button = new Button(container, SWT.PUSH);
		button.setText(WeaverPlugin.getPlugin().getString("_UI_wizard_button_browse"));
		button.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				handleProjectBrowse();
			}
		});
	}

	/**
	 * Creates the new model line (for create a new weaving model)
	 * @param container the container
	 */
	private void createNewModelLine(Composite container) {
		// Line Radio Button "new"
		GridData gdata = new GridData(GridData.BEGINNING);
		gdata.horizontalSpan = 3;
		
		newModelButton = new Button(container, SWT.RADIO);
		newModelButton.setText(WeaverPlugin.getPlugin().getString("_UI_wizard_button_new"));
		newModelButton.setLayoutData(gdata);
		newModelButton.addSelectionListener(new SelectionListener(){
			public void widgetSelected(SelectionEvent e) {
				newModelLineEnabled(true);
				existingModelLineEnabled(false);
				existingModelFileText.setText("");
			}

			public void widgetDefaultSelected(SelectionEvent e) {
				newModelLineEnabled(true);
				existingModelLineEnabled(false);
				existingModelFileText.setText("");
			}});
		
		// Line for enter the name
		newModelLabel1 = new Label(container, SWT.NULL);
		newModelLabel1.setText("&"+WeaverPlugin.getPlugin().getString("_UI_wizard_question_filename"));
		
		newModelFileText = new Text(container, SWT.BORDER | SWT.SINGLE);
		GridData gd = new GridData(GridData.FILL_HORIZONTAL);
		newModelFileText.setLayoutData(gd);
		newModelFileText.addModifyListener(new ModifyListener() {
			public void modifyText(ModifyEvent e) {
				dialogChanged();
			}
		});
		
		Label label = new Label(container, SWT.NULL);
		label.setText(" ");		
		
		// Init
		newModelButton.setSelection(true);
		newModelLineEnabled(true);
	}	
	
	/**
	 * Creates the existing model line (for choose a existing weaving model)
	 * @param container the container
	 */
	private void createExistingModelLine(Composite container) {
		// Line Radio Button "existing"
		GridData gdata = new GridData(GridData.BEGINNING);
		gdata.horizontalSpan = 3;
		existingModelButton = new Button(container, SWT.RADIO);
		existingModelButton.setText(WeaverPlugin.getPlugin().getString("_UI_wizard_button_existing"));
		existingModelButton.setLayoutData(gdata);
		existingModelButton.addSelectionListener(new SelectionListener(){
			public void widgetSelected(SelectionEvent e) {
				newModelLineEnabled(false);
				existingModelLineEnabled(true);
				newModelFileText.setText("");
			}

			public void widgetDefaultSelected(SelectionEvent e) {
				newModelLineEnabled(false);
				existingModelLineEnabled(true);
				newModelFileText.setText("");
			}});	
		
		// Line for enter the file name		
		existingModelLabel1 = new Label(container, SWT.NULL);
		existingModelLabel1.setText("&"+WeaverPlugin.getPlugin().getString("_UI_wizard_question_filename"));
		
		existingModelFileText = new Text(container, SWT.BORDER | SWT.SINGLE);
		GridData gd = new GridData(GridData.FILL_HORIZONTAL);
		existingModelFileText.setLayoutData(gd);
		existingModelFileText.addModifyListener(new ModifyListener() {
			public void modifyText(ModifyEvent e) {
				dialogChanged();
			}
		});
		
		browseModelButton = new Button(container, SWT.PUSH);
		browseModelButton.setText(WeaverPlugin.getPlugin().getString("_UI_wizard_button_browse"));
		browseModelButton.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				handleWorkspaceFileBrowse();
			}
		});	
		
		infoLabel = new Label(container,SWT.NULL);
		GridData gd2 = new GridData(GridData.BEGINNING);
		gd2.horizontalSpan = 3;
		infoLabel.setLayoutData(gd2);
		infoLabel.setText("if existing model is not in the container, it will be created in");
		
		// Init
		//buttonExisting.setSelection(false);
		existingModelLineEnabled(false);
	}
	
	/**
	 * Creates the extension panel line (for choose a weaving panel)
	 * @param container the container
	 */
	private void createExtensionEditorLine(Composite container) {
		Label label = new Label(container, SWT.NULL);
		label.setText("&"+WeaverPlugin.getPlugin().getString("_UI_wizard_panel"));
		
		extensionEditorCombo = new Combo(container, SWT.BORDER | SWT.READ_ONLY);
		extensionEditorCombo.addSelectionListener(new SelectionListener(){

			public void widgetSelected(SelectionEvent e) {
				Combo c = (Combo) e.getSource();
				editorExtensionSelected = c.getItem(c.getSelectionIndex());
			}
			public void widgetDefaultSelected(SelectionEvent e) {
			}
		});
	}	
	
	/**
	 * Creates the wmodel line (for choose a wmodel whath we use)
	 * @param container the container
	 */
	private void createWModelLine(Composite container) {
		Label label = new Label(container, SWT.NULL);
		label.setText("&"+WeaverPlugin.getPlugin().getString("_UI_wizard_wmodel"));
		
		wmodelCombo = new Combo(container, SWT.BORDER | SWT.READ_ONLY);
		wmodelCombo.addSelectionListener(new SelectionListener(){

			public void widgetSelected(SelectionEvent e) {
				Combo c = (Combo) e.getSource();
				wmodelSelected = c.getItem(c.getSelectionIndex());
				try {
					parentWizard.initializeWModelRef(wmodelSelected);
				} catch (WeaverException exc) {
					exc.printStackTrace();
				}
			}
			public void widgetDefaultSelected(SelectionEvent e) {
			}
		});
	}	
	
	/**
	 * Initializes all controls of this page with extensions found
	 */
	private void initControls(){
		// editor Extension
		extensionEditorCombo.removeAll();
		extensionEditorCombo.setEnabled(true);
		for(int i=0;i<parentWizard.getWeavingPanelExtensionNames().length;i++){
			extensionEditorCombo.add(parentWizard.getWeavingPanelExtensionNames()[i]);
		}
		if(extensionEditorCombo.getItemCount()>0){
			extensionEditorCombo.select(0);
			editorExtensionSelected = extensionEditorCombo.getItem(extensionEditorCombo.getSelectionIndex());
			if(extensionEditorCombo.getItemCount()==1)
				extensionEditorCombo.setEnabled(false);
		}	
	}	
	
	/**
	 * Initializes the wmodel control
	 * @throws WeaverException 
	 */
	protected void initWmodelControl(List wmodels) throws WeaverException{
		// wmodel
		wmodelCombo.removeAll();
		wmodelCombo.setEnabled(true);
		for(Iterator i = wmodels.iterator();i.hasNext();){
			wmodelCombo.add((String)i.next());
		}
		if(wmodelCombo.getItemCount()>0){
			wmodelCombo.select(0);
			wmodelSelected = wmodelCombo.getItem(wmodelCombo.getSelectionIndex());
			if(wmodelCombo.getItemCount()==1)
				wmodelCombo.setEnabled(false);
		}
		// initialize with the first wModel
		parentWizard.initializeWModelRef(wmodelSelected);
	}		
	
	/**
	 * sets enabled the new model line
	 * @param enabled
	 */
	private void newModelLineEnabled(boolean enabled){
		newModelLabel1.setEnabled(enabled);
		newModelFileText.setEnabled(enabled);
	}
	
	/**
	 * Sets enabled the existing model line
	 * @param enabled
	 */
	private void existingModelLineEnabled(boolean enabled){
		existingModelLabel1.setEnabled(enabled);
		existingModelFileText.setEnabled(enabled);
		browseModelButton.setEnabled(enabled);
		infoLabel.setEnabled(enabled);
	}
	
	/**
	 * Tests if the current workbench selection is a suitable
	 * container to use.
	 */
	private void initialize() {
		if (selection!=null && selection.isEmpty()==false && selection instanceof IStructuredSelection) {
			IStructuredSelection ssel = (IStructuredSelection)selection;
			if (ssel.size()>1) return;
			Object obj = ssel.getFirstElement();
			if (obj instanceof IResource) {
				IContainer container;
				if (obj instanceof IContainer)
					container = (IContainer)obj;
				else
					container = ((IResource)obj).getParent();
				projectText.setText(container.getFullPath().toString());
			}
		}
	}
	
	/**
	 * Opens the browse container file
	 */
	private void handleProjectBrowse() {
		ContainerSelectionDialog dialog =
			new ContainerSelectionDialog(
				getShell(),
				ResourcesPlugin.getWorkspace().getRoot(),
				false,
				WeaverPlugin.getPlugin().getString("_UI_wizard_select_project"));
		if (dialog.open() == ContainerSelectionDialog.OK) {
			Object[] result = dialog.getResult();
			if (result.length == 1) {
				projectText.setText(((Path)result[0]).toOSString());
			}
		}
	}		
	
	/**
	 * Opens the browse file - the file must be in the workspace
	 */	
	private void handleWorkspaceFileBrowse() {
        ResourceSelectionDialog resourceSelectionDialog = 
            new ResourceSelectionDialog            
            (getShell(), 
            ResourcesPlugin.getWorkspace().getRoot(),
            WeaverPlugin.getPlugin().getString("_UI_wizard_select_metamodel_ext"));
        //resourceSelectionDialog.setInitialSelections(resources.toArray());
        resourceSelectionDialog.open();
        if (resourceSelectionDialog.getReturnCode() == ResourceSelectionDialog.OK){
	        Object [] result = resourceSelectionDialog.getResult();
	        if(result != null){//always takes the first selected element (must be improved to allow only single selection	        	
	        	existingModelFileText.setText(((IFile)result[0]).getName());
        }
        }
	}	

	
	/**
	 * Ensures that both text fields are set.
	 */
	private void dialogChanged() {
		String projectName = getProjectName();
		String newModelFileName = getNewModelFileName();
		String existingModelFileName = getExistingModelFileName();
		boolean pageComplete = false;
		//String textName;
		
		// Management of status and finish button
		if (projectName.length()>0){

			if(newModelButton.getSelection()){
				if(newModelFileName.length()>0){
					setMessage(null);
					pageComplete = !setStatusModelFileExtension(newModelFileName,newWeavingModelString);
				}
				else{
					setStatusMustBeSpecified(newModelFileName,newWeavingModelString);
				}
			}
			else{
				if(existingModelFileName.length()>0){
					setMessage(null);
					pageComplete = !setStatusModelFileExtension(existingModelFileName,existingWeavingModelString);
				}
				else{
					setStatusMustBeSpecified(existingModelFileName,existingWeavingModelString);
				}
			}
		}
		else{
			setErrorMessage(null);
			setStatusMustBeSpecified(projectName,projectString);
			pageComplete = false;
		}
		
		setPageComplete(pageComplete);
	}
	
	/**
	 * Puts message in status bar
	 * @param inputText the contents of the input
	 * @param textName the name of the input
	 */
	private boolean setStatusMustBeSpecified(String inputText, String textName) {
		if (inputText.length() == 0) {
			setMessage(WeaverPlugin.getPlugin().getString("_UI_wizard_message",textName));
			return true;
		}
		else{
			setMessage(null);
			return false;
		}
	}	

	/**
	 * Puts message in status bar for model file extension error
	 * @param inputText the contents of the input
	 * @param textName the name of the input
	 */
	private boolean setStatusModelFileExtension(String inputText, String textName) {
		if(inputText.length()>0){
			int dotLoc = inputText.lastIndexOf('.');
			if ((dotLoc != -1)&&(dotLoc != 0)) {
				String ext = inputText.substring(dotLoc + 1);
				if ((ext.equalsIgnoreCase(extensionXMIString) == false) 
						&& (ext.equalsIgnoreCase(extensionEcoreString) == false)
						   && (ext.equalsIgnoreCase(extensionAMWString) == false)){
					setMessage(null);
					setErrorMessage(WeaverPlugin.getPlugin().getString(
							"_UI_wizard_error_message2", 
							new Object [] {textName,extensionAMWString}));
					return true;
				}
				else{
					setErrorMessage(null);
					return false;
				}
			}
			else {
				setMessage(null);
				setErrorMessage(WeaverPlugin.getPlugin().getString(
						"_UI_wizard_error_message2", 
						new Object [] {textName,extensionAMWString}));
				return true;
			}
		}
		else{
			setErrorMessage(null);
			return false;
		}
	}

	/**
	 * Returns the relative path of project
	 * @return weaving project
	 */
	public String getProjectName() {
		return projectText.getText();
	}
	/**
	 * Returns the relative path of new model file
	 * @return weaving model file
	 */
	public String getNewModelFileName() {
		return newModelFileText.getText();
	}
	/**
	 * Returns the full path of existing model file
	 * @return weaving model file
	 */
	public String getExistingModelFileName(){
		return existingModelFileText.getText();
	}
	/**
	 * Returns true if we work with a new model
	 * @return true or false
	 */
	public boolean isNewWeavingModel(){
		return newModelButton.getSelection();
	}	
	/**
	 * Returns the panel Extension selected id
	 * @return weaving panel id
	 */
	public String getEditorExtensionSelected() {
		return editorExtensionSelected;
	}
	/**
	 * Returns the wmodel extension selected id
	 * @return WModel id
	 */
	public String getWmodelSelected() {
		return wmodelSelected;
	}
	/**
	 * @return Returns the parentWizard.
	 */
	public WeaverWizard getParentWizard() {
		return parentWizard;
	}
	/**
	 * @param parentWizard The parentWizard to set.
	 */
	public void setParentWizard(WeaverWizard parentWizard) {
		this.parentWizard = parentWizard;
	}
}
