/*******************************************************************************
 * Copyright (c) 2005 INRIA.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Marcos Didonet Del Fabro - initial API and implementation
 *    Guillaume Gueltas
 *******************************************************************************/


package org.eclipse.gmt.weaver.wizards;

import java.util.List;
import java.util.Map;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.gmt.weaver.core.data.WovenModelDescriptor;
import org.eclipse.gmt.weaver.plugin.WeaverConstant;
import org.eclipse.gmt.weaver.plugin.WeaverPlugin;
import org.eclipse.gmt.weaver.plugin.util.WeaverResourceUtil;
import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.wizard.WizardPage;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.FileDialog;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.dialogs.ResourceSelectionDialog;

/**
 * Wizard dialog for the third page
 * @since 1.0
 */
public class WeaverThirdWizardDialog extends Dialog {
	
	private WeaverThirdWizardPage pageParent;

	private Text modelNameText;
	private String modelPanel;
	private Combo modelPanelCombo;
	private Combo wModelRefCombo;
	
	private String wModelRef; 
	private Text firstFileText;
	private Text secondFileText;
	// Works with metamodel (METAMODEL_TYPE) or with model-metamodel (MODEL_TYPE)
	private String resourceType = WeaverConstant.MODEL_TYPE; 

	private Button firstBrowseButton; 
	private Button secondBrowseButton;
	private Label firstFileLabel;
	private Label secondFileLabel; 	
	
	private Button button1;
	private Button button2;
	
	private List allModelPanel; 
	private String[] allWModelRef;
	
	private Map modelMap;

	private String projectName; 
	private WovenModelDescriptor initModelDescriptor = new WovenModelDescriptor();
	private int index=-1;

	private boolean fixedModel;
	
	/**
	 * Creates a new wizard dialog for add model/metamodel
	 * @param parentShell
	 * @param parent
	 * @param extensionsPanel the array of extensions woven panel
	 * @param wModelRefArray the array of wmodelref we find in metamodel extensions
	 */
	public WeaverThirdWizardDialog(Shell parentShell, WizardPage parent,
			String projectName, List extensionsPanel, String[] wModelRefArray, Map modelMap) {
		super(parentShell);
		this.pageParent = (WeaverThirdWizardPage) parent;
		this.allModelPanel = extensionsPanel;
		this.allWModelRef = wModelRefArray;
		this.projectName = projectName;
		this.modelMap = modelMap;
	}
	
	/**
	 * @param index
	 * @param name
	 * @param panel
	 * @param wModelRef
	 * @param model
	 * @param metamodel
	 * @param fixedModel false if is a model added with wizard
	 */
	public void initDialog(int index, WovenModelDescriptor modelDescriptor, boolean fixedModel){
		this.initModelDescriptor = modelDescriptor;
		this.index = index;
		this.fixedModel = fixedModel;
		
	}	
	
	/* (non-Javadoc)
	 * @see org.eclipse.jface.window.Window#configureShell(org.eclipse.swt.widgets.Shell)
	 */
	public void configureShell(Shell shell){
		super.configureShell(shell);
		shell.setText(WeaverPlugin.getPlugin().getString("_UI_wizard_dialog_metamodel"));
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.jface.dialogs.Dialog#createDialogArea(org.eclipse.swt.widgets.Composite)
	 */
	public Control createDialogArea(Composite parent){
		Composite container = new Composite(parent, SWT.NULL);
		{
			GridLayout layout = new GridLayout();
			layout.numColumns = 3;
			container.setLayout(layout);
		}	
		
		// Create controls
		createTheModelInput(container);
		initControls();

		return parent;
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.jface.dialogs.Dialog#okPressed()
	 */
	public void okPressed(){
		// Verify they are all not null		
		if (this.verifyData() == false){
			MessageDialog.openError(getShell(), 
					WeaverPlugin.getPlugin().getString("_UI_warning_label"),
							WeaverPlugin.getPlugin().getString("_UI_wizard_dialog_warning"));
			return;
		}
		
		// Send Array to wizard page
		WovenModelDescriptor desc = new WovenModelDescriptor();
		if(resourceType.equals(WeaverConstant.MODEL_TYPE)){
			desc.setMetamodelFile(secondFileText.getText());
			desc.setModelFile(firstFileText.getText());
		}
		else{
			desc.setMetamodelFile(firstFileText.getText());
			desc.setModelFile("");
		}
		
		desc.setPanel(modelPanel);
		desc.setWModelRef(wModelRef);
		desc.setNameProperty(modelNameText.getText());		
		if (initModelDescriptor.getReferenceName()!=null)
			if (!initModelDescriptor.getReferenceName().equals(""))
				desc.setReferenceName(initModelDescriptor.getReferenceName());
			else
				desc.setReferenceName((String) modelMap.get(wModelRef));
		else
			desc.setReferenceName((String) modelMap.get(wModelRef));
		
		// if add model or edit model
		if (index > -1){
			pageParent.setModel(index,desc);
		}else{
			pageParent.addOneModel();
			pageParent.addModel(desc);
		}
		pageParent.redraw();
		
		// Close dialog
		this.close();
	}

	/**
	 * Tests the controls
	 * @return
	 */
	private boolean verifyData() {
		boolean result = true;
		if(resourceType.equals(WeaverConstant.MODEL_TYPE))
			result = result && (secondFileText.getText() != "");
		result = result && (firstFileText.getText() != "");
		result = result && (modelPanel != "");
		result = result && (wModelRef != "");
		result = result && (modelNameText.getText() != "");
		return result;
	}

	/**
	 * Initializes the controls
	 */
	private void initControls(){
		// contents
		for(int i=0;i<allModelPanel.size();i++){
			modelPanelCombo.add((String)allModelPanel.get(i));
		}
		for(int i=0;i<allWModelRef.length;i++){
			wModelRefCombo.add(allWModelRef[i]);
		}
		// init combo
		modelPanelCombo.select(0);
		wModelRefCombo.select(0);
		for(int i =0;i<allWModelRef.length;i++){
			if(wModelRefCombo.getItem(i).equals(getInitModelDescriptor().getWModelRef()))
				wModelRefCombo.select(i);
		}
		for(int i =0;i<allModelPanel.size();i++){
			if(modelPanelCombo.getItem(i).equals(getInitModelDescriptor().getPanel()))
				modelPanelCombo.select(i);
		}
		// init text
		modelNameText.setText(getInitModelDescriptor().getNameProperty());
		if(getInitModelDescriptor().getModelFile().length() == 0){
			resourceType = WeaverConstant.METAMODEL_TYPE;
			button1.setSelection(true);
			secondFileLabel.setVisible(false);
			secondFileText.setVisible(false);
			secondBrowseButton.setVisible(false);
			firstFileLabel.setText("&"+WeaverPlugin.getPlugin().getString("_UI_wizard_filename_metamodel"));
			firstFileText.setText(getInitModelDescriptor().getMetamodelFile());
			secondFileText.setText("");
		}
		else{
			firstFileLabel.setText("&"+WeaverPlugin.getPlugin().getString("_UI_wizard_filename_model")+"           ");
			resourceType=WeaverConstant.MODEL_TYPE;
			button2.setSelection(true);
			firstFileText.setText(getInitModelDescriptor().getModelFile());
			secondFileText.setText(getInitModelDescriptor().getMetamodelFile());
		}
		
		// enabled combo 
		if(modelPanelCombo.getItemCount() == 1)
			modelPanelCombo.setEnabled(false);
		modelPanel = modelPanelCombo.getItem(modelPanelCombo.getSelectionIndex());
		
		if(wModelRefCombo.getItemCount() == 1)
			wModelRefCombo.setEnabled(false);
		wModelRef = wModelRefCombo.getItem(wModelRefCombo.getSelectionIndex());
		
		// enabled name
		if(fixedModel) modelNameText.setEnabled(false);
	}		

	
	/**
	 * creates the controls for this model/metamdodel
	 * @param container
	 */
	private void createTheModelInput(Composite container) {
		// First line : name
		Composite composite = new Composite(container,SWT.BEGINNING);
		{
			GridData gdata = new GridData(GridData.FILL_HORIZONTAL);
			gdata.horizontalSpan=3;
			composite.setLayoutData(gdata);
		}
		{
			GridLayout layout = new GridLayout();
			composite.setLayout(layout);
			layout.numColumns = 4;
			layout.marginWidth = 4;
		}
		
		Label label = new Label(composite, SWT.BEGINNING);
		label.setText("&"+WeaverPlugin.getPlugin().getString("_UI_wizard_question_name"));
		
		modelNameText = new Text(composite, SWT.BORDER | SWT.SINGLE);
		{
			GridData gd = new GridData(GridData.GRAB_HORIZONTAL);
			gd.widthHint = 170;
			modelNameText.setLayoutData(gd);
		}
		
		modelNameText.addModifyListener(new ModifyListener() {
			public void modifyText(ModifyEvent e) {
				//dialogChanged();
			}
		});
		
		label = new Label(composite, SWT.BEGINNING);
		label = new Label(composite, SWT.BEGINNING);
		
		// third line 
		label = new Label(composite, SWT.BEGINNING);
		label.setText("&"+WeaverPlugin.getPlugin().getString("_UI_wizard_wmodel_ref"));
		
		wModelRefCombo = new Combo(composite, SWT.BORDER | SWT.READ_ONLY);
		{
			GridData gd = new GridData(GridData.GRAB_HORIZONTAL);
			gd.widthHint = 170;
			wModelRefCombo.setLayoutData(gd);
		}
		wModelRefCombo.addSelectionListener(new SelectionListener(){

			public void widgetSelected(SelectionEvent e) {
				Combo c = (Combo) e.getSource();
				wModelRef = c.getItem(c.getSelectionIndex());
			}
			public void widgetDefaultSelected(SelectionEvent e) {
			}
		});	
		

		
		label = new Label(composite, SWT.BEGINNING);
		label.setText("&"+WeaverPlugin.getPlugin().getString("_UI_wizard_panel"));
		
		modelPanelCombo = new Combo(composite, SWT.BORDER | SWT.READ_ONLY);
		{
			GridData gd = new GridData(GridData.GRAB_HORIZONTAL);
			gd.widthHint = 170;
			modelPanelCombo.setLayoutData(gd);
		}
		modelPanelCombo.addSelectionListener(new SelectionListener(){

			public void widgetSelected(SelectionEvent e) {
				Combo c = (Combo) e.getSource();
				modelPanel = c.getItem(c.getSelectionIndex());
			}
			public void widgetDefaultSelected(SelectionEvent e) {
			}
		});
		
		// third line 
		
		label = new Label(composite,SWT.BEGINNING);
		label.setText(WeaverPlugin.getPlugin().getString("_UI_wizard_question_what_weave"));
		
		Composite radios = new Composite(composite,SWT.BEGINNING);
		{
			GridData gdata = new GridData(GridData.GRAB_HORIZONTAL);
			gdata.widthHint = 170;
			radios.setLayoutData(gdata);
			
			GridLayout layout = new GridLayout();
			radios.setLayout(layout);
			layout.numColumns = 2;
			layout.marginWidth = 0;
		}
		button1 = new Button(radios, SWT.RADIO);
		button1.setText(WeaverPlugin.getPlugin().getString("_UI_wizard_button_metamodels"));
		button1.addSelectionListener(new SelectionListener(){
			public void widgetSelected(SelectionEvent e) {
				// Work with metamodel or with model-metamodel
				resourceType = WeaverConstant.METAMODEL_TYPE;
				secondFileLabel.setVisible(false);
				secondFileText.setVisible(false);
				secondBrowseButton.setVisible(false);
				firstFileLabel.setText("&"+WeaverPlugin.getPlugin().getString("_UI_wizard_filename_metamodel"));
				firstFileText.setText("");
				secondFileText.setText("");
				//dialogChanged();
			}

			public void widgetDefaultSelected(SelectionEvent e) {
			}});
		
		button2 = new Button(radios, SWT.RADIO);
		button2.setText(WeaverPlugin.getPlugin().getString("_UI_wizard_button_models"));
		button2.addSelectionListener(new SelectionListener(){
			public void widgetSelected(SelectionEvent e) {
				resourceType=WeaverConstant.MODEL_TYPE;
				firstFileLabel.setText("&"+WeaverPlugin.getPlugin().getString("_UI_wizard_filename_model"));
				secondFileLabel.setVisible(true);
				secondFileText.setVisible(true);
				secondBrowseButton.setVisible(true);
				firstFileText.setText("");
				secondFileText.setText("");
				//dialogChanged();
			}

			public void widgetDefaultSelected(SelectionEvent e) {
			}});
		label = new Label(composite, SWT.BEGINNING);
		label = new Label(composite, SWT.BEGINNING);
		
		// Third Line : metamodel
		firstFileLabel = new Label(container, SWT.BEGINNING);
		firstFileLabel.setText("&"+WeaverPlugin.getPlugin().getString("_UI_wizard_filename_metamodel"));
		
		firstFileText = new Text(container, SWT.BORDER | SWT.SINGLE);
		GridData gd = new GridData(GridData.FILL_HORIZONTAL);
		gd.widthHint = 305;
		firstFileText.setLayoutData(gd);
		
		firstFileText.addModifyListener(new ModifyListener() {
			public void modifyText(ModifyEvent e) {
				//dialogChanged();
			}
		});

		firstBrowseButton = new Button(container, SWT.PUSH);
		firstBrowseButton.setText(WeaverPlugin.getPlugin().getString("_UI_wizard_button_browse"));
		firstBrowseButton.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				handleWorkspaceFileBrowse(firstFileText);
			}
		});	
		
		// Fourth line : model
		secondFileLabel = new Label(container, SWT.BEGINNING);
		secondFileLabel.setText("&"+WeaverPlugin.getPlugin().getString("_UI_wizard_filename_model_meta"));
		
		secondFileText = new Text(container, SWT.BORDER | SWT.SINGLE);
		gd = new GridData(GridData.FILL_HORIZONTAL);
		secondFileText.setLayoutData(gd);
		secondFileText.addModifyListener(new ModifyListener() {
			public void modifyText(ModifyEvent e) {
				//dialogChanged();
			}
		});
		
		secondBrowseButton = new Button(container, SWT.PUSH);
		secondBrowseButton.setText(WeaverPlugin.getPlugin().getString("_UI_wizard_button_browse"));
		secondBrowseButton.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				handleWorkspaceFileBrowse(secondFileText);
			}
		});	
	}	
	
	
	/**
	 * Opens the browse file - the file must be in the workspace
	 */	
	private void handleWorkspaceFileBrowse(Text aText) {
        ResourceSelectionDialog resourceSelectionDialog = 
            new ResourceSelectionDialog            
            (getShell(), 
            ResourcesPlugin.getWorkspace().getRoot(),
            WeaverPlugin.getPlugin().getString("_UI_wizard_button_browse"));
        //resourceSelectionDialog.setInitialSelections(resources.toArray());
        resourceSelectionDialog.open();
        if (resourceSelectionDialog.getReturnCode() == ResourceSelectionDialog.OK){
	        Object [] result = resourceSelectionDialog.getResult();
	        if(result != null){//always takes the first selected element (must be improved to allow only single selection	        	
	        	aText.setText(((IFile)result[0]).getFullPath().toString());
        }
        }
	}	
	
	/**
	 * Opens the browse first file
	 */
	private void handleFirstFileBrowse() {
		String nomFichier;
		FileDialog dialog = new FileDialog(this.getShell(), SWT.OPEN);
		if(resourceType.equals(WeaverConstant.METAMODEL_TYPE)){
			dialog.setFilterExtensions(new String[] {
					"*."+(pageParent).extensionEcoreString,
					"*."+(pageParent).extensionKM3String,
					"*."+(pageParent).extensionXMIString});
		}/*
		else{
			dialog.setFilterExtensions(new String[] {
					"*."+(pageParent).extensionEcoreString,
					"*."+(pageParent).extensionXMIString});
		}*/
		//TODO verify why we previously used toOSSTring instead of toString
		dialog.setFilterPath(WeaverResourceUtil.getFullPath(projectName));
		dialog.setText(WeaverPlugin.getPlugin().getString("_UI_wizard_button_browse"));
		nomFichier = dialog.open();	
		if ((nomFichier != null) && (nomFichier.length() != 0)){
			firstFileText.setText(nomFichier);
		}
	}
	
	/**
	 * Opens the browse second file
	 */
	private void handleSecondFileBrowse() {
		String fileName;
		FileDialog dialog = new FileDialog(this.getShell(), SWT.OPEN);
		dialog.setFilterExtensions(new String[] {
				"*."+(pageParent).extensionEcoreString,
				"*."+(pageParent).extensionKM3String,
				"*."+(pageParent).extensionXMIString});
		dialog.setFilterPath(ResourcesPlugin.getWorkspace().getRoot().getLocation().toOSString()
				+projectName);
		dialog.setText(WeaverPlugin.getPlugin().getString("_UI_wizard_button_browse"));
		fileName = dialog.open();	
		if ((fileName != null) && (fileName.length() != 0)){
			secondFileText.setText(fileName);
		}
	}
	/**
	 * @return Returns the allModelPanel.
	 */
	public List getAllModelPanel() {
		return allModelPanel;
	}
	/**
	 * @param allModelPanel The allModelPanel to set.
	 */
	public void setAllModelPanel(List allModelPanel) {
		this.allModelPanel = allModelPanel;
	}
	/**
	 * @return Returns the allWModelRef.
	 */
	public String[] getAllWModelRef() {
		return allWModelRef;
	}
	/**
	 * @param allWModelRef The allWModelRef to set.
	 */
	public void setAllWModelRef(String[] allWModelRef) {
		this.allWModelRef = allWModelRef;
	}
	/**
	 * @return Returns the index.
	 */
	public int getIndex() {
		return index;
	}
	/**
	 * @param index The index to set.
	 */
	public void setIndex(int index) {
		this.index = index;
	}
	/**
	 * @return Returns the pageParent.
	 */
	public WeaverThirdWizardPage getPageParent() {
		return pageParent;
	}
	/**
	 * @param pageParent The pageParent to set.
	 */
	public void setPageParent(WeaverThirdWizardPage pageParent) {
		this.pageParent = pageParent;
	}
	/**
	 * @return Returns the projectName.
	 */
	public String getProjectName() {
		return projectName;
	}
	/**
	 * @param projectName The projectName to set.
	 */
	public void setProjectName(String projectName) {
		this.projectName = projectName;
	}
	/**
	 * @return Returns the resourceType.
	 */
	public String getResourceType() {
		return resourceType;
	}
	/**
	 * @param resourceType The resourceType to set.
	 */
	public void setResourceType(String resourceType) {
		this.resourceType = resourceType;
	}
	/**
	 * @return Returns the wModelRef.
	 */
	public String getWModelRef() {
		return wModelRef;
	}
	/**
	 * @param modelRef The wModelRef to set.
	 */
	public void setWModelRef(String modelRef) {
		wModelRef = modelRef;
	}
	/**
	 * @return Returns the initModelDescriptor.
	 */
	public WovenModelDescriptor getInitModelDescriptor() {
		return initModelDescriptor;
	}
	/**
	 * @param initModelDescriptor The initModelDescriptor to set.
	 */
	public void setInitModelDescriptor(WovenModelDescriptor initModelDescriptor) {
		this.initModelDescriptor = initModelDescriptor;
	}
}
