/*******************************************************************************
 * Copyright (c) 2005 INRIA.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Marcos Didonet Del Fabro - initial API and implementation
 *    Guillaume Gueltas
 *******************************************************************************/


package org.eclipse.gmt.weaver.wizards;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;

import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EReference;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.gmt.weaver.core.data.WovenModelDescriptor;
import org.eclipse.gmt.weaver.plugin.WeaverPlugin;
import org.eclipse.gmt.weaver.plugin.util.WeaverUtil;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.wizard.WizardPage;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.swt.widgets.TableItem;

/**
 * Third page of the wizard (woven data)
 * @since 1.0
 */
public class WeaverThirdWizardPage extends WizardPage {
	protected ISelection selection;
	protected WeaverWizard parentWizard;
	private List AllWModelRef;	 //private String[] wModelRefArray = new String[0];
	private int numberOfFixedModel = 0 ; // it's the number of derived value of wovenModel which is not many
	private int numberOfModel = 0;
	// Text Label	
	public String extensionXMIString = WeaverPlugin.getPlugin().getString("_UI_extension_1");	
	public String extensionKM3String = WeaverPlugin.getPlugin().getString("_UI_extension_2");
	public String extensionEcoreString = WeaverPlugin.getPlugin().getString("_UI_extension_3");

	// Models / MetaModels of the "ADD" dialogs
	private List models = new ArrayList(); /// list of WovenModelDescriptor
	private HashMap modelRef = new HashMap();
	
	private Button addModel = null;
	private Button editModel = null;
	private Button removeModel = null;
	private Table table;
	Font font=null;

	/**
	 * Creates a new third wizard page
	 * @param parent the wizard parent of this page
	 * @param pageName the name of the page
	 * @param selection the current object selection
	 */
	public WeaverThirdWizardPage(WeaverWizard parent,String pageName, ISelection selection) {
		super(pageName);
		this.selection = selection;
		this.parentWizard = parent;
		setPageComplete(false);
	}	
	
	/**
	 * Creates a new third wizard page
	 * @param parent the wizard parent of this page
	 * @param pageName the name of the page
	 * @param title the title for this wizard page,
	 *   or <code>null</code> if none
	 * @param titleImage the image descriptor for the title of this wizard page,
	 *   or <code>null</code> if none
	 */
	public WeaverThirdWizardPage(WeaverWizard parent,String pageName, String title,
			ImageDescriptor titleImage) {
		super(pageName, title, titleImage);
		this.parentWizard = parent;
		setPageComplete(false);
	}

	/* (non-Javadoc)
	 * @see org.eclipse.jface.dialogs.IDialogPage#createControl(org.eclipse.swt.widgets.Composite)
	 */
	public void createControl(Composite parent) {
		font = new Font(this.getShell().getDisplay()
				, "Arial", 8, SWT.BOLD);
		
		Composite container = new Composite(parent, SWT.NULL);
		{
			GridLayout layout = new GridLayout();
			layout.numColumns = 3;
			container.setLayout(layout);
		}	
		
		// Table
		Label title = new Label(container,SWT.BEGINNING);
		title.setText("Enter model and metamodel information: ");
		{
			GridData gd = new GridData(GridData.BEGINNING);
			gd.horizontalSpan = 3;
			title.setLayoutData(gd);
		}
		Label end = new Label(container,SWT.BEGINNING);
		end.setText("(Extensions of weaving metamodel define name and number of weaving model's references)");
		{
			GridData gd = new GridData(GridData.BEGINNING);
			gd.horizontalSpan = 3;
			end.setLayoutData(gd);
		}
		
		table = new Table(container, SWT.BORDER | SWT.H_SCROLL | SWT.V_SCROLL | SWT.MULTI| SWT.FULL_SELECTION );
		{
			GridData gdt = new GridData(GridData.HORIZONTAL_ALIGN_FILL);
			gdt.horizontalSpan = 3;
			gdt.grabExcessHorizontalSpace=true;
			gdt.grabExcessVerticalSpace=true;
			gdt.verticalAlignment = GridData.FILL;
	        gdt.horizontalAlignment = GridData.FILL;
			gdt.heightHint=220;
			gdt.widthHint=330;
			table.setLayoutData(gdt);
		}
		
		TableColumn column2 = new TableColumn(table, SWT.LEFT);
		column2.setWidth(70);	
		column2.setText(WeaverPlugin.getPlugin().getString("_UI_wizard_table_column1"));
		column2.setResizable(true);
		TableColumn column3 = new TableColumn(table, SWT.LEFT);
		column3.setWidth(380);	
		column3.setText(WeaverPlugin.getPlugin().getString("_UI_wizard_table_column2"));
		column3.setResizable(true);
		
		table.setLinesVisible(true); 	
		table.setHeaderVisible(true); 
		table.addSelectionListener(new SelectionListener(){

			public void widgetSelected(SelectionEvent e) {
				Table tab = (Table) e.getSource();
				TableItem item = (TableItem) e.item;
				int index = tab.indexOf(item);
				if(((index + 1) % 6) == 0) table.deselect(index);
				int first = index / 6;
				int begin = first * 6;
				tab.select(begin,begin+4);
			}

			public void widgetDefaultSelected(SelectionEvent e) {
			}
		});

		// Buttons
		editModel = new Button(container,SWT.PUSH);
		addModel = new Button(container,SWT.PUSH);
		removeModel = new Button(container,SWT.PUSH);
		addModel.setText(WeaverPlugin.getPlugin().getString("_UI_wizard_button_add"));
		removeModel.setText(WeaverPlugin.getPlugin().getString("_UI_wizard_button_remove"));
		editModel.setText(WeaverPlugin.getPlugin().getString("_UI_wizard_button_edit"));
		
		addModel.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				
				WeaverThirdWizardDialog dialog = new WeaverThirdWizardDialog(getShell(),
						parentWizard.pageThree,
						parentWizard.pageTwo.getProjectName(),
						parentWizard.getWovenPanelExtensionNames(),
						(String[]) AllWModelRef.get(numberOfFixedModel),
						modelRef
						);
				try{
					WovenModelDescriptor desc = new WovenModelDescriptor();					
					//TODO verify desc.setReferenceName(getFixedReferenceName());
					desc.setReferenceName(null);
					dialog.initDialog(-1,desc,false);
				}catch(Exception ex){}
				dialog.open();
			}
		}); 

		editModel.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				int[] indice = table.getSelectionIndices();
				if((indice != null)&&(indice.length > 0)){
					int i = indice[0] / 6;
					WeaverThirdWizardDialog dialog = new WeaverThirdWizardDialog(getShell(),
							parentWizard.pageThree,
							parentWizard.pageTwo.getProjectName(),
							parentWizard.getWovenPanelExtensionNames(),
							((i>numberOfFixedModel)?((String[]) AllWModelRef.get(numberOfFixedModel)):((String[]) AllWModelRef.get(i))),
							modelRef
							);
					try{
						//((WovenModelDescriptor) models.get(i)).setReferenceName(name);
						dialog.initDialog(i,(WovenModelDescriptor) models.get(i),(i<numberOfFixedModel));
					}catch(Exception ex){}
					dialog.open();
				}
			}
		});
		
		removeModel.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				int[] indice = table.getSelectionIndices();
				if((indice != null)&&(indice.length > 0)){
					int index = indice[0] / 6;
					if(index<numberOfFixedModel){
						//do nothing
					}
					else{
						try{
							models.remove(index);
							removeOneModel();
						}catch(Exception ex){}
					}
				}
				redraw();
			}
		});
		
		setPageComplete(false);
		setControl(container);
	}
	
	/**
	 * initializes the table for the derived values (or relation WModel->WModelRef) found 
	 * @param name the name of association
	 */
	private void addNameInTable(String title, String referenceName) {
		WovenModelDescriptor desc = new WovenModelDescriptor();
		desc.setNameProperty(title);
		desc.setReferenceName(referenceName);
		models.add(desc);
	}
	
	/**
	 * Redraws the table
	 */
	public void redraw(){
		table.removeAll();
		for(int i =0;i<(numberOfFixedModel+numberOfModel);i++){
			TableItem[] item = new TableItem[6];
			item[0] = new TableItem(table,SWT.NONE);
			item[0].setText(0,WeaverPlugin.getPlugin().getString("_UI_wizard_table_line1"));
			item[0].setFont(0,font);
			item[0].setText(1,((WovenModelDescriptor)models.get(i)).getNameProperty());
			item[1] = new TableItem(table,SWT.NONE);
			item[1].setText(0,WeaverPlugin.getPlugin().getString("_UI_wizard_table_line2"));
			item[1].setFont(0,font);
			item[1].setText(1,((WovenModelDescriptor)models.get(i)).getPanel());
			item[2] = new TableItem(table,SWT.NONE);
			item[2].setText(0,WeaverPlugin.getPlugin().getString("_UI_wizard_table_line3"));
			item[2].setFont(0,font);
			item[2].setText(1,((WovenModelDescriptor)models.get(i)).getWModelRef());
			item[3] = new TableItem(table,SWT.NONE);
			item[3].setText(0,WeaverPlugin.getPlugin().getString("_UI_wizard_table_line4"));
			item[3].setFont(0,font);
			item[3].setText(1,((WovenModelDescriptor)models.get(i)).getMetamodelFile());
			item[4] = new TableItem(table,SWT.NONE);
			item[4].setText(0,WeaverPlugin.getPlugin().getString("_UI_wizard_table_line5"));
			item[4].setFont(0,font);
			item[4].setText(1,((WovenModelDescriptor)models.get(i)).getModelFile());
			item[5] = new TableItem(table,SWT.NONE);
			item[5].setText(0,"");
			item[5].setText(1,"");	
		}
		dialogChanged();
	}	
	
	/**
	 * Initializes controls 
	 * @param references the list of eReference (derived of wovenModel) found
	 * @param weavingMetamodel the weaving metamodel
	 */
	protected void initControls(List references, Resource weavingMetamodel){
		// init
		numberOfFixedModel = 0 ; 
		numberOfModel = 0;
		models = new ArrayList();
		removeModel.setEnabled(false);
		addModel.setEnabled(false);
		
		AllWModelRef = new ArrayList();
		for(Iterator iter=references.iterator();iter.hasNext();){
			EReference ref = (EReference) iter.next();
			// init the combo with wModelRef associated
			EClass wmodelref = ref.getEReferenceType();
			List subtype = WeaverUtil.getSubTypeListOf(wmodelref.getName(),weavingMetamodel);
			int size = subtype.size();
			String[] wModelRefArray = new String[size];
			if(size == 0){
				wModelRefArray = new String[1];
				wModelRefArray[0]= wmodelref.getName();
			}
			else{
				for(int i = 0; i<size;i++){
					wModelRefArray[i] = (String) subtype.get(i);
				}
			}
			AllWModelRef.add(wModelRefArray);
			
			// init table with title and reference name
			if(ref.isMany()){
				removeModel.setEnabled(true);
				addModel.setEnabled(true);
				//this.setFixedReferenceName(ref.getName());
			}
			else{
				this.addNameInTable(ref.getName(),ref.getName());
				numberOfFixedModel++;
			}	
			modelRef.put(wmodelref.getName(), ref.getName());
		}
		this.redraw();
	
	}		

	/**
	 * Ensures that both text fields are set.
	 */
	private void dialogChanged() {
		setPageComplete(false);
		for(int i =0;i<models.size();i++){
			if((((WovenModelDescriptor)models.get(i)).getPanel()).length() == 0){
				return;
			}
		}
		setPageComplete(true);
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.jface.dialogs.IDialogPage#dispose()
	 */
	public void dispose(){
		super.dispose();
		font.dispose();
	}
	

	/**
	 * @param element
	 */
	protected void addModel(WovenModelDescriptor element){
		models.add(element);
	}
	/**
	 * @param index
	 * @param element
	 */
	protected void setModel(int index, WovenModelDescriptor element){
		models.set(index, element);
	}
	/**
	 * @return Returns the models.
	 */
	public List getModels() {
		return models;
	}
	
	

	/**
	 * @return
	 */
	public int getNumberOfModel() {
		return numberOfModel;
	}
	/**
	 * make +1 to number of model
	 */
	public void addOneModel() {
		this.numberOfModel++;
	}
	/**
	 * makes -1 to number of model
	 */
	public void removeOneModel() {
		this.numberOfModel--;
	}
	/**
	 * @return Returns the parentWizard.
	 */
	public WeaverWizard getParentWizard() {
		return parentWizard;
	}
	/**
	 * @param parentWizard The parentWizard to set.
	 */
	public void setParentWizard(WeaverWizard parentWizard) {
		this.parentWizard = parentWizard;
	}
	/**
	 * @return Returns the numberOfFixedModel.
	 */
	public int getNumberOfFixedModel() {
		return numberOfFixedModel;
	}
	/**
	 * @param numberOfFixedModel The numberOfFixedModel to set.
	 */
	public void setNumberOfFixedModel(int numberOfFixedModel) {
		this.numberOfFixedModel = numberOfFixedModel;
	}
	/**
	 * @return Returns the fixedReferenceName.
	 */
	/**
	 * @param fixedReferenceName The fixedReferenceName to set.
	 */
	public String getRefName(String modelName) {
		return (String) modelRef.get(modelName);
	}
}
